/* MTP3 management test application
 *
 * Sample MTP3/MTP2 management application; sends commands to MTP task on Tx series
 * processor and waits for response (status and optional data, such as entity
 * status or statistics).
 *
 * To run:
 *
 *  mtp3mgr  [-b <board>]
 *
 *      <board>     Optional Board number to open (defaults to 1)
 *
 * User Commands:
 *
 *      LINK    <link number>  ENA | DIS | INH | UNI | DEL | RST | LPO | LPR | EMG | NRM | TRE | TRD
 *      LINKSET <linkset number>  ENA | DIS
 *      ALARMS  ON | OFF
 * 		FLOW	ON | OFF
 *      TRACE   ON | OFF
 *      STATUS  LINK    <link number> | *
 *  	STATUS	LINKSET	<linkset number>
 *      STATUS  ROUTE   <point code>    ! point code is x.y.z or a number
 *  	STATUS	NSAP	<nsap number>
 *  	STATUS	MTP3
 *      STATS   LINK    <link number> [ RESET ]
 *      STATS   LINKSET <linkset number> [ RESET ]
 *      STATS   ROUTE   <point code> [ RESET ]   ! point code is x.y.z or a number
 *      STATS   MTP3     [ RESET ]
 *      ?       [<command name>]        ! help all commands or specific command
 *      BOARD   <board number>          ! attach to new board
 *      Q                               ! quit
 */


#include <stdio.h>
#include <ctype.h>
#ifdef UNIX
#define getch	getchar
#else
#include <conio.h>          /* For getch() call */
#endif
#include <string.h>
#include <stdlib.h>
#include "ss7type.h"
#include "mtp3mgmt.h"
#include "mtp2mapi.h"
#include "mtperr.h"
#include "cfgscan.h"                    /* for point code display and scanning utilities */
#include "osdep.h"

#ifndef MTP2_ONLY
#include "mtp3mapi.h"
#include "mtp3ver.h"
void mtp3Ctrl( U32 *parms );
#else
#include "mtp2ver.h"
#endif

void linkCtrl( U32 *parms );
void linksetCtrl( U32 *parms );
void mtp2Ctrl( U32 *parms );
void statusReq( U32 *parms );
void statsReq( U32 *parms );
void HelpCommand( U32 *parms );
void setCP( U32 *parms );
void quit( U32 *parms );
void routeTest( U32 *parms );

char *hlStates[] = {            /* Higher level states.  Used for MTP2 status display */
    "UNBOUND",
    "CONFIG'D",
    "DISABLED",
    "ENABLED"
};

char *llStates[] = {            /* Lower level states. Used for MTP2 status display */
    "UNUSED",
    "OUT_OF_SERVICE",
    "INITIAL_ALIGNMENT",
    "ALIGNED_READY",
    "ALIGNED_NOT_READY",
    "PROC_OUT",
    "IN_SERVICE",
    "IDLE",
    "NOT_ALIGNED",
    "IS_ALIGNED",
    "PROVING",
    "LOCAL_PROC_OUT",
    "REMOTE_PROC_OUT",
    "BOTH_PROC_OUT",
    "MONITORING",
    "CONGESTION",
    "POWER_OFF"
};

/* lookup tables for translating names into values and vice versa */
struct lookupEnt {
    char            *name;
    U32   value;
};

struct lookupEnt LinkStatesTbl[] = {	/* link state names */
	{ "INACTIVE", 0 },
	{ "CONNECTING", 1 },
	{ "ACTIVE", 2 },
	{ "FAILED", 3 },
	{ "WAITING CONN", 4 },
    { "BINDING", 5 },
    { "UNBINDING", 6 },
    { "UNBOUND", 7},
	{ 0,  0 }/* end of table */
};

struct lookupEnt NSapStatesTbl[] = {	/* NSap state names */
	{ "UNBOUND", 1 },
	{ "BOUND", 2 },
	{ 0,  0 }/* end of table */
};


#ifndef MTP2_ONLY
/* parameter name tables */
struct lookupEnt LinkCtrlLookupTbl[] = {	/* link control action values */
	{ "ENA", MTP3_CTRL_LINK_ENA },
	{ "DIS", MTP3_CTRL_LINK_DIS },
	{ "INH", MTP3_CTRL_LINK_INH },
	{ "UNI", MTP3_CTRL_LINK_UNINH },
    { "DEL", MTP3_CTRL_LINK_DEL },
	{ "RST", MTP3_CTRL_LINK_SPRST },
    { "LPO", MTP3_CTRL_LINK_LPO },          /* Local processor outage */
    { "LPR", MTP3_CTRL_LINK_LPR },          /* Local processor recovery */
    { "EMG", MTP3_CTRL_LINK_EMG },          /* Emergency alignment condition */
    { "NRM", MTP3_CTRL_LINK_NRM },          /* Normal alignment condition */
    { "TRE", MTP3_CTRL_LINK_TRCON },        /* Start Link trace data */
    { "TRD", MTP3_CTRL_LINK_TRCOFF },       /* Link Trace Data off */
	{ 0,  0 }/* end of table */
};

struct lookupEnt LinksetCtrlLookupTbl[] = { 	/* linkset control action values */
	{ "ENA", MTP3_CTRL_LINKSET_ENA },
	{ "DIS", MTP3_CTRL_LINKSET_DIS },
	{ 0,  0 }/* end of table */
};

struct lookupEnt AlarmCtrlLookupTbl[] = { 	/* MTP3 alarms control action values  */
	{ "ON", MTP3_CTRL_ALARM_ENA },
	{ "OFF", MTP3_CTRL_ALARM_DIS },
	{ 0,	0 }/* end of table */
};

struct lookupEnt FlowCtrlLookupTbl[] = { 	/* MTP3 flow control action values  */
	{ "ON", MTP3_CTRL_FLOWCTL_ON },
	{ "OFF", MTP3_CTRL_FLOWCTL_OFF },
	{ 0,	0 }/* end of table */
};

struct lookupEnt TraceDataLookupTbl[] = {    /* MTP3 Trace Data action values  */
    { "ON", MTP3_CTRL_TRACE_ON },
    { "OFF", MTP3_CTRL_TRACE_OFF },
	{ 0,	0 }/* end of table */
};

#endif

#define ALL_LINKS   69

struct lookupEnt StatusLookupTbl[] = {  /* status command values */
	{ "LINK", MTP3_DLSAP_STATUS },
	{ "LINKSET", MTP3_LNKSET_STATUS },
	{ "NSAP", MTP3_NSAP_STATUS },
	{ "MTP3", MTP3_GEN_STATUS },
	{ "ROUTE", MTP3_ROUTE_STATUS },
	{ 0,	0 }/* end of table */
};

struct lookupEnt StatsLookupTbl[] = { 	/* statistics command values */
	{ "LINK", MTP3_DLSAP_STATS },
	{ "LINKSET", MTP3_LNKSET_STATS },
	{ "MTP3", MTP3_GEN_STATS },
	{ "ROUTE", MTP3_ROUTE_STATS },
	{ 0,	0 }/* end of table */
};


struct lookupEnt Mtp2FlowCtrlLookupTbl[] = {    /* MTP2 flow control action values  */
    { "ON", MTP2_CTRL_FLOWCTL_ON },
    { "OFF", MTP2_CTRL_FLOWCTL_OFF },
	{ 0,	0 }/* end of table */
};

struct lookupEnt ResetLookupTbl[] = {     /* stats reset flag values */
    { "RESET", 255 },
	{ 0,  0 }/* end of table */
};

struct lookupEnt CmdLookupTbl[] =
{
#ifndef MTP2_ONLY
      { "LINK",       1 },
      { "LINKSET",    2 },
      { "SRT",        3 },
      { "MTP3FLOW",   4 },
      { "TRACE",      5 },
      { "MTP2FLOW",   6 },
      { "STATUS",     7 },
      { "STATS",      8 },
      { "BOARD",      9 },
      { "Q",          10},
      { "ALL",        0 },
#else
      { "MTP2FLOW",   1 },
      { "STATUS",     2 },
      { "STATS",      3 },
      { "BOARD",      4 },
      { "Q",          5 },
      { "ALL",        0 },
#endif
	    { 0,  0 }}; /* end of table */

static char linkCtrlHelp[]        =   "    Syntax: LINK <link number> <action>\n\n"
"     where:\n"
"            <action>      [ENA | DIS | INH | UNI | DEL | RST | LPO | LPR | EMG | NRM | TRE | TRD]\n\n";

static char linksetCtrlHelp[]        =   "    Syntax: LINKSET <linkset number> <action>\n\n"
"     where:\n"
"            <action>      [ENA | DIS]\n\n";

static char alarmsCtrlHelp[]        =   "    Syntax: ALARMS [ON | OFF]\n\n";

static char mtp3flowCtrlHelp[]        =   "    Syntax: MTP3FLOW [ON | OFF]\n\n";
static char mtp2flowCtrlHelp[]        =   "    Syntax: MTP2FLOW [ON | OFF]\n\n";
static char mtp3traceDataHelp[]       =   "    Syntax: TRACE [ON | OFF]\n\n";

static char statusHelp[]        =   "    Syntax: STATUS <entity> [<number> | *]\n\n"
"     where:\n"
#ifndef MTP2_ONLY
"            <entity>      [LINK | LINKSET | NSAP | ROUTE | MTP3 ]\n"
"            <number>      not used for STATUS MTP3\n\n"
"                          wildcard '*' can only be used with STATUS LINK *\n";

#else
"            <entity>      [LINK]\n"
"            <number>      link number to obtain status on\n\n";
#endif

static char statsHelp[]        =   "    Syntax: STATS <entity> [<number>]\n\n"
"     where:\n"
#ifndef MTP2_ONLY
"            <entity>      [LINK | LINKSET | ROUTE | MTP3 ]\n"
"            <number>      not used for MTP3\n\n";
#else
"            <entity>      [LINK]\n"
"            <number>      link number to obtain stats from\n\n";
#endif

static char quitHelp[]              =   "    Syntax: Q \n\n"
"    Quit mtpmgr application\n\n\n";

static char boardHelp[]        =   "    Syntax: BOARD <new target board number>\n\n";

static char routeTestHelp[]    =   "    Syntax: SRT <point code>\n\n";

static char allHelp[]               =
#ifndef MTP2_ONLY
"    LINK       Link Control\n"
"    LINKSET    Linkset Control\n"
"    SRT        Route Test (JNTT only)\n"
"    MTP3FLOW   Mtp3Flow Control\n"
"    TRACE      Mtp3Trace Control\n"
#endif
"    MTP2FLOW   Mtp2Flow Control\n"
#ifndef MTP2_ONLY
"    STATUS     Link | Linkset | Route | NSap | General status\n"
"    STATS      Link | Linkset | Route | General statistics\n"
#else
"    STATUS     Link\n"
"    STATS      Link\n"
#endif
"    BOARD      Changes target board\n"
#ifndef MTP2_ONLY
"    Q          Quit mtp3mgr application\n\n";
#else
"    Q          Quit mtp2mgr application\n\n";
#endif

static char* helpTbl[] =
{
    allHelp,
#ifndef MTP2_ONLY
    linkCtrlHelp,
    linksetCtrlHelp,
    routeTestHelp,
    mtp3flowCtrlHelp,
    mtp3traceDataHelp,
#endif
    mtp2flowCtrlHelp,
    statusHelp,
    statsHelp,
		boardHelp,
    quitHelp
};
/* structure for command parse tables */
#define MAX_PARMS	10

struct CmdSpec {
	char				*command;
	struct lookupEnt	*parmTbl[MAX_PARMS];
    void                (*func)(U32 *);
};

/* Help Command */
struct CmdSpec  helpCmd =
{
	"?",               /* command name */
	    { CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	    HelpCommand         /* handler function */
};

/* mtp3mgmt API commands */
#ifndef MTP2_ONLY
struct CmdSpec	linkCtrlCmd =
{
	"LINK",				/* command name */
	{ 0, LinkCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },
	linkCtrl			/* handler function */
};

struct CmdSpec	linksetCtrlCmd =
{
	"LINKSET",			   					/* command name */
	{ 0, LinksetCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },	/* parameter lookups */
	linksetCtrl								/* handler function */
};

struct CmdSpec	alarmCtrlCmd =
{
	"ALARMS",			   					/* command name */
	{ AlarmCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },	/* parameter lookups */
	mtp3Ctrl								/* handler function */
};

struct CmdSpec  mtp3flowCtrlCmd =
{
    "MTP3FLOW",                             /* command name */
	{ FlowCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },	/* parameter lookups */
	mtp3Ctrl							/* handler function */
};

struct CmdSpec  mtp3traceDataCmd =
{
    "TRACE",                             /* command name */
    { TraceDataLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* parameter lookups */
	mtp3Ctrl							/* handler function */
};

#endif      /* not MTP2_ONLY */

struct CmdSpec  mtp2flowCtrlCmd =
{
    "MTP2FLOW",                             /* command name */
    { Mtp2FlowCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* parameter lookups */
    mtp2Ctrl                            /* handler function */
};

struct CmdSpec	statusCmd =
{
	"STATUS",			   				/* command name */
	{ StatusLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },	/* parameter lookups */
	statusReq							/* handler function */
};

struct CmdSpec	statsCmd =
{
	"STATS",			   				/* command name */
    { StatsLookupTbl, ResetLookupTbl, ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
	statsReq							/* handler function */
};

struct CmdSpec	boardCmd =
{
	"BOARD",			   							/* command name */
	{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
	setCP											/* handler function */
};

struct CmdSpec	quitCmd =
{
	"Q",			   								/* command name */
		{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
		quit											/* handler function */
};

#ifndef MTP2_ONLY
struct CmdSpec  routeTestCmd =
{
  "SRT",                 /* command name */
		{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
  routeTest              /* handler function */
};
#endif

struct CmdSpec *CmdTable[] = {
#ifndef MTP2_ONLY
		&linkCtrlCmd,
		&linksetCtrlCmd,
    &routeTestCmd,
    &mtp3flowCtrlCmd,
    &mtp3traceDataCmd,
#endif
    &mtp2flowCtrlCmd,
		&statusCmd,
		&statsCmd,
		&boardCmd,
		&quitCmd,
    &helpCmd,
    0
};


/* Misc. globals */
short   cp = 1;                 /* current board number attached to */
char	valStr[20];
short   quittinTime = 0;        /* set to 1 when user 'quit' command entered */
U8      myEntId = 49;           /* entity ID must be unique; would need to change
                                 * this to have multiple copies running simultaneously */
U8      myInstId = 0;           /* identifies processor I'm running on (0==PC) */

char    msg[200];               /* Used for error messages */
U8      bDebug = FALSE;         /* Turn debug (additional config display) on or not */
char    *pErrstr;               /* Used for return of error text */

#ifndef MTP2_ONLY
U8      pcDispFmt = SS7_PCDISP_DFLT;    /* desired display format for point codes */
#endif

/* Forward prototypes */
void StatusAllLinks (void);

/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char *getname( struct lookupEnt *tbl, U32 val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( tbl->value == val )
		{
			/* match found */
			return( tbl->name );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed - convert number to ascii */
    sprintf( valStr, "%d", val );
	return( valStr );

}	/* end getname() */


/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval( struct lookupEnt *tbl, char *s, U32 *val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( strcmp( tbl->name, s ) == 0 )
		{
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed */
	return( 0 );
}	/* end getval() */


#ifndef MTP2_ONLY
/*
 * linkCtrl command handler; command syntax:
 *      LINK <link number>  ENA | DIS | INH | UNI | DEL | RST | LPO | LPR | EMG | NRM | TRE | TRD
 */
void linkCtrl( U32 *parms )
{
    MTP3_STATUS status;
    S16         linkno;
    U8          action;

	linkno = (S16) parms[0];
	action = (U8) parms[1];
    status = Mtp3MgmtCtrl( (U8) cp, linkno, action );
    if( status != MTP3_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d Link %d Control Request %d failed: status = %s\n",
                cp, linkno, action, pErrstr);
	}
}

/*
 * linksetCtrl command handler; command syntax:
 *      LINKSET <linkset number>  ENA | DIS
 */
void linksetCtrl( U32 *parms )
{
    MTP3_STATUS status;
    S16         linkset;
    U8          action;

	linkset = (S16) parms[0];
	action = (U8) parms[1];
    status = Mtp3MgmtCtrl( (U8) cp, linkset, action );
    if( status != MTP3_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d Linkset %d Control Request %d failed: status = %s\n",
                cp, linkset, action, pErrstr );
	}
}

/*
 * mtp3Ctrl command handler; command syntax:
 *      MTP3FLOW    ON | OFF
 *      TRACE       ON | OFF
 *      ALARM      ENA | DIS
 */
void mtp3Ctrl( U32 *parms )
{
    MTP3_STATUS status;
    U8          action;

    action = (U8) parms[0];
    status = Mtp3MgmtCtrl( (U8) cp, 0, action );
    if( status != MTP3_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d MTP3 Control Request %d failed: status = %s\n",
                cp, action, pErrstr );
	}
}
/*
 *  routeTest command handler; command syntax:
 *    SRT pointcode
 */
void routeTest( U32 *parms )
{
    MTP3_STATUS status;
    U32         dpc;

    dpc = (U32) parms[0];

    status = Mtp3RouteTest( (U8) cp, dpc );
    if( status != MTP3_SUCCESS )
    {
        printf( "Board %d JNTT Route Test to point code %x failed: status = %x\n", cp, dpc, status);
    }
}
#endif      /* Not MTP2_ONLY */

/*
 * mtp2Ctrl command handler; command syntax:
 *      MTP2FLOW    ON | OFF
 */
void mtp2Ctrl( U32 *parms )
{
    MTP2_STATUS status;
    U8          action;

	action = (U8) parms[0];
    status = Mtp2MgmtCtrl( (U8) cp, 0, action );
    if( status != MTP2_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d MTP2 Control Request %d failed: status = %s\n",
                cp, action, pErrstr );
	}
}

/*
 * statusReq command handler; command syntax:
 *  	STATUS	LINK	<link number>
 *  	STATUS	LINKSET	<linkset number>
 *      STATUS  ROUTE   <point code>    ! point code is x.y.z or a number
 *  	STATUS	NSAP	<nsap number>
 *  	STATUS	MTP3
 */
void statusReq( U32 *parms )
{
    MTP2_STATUS status;
    U8          action;
    S16         entity;
#ifndef MTP2_ONLY
    U32         dpc;
    U32         opc;
    char        text[20];       /* Used for ASCII status displays */
    short       i;              /* Loop counter */
#else
    MTP2LinkCfg Link2Cfg;
#endif

    union {
#ifndef MTP2_ONLY
        MTP3GenStatus       genStatus;      /* MTP 3 Signaling point status */
        MTP3NSapStatus      nsapStatus;     /* MTP 3 NSAP status */
        MTP3LinkStatus      linkStatus;     /* MTP 3 Link status */
        MTP3LinkSetStatus   linkSetStatus;  /* MTP 3 Linkset status */
        MTP3RouteStatus     routeStatus;    /* MTP 3 Route status */
#endif
        MTP2LinkStatus      link2Status;    /* MTP 2 Link status */
    } sta;

	action = (U8) parms[0];
	entity = (S16) parms[1];

    switch( action )
    {
#ifndef MTP2_ONLY
        case MTP3_GEN_STATUS:
            status = Mtp3GenStatus( (U8) cp, &sta.genStatus );
            break;

        case MTP3_NSAP_STATUS:
            status = Mtp3NSapStatus( (U8) cp, entity, &sta.nsapStatus );
            break;

        case MTP3_DLSAP_STATUS:
            if (entity == ALL_LINKS)
                {
                StatusAllLinks ();
                return;
                }

            status = Mtp3LinkStatus( (U8) cp, entity, &sta.linkStatus );
            break;

        case MTP3_ROUTE_STATUS:
            dpc = (U32) parms[1];
            opc = (U32) parms[2];
            status = Mtp3RouteStatus( (U8) cp, dpc, opc, &sta.routeStatus );
            break;

        case MTP3_LNKSET_STATUS:
            status = Mtp3LinkSetStatus( (U8) cp, entity, &sta.linkSetStatus );
            break;
#endif

#ifdef MTP2_ONLY
        case MTP3_DLSAP_STATUS:
            if (entity == ALL_LINKS)
                {
                StatusAllLinks ();
                return;
                }

            status = Mtp2LinkStatus( (U8) cp, entity, &sta.link2Status );
            if (status != MTP2_SUCCESS)
                {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Status Request failed: status = %s\n",
                        cp, pErrstr );
                return;
                }

            printf( "\nMTP2 Board %d, Link %d:\n", cp, entity );
            printf ("High lvl state = %-8s   Frames out    = %-5d Frames dropped = %-8d\n",
                hlStates[sta.link2Status.hlSt], sta.link2Status.psOutsFrm, sta.link2Status.cntMaDrop);
            printf ("Low  lvl state = %-20s\n",
                llStates[sta.link2Status.llSt]);

            printf ("---- Current Link (%d) Layer 2 configuration ----\n", entity);
            status = Mtp2GetLinkCfg( (U8) cp, &Link2Cfg, entity );
            if (status != MTP2_SUCCESS)
                {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Get Cfg Request failed: status = %s\n",
                        cp, pErrstr );
                return;
                }

            printf ("maxFrmLen  = %-8u   congDisc    = %-8u\n",
                Link2Cfg.maxFrmLen    , Link2Cfg.congDisc  );
            printf ("errType    = %-8s   lssuLen     = %-8u   SUERMD rate = %-8u\n",
                Link2Cfg.errType ? "CYCLICAL" : "NORMAL " ,
                Link2Cfg.lssuLen, Link2Cfg.sdD );
            printf ("SUERM Thrsh= %-8u   AERM Thrsh N= %-8u   AERM Thrsh E= %-8u\n",
                Link2Cfg.sdT          , Link2Cfg.sdTin     , Link2Cfg.sdTie     );
            printf ("Max RTB Msg= %-8u   Max RTB Oct = %-8u   Max Prov Abt= %-8u\n",
                Link2Cfg.sdN1         , Link2Cfg.sdN2      , Link2Cfg.sdCp      );
            printf ("IsoThresh  = %-8u   TxqThresh1  = %-8u   TxqThresh1A = %-8u\n",
                Link2Cfg.isoThresh    , Link2Cfg.txqThresh1, Link2Cfg.txqThresh1Abate);
            printf ("TxqThresh2 = %-8u   TxqThresh2A = %-8u   SapThresh    = %-8u\n",
                Link2Cfg.txqThresh2, Link2Cfg.txqThresh2Abate, Link2Cfg.sapThresh);
            printf ("SapThreshA = %-8u   IdleFreq    = %-8u   RtFreq        = %-8u\n\n",
                Link2Cfg.sapThreshAbate, Link2Cfg.idleFreq, Link2Cfg.rtFreq);

            printf ("L2 t1      = %-8u   L2 t2       = %-8u   L2 t3       = %-8u\n",
                Link2Cfg.t1.value     , Link2Cfg.t2.value  , Link2Cfg.t3.value  );
            printf ("L2 t4 norm = %-8u   L2 t4 emrg  = %-8u   L2 t5       = %-8u\n",
                Link2Cfg.provNormal   , Link2Cfg.provEmrgcy, Link2Cfg.t5.value  );
            printf ("L2 t6      = %-8u   L2 t7       = %-8u   L2 t10      = %-8u\n",
                Link2Cfg.t6.value     , Link2Cfg.t7.value,   Link2Cfg.t10.value);
            printf ("L2 t11     = %-8u   L2 t12      = %-8u   L2 t13      = %-8u\n\n",
                Link2Cfg.t11.value     , Link2Cfg.t12.value,   Link2Cfg.t13.value);

            return;
#endif

        default:
#ifdef MTP2_ONLY
            printf( "Unknown status request: LINK expectd\n" );
#else
            printf( "Unknown status request: LINK, LINKSET, ROUTE, NSAP, or MTP3 expectd\n" );
#endif
            return;
    }

#ifndef MTP2_ONLY
    if( status != MTP3_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d MTP3 Status Request failed: status = %s\n",
                cp, pErrstr );
		return;
	}

	switch( action )
	{
		case MTP3_GEN_STATUS:
            {
            MTP3GenCfg  GenCfg;
            short       i;

            switch (sta.genStatus.haState)
            {
                case SN_HAST_STANDALONE:
                    strcpy (text, "STAND_ALONE");
                    break;

                case SN_HAST_STARTING:
                    strcpy (text, "STARTING");
                    break;

                case SN_HAST_PRIMARY:
                    strcpy (text, "PRIMARY");
                    break;

                case SN_HAST_BACKUP:
                    strcpy (text, "BACKUP");
                    break;

                default:
                    sprintf (text, "UNKNOWN (%d)", sta.genStatus.haState);
                    break;
            }
            if( sta.genStatus.spRst )
                printf( "\nBoard %d SP State: Restarting      HA State: %s\n\n", cp, text );
			else
                printf( "\nBoard %d SP State: Normal          HA State: %s\n\n", cp, text );

            status = Mtp3GetGenCfg( (U8) cp, &GenCfg );
            if (status == MTP3_SUCCESS)
                {
                if( GenCfg.pcDispFmt == SS7_PCDISP_INTL )
                    strcpy( text, "INTL" );
                else if( GenCfg.pcDispFmt == SS7_PCDISP_JNTT )
                    strcpy( text, "JNTT/JTTC" );
                else
                    strcpy( text, "DEFAULT" );

                printf ("---------------- Current general configuration ---------------\n");
                printf ("typeSP     = %-8s   PcDispFmt   = %-8s\n",
                    (GenCfg.typeSP == MTP3TYPE_SP) ? "SP " : "STP", text );
                printf ("spCode1    = %-22s\n", CfgFmtPc( GenCfg.spCode1, PCLEN_DEFAULT, pcDispFmt ) );
                printf ("spCode2    = %-22s\n", CfgFmtPc( GenCfg.spCode2, PCLEN_DEFAULT, pcDispFmt ) );
                printf ("rstReq     = %-8s   nmbLinks    = %-8u  nmbNSap     = %-8u\n",
                    GenCfg.rstReq ? "TRUE " : "FALSE", GenCfg.nmbLinks, GenCfg.nmbNSap);
                printf ("nmbLnkSets = %-8u   nmbRouts    = %-8u  nmbRteInst  = %-8u\n",
                    GenCfg.nmbLnkSets, GenCfg.nmbRouts, GenCfg.nmbRteInst);

                printf ("nmbMasks   = %-8u\n",
                    GenCfg.nmbMasks);
                for (i = 0; i < GenCfg.nmbMasks; i++)
                    printf ("  Mask[%-2d] = %-8lx\n", i, GenCfg.masks[i]);
                printf ("\n    ----- Signalling Point Timers (in tenths/sec) -----\n");
                printf ("t15         = %-8u  t16         = %-8u  tRteInst    = %-8u\n",
                    GenCfg.tmr.t15.value, GenCfg.tmr.t16.value, GenCfg.tmr.tRteInst.value);
                printf ("t18_itu     = %-8u  t20_itu     = %-8u\n",
                    GenCfg.tmr.t18_itu.value, GenCfg.tmr.t20_itu.value);
                printf ("t22_ansi    = %-8u  t23_ansi    = %-8u  t24_ansi    = %-8u\n",
                    GenCfg.tmr.t22_ansi.value, GenCfg.tmr.t23_ansi.value, GenCfg.tmr.t24_ansi.value);
                printf ("t26_ansi    = %-8u  t27_ansi    = %-8u\n\n",
                    GenCfg.tmr.t26_ansi.value, GenCfg.tmr.t27_ansi.value);
                if (bDebug)
                    {
                    printf ("\nNon-configurable values\n\n");
                    printf ("ssfValid   = %-8u   cbTimeRes   = %-8u\n",
                        GenCfg.ssfValid  , GenCfg.cbTimeRes);
                    printf ("icbTimeRes = %-8u   spTimeRes   = %-8u  rteTimeRes  = %-8u\n",
                        GenCfg.icbTimeRes, GenCfg.spTimeRes, GenCfg.rteTimeRes);
                    }
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf ("Mtp3GetGenCfg returned %s\n", pErrstr);
                }
            }
			break;

		case MTP3_NSAP_STATUS:
            {
            MTP3NSapCfg NSapCfg;

            printf( "\nBoard %d, NSap %d:\n", cp, entity );
			printf( "  State  = %s\n",
                    getname( NSapStatesTbl, (short) sta.nsapStatus.sapState ) );
			printf( "  Flow Control: %s\n\n",
                    ((sta.nsapStatus.flcSt) ? "On" : "Off"));
            printf( "  Congestion Discards: %d", sta.nsapStatus.congDiscard);
            printf ("\n---- Current NSAP (%d) configuration ----\n", entity);
            status = Mtp3GetNSapCfg( (U8) cp, &NSapCfg, entity );
            if (status == MTP3_SUCCESS)
                {
                char text[20];

                if (NSapCfg.lnkType == MTP3LNK_CCITT)
                    strcpy (text, "CCITT");
                else if (NSapCfg.lnkType == MTP3LNK_ANSI88)
                    strcpy (text, "ANSI88");
                else if (NSapCfg.lnkType == MTP3LNK_ANSI)
                    strcpy (text, "ANSI");
                else if (NSapCfg.lnkType == MTP3LNK_JNTT)
                    strcpy (text, "JNTT");
                else if (NSapCfg.lnkType == MTP3LNK_JTTC)
                    strcpy (text, "JTTC");
                else
                    strcpy (text, "UNKNOWN");
                printf ("p0QLen     = %-8u   p1QLen      = %-8u   p2QLen      = %-8u\n",
                    NSapCfg.p0QLen, NSapCfg.p1QLen, NSapCfg.p2QLen);
                printf ("p3QLen     = %-8u   lnkType     = %-8s   dpcLen      = %-8s\n\n",
                    NSapCfg.p3QLen, text,
                    (NSapCfg.dpcLen == PCLEN_ITU) ? "14" : ((NSapCfg.dpcLen == PCLEN_JAPAN) ? "16" : "24") );
                if (bDebug)
                    {
                    printf ("\nNon-configurable values\n\n");
                    printf ("selector    = %-8u   opc        = %-8lx  route        = %-8lx\n",
                        NSapCfg.selector, NSapCfg.opc, NSapCfg.route);
                    printf ("priority    = %-8u\n", NSapCfg.prior);
                    }
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf ("Mtp3GetNSapCfg returned %s", pErrstr);
                }
            }
			break;

		case MTP3_DLSAP_STATUS:
            {
            MTP3LinkCfg LinkCfg;
            MTP2LinkCfg Link2Cfg;

            printf( "\nMTP3 Board %d, Link %d:\n", cp, entity);
			printf( "  State  = %s\n",
                    getname( LinkStatesTbl, (short) sta.linkStatus.state ) );
            if(  sta.linkStatus.locBlkd )
				printf( "Blocked (Local)\n" );
            else if(  sta.linkStatus.blkd )
				printf( "Blocked\n" );
            if ( sta.linkStatus.locInhbt || sta.linkStatus.rmtInhbt ||
                sta.linkStatus.inhbt)
                printf("Inhibited : %s %s\n",
                    sta.linkStatus.locInhbt ? "Local" : "",
                    sta.linkStatus.rmtInhbt ? "Remote" : "");
            if(  sta.linkStatus.uninhbt )
				printf( "Uninhibiting\n" );
            if(  sta.linkStatus.cong )
				printf( "Congested\n" );
            if(  sta.linkStatus.emerg )
				printf( "Emergency\n" );
			printf( "\n" );

            printf ("---- Current Link (%d) Layer 3 configuration ----\n", entity);
            status = Mtp3GetLinkCfg( (U8) cp, &LinkCfg, entity );
            if (status == MTP3_SUCCESS)
                {
                if (LinkCfg.lnkType == MTP3LNK_CCITT)
                    strcpy (text, "CCITT");
                else if (LinkCfg.lnkType == MTP3LNK_ANSI88)
                    strcpy (text, "ANSI88");
                else if (LinkCfg.lnkType == MTP3LNK_ANSI)
                    strcpy (text, "ANSI");
                else if (LinkCfg.lnkType == MTP3LNK_JNTT)
                    strcpy (text, "JNTT");
                else if (LinkCfg.lnkType == MTP3LNK_JTTC)
                    strcpy (text, "JTTC");
                else
                    strcpy (text, "UNKNOWN");
                printf ("bandwidth= %-8s   seqNum    = %-8s   opc       = %-16s\n",
                    (LinkCfg.hsBits & HS_LINK) ? "HSL" : "LSL",
                    (LinkCfg.hsBits & HS_EXT_SEQ) ? "Extended" : "Normal",
                    CfgFmtPc( LinkCfg.opc, LinkCfg.dpcLen, pcDispFmt ) );
                printf ("lnkSetId = %-8u   lnkType   = %-8s   adjDpc    = %-16s\n",
                    LinkCfg.lnkSetId, text, CfgFmtPc( LinkCfg.adjDpc, LinkCfg.dpcLen, pcDispFmt ) );
                if (LinkCfg.ssf == MTP3SSF_INTL || LinkCfg.ssf == MTP3SSF_NAT)
                    printf ("ssf      = %-8s   dpcLen    = %-8s   lnkPrior  = %-8u\n",
                        (LinkCfg.ssf == MTP3SSF_INTL) ? "INTL" : "NATL" ,
                        (LinkCfg.dpcLen == PCLEN_ITU) ? "14" : ((LinkCfg.dpcLen == PCLEN_JAPAN) ? "16" : "24"),
                         LinkCfg.lnkPrior);
                else
                    printf ("ssf      = %-8u   dpcLen    = %-8s   lnkPrior  = %-8u\n",
                         LinkCfg.ssf,
                        (LinkCfg.dpcLen == PCLEN_ITU) ? "14" : ((LinkCfg.dpcLen == PCLEN_JAPAN) ? "16" : "24"),
                         LinkCfg.lnkPrior);
                printf ("msgSize  = %-8u   msgPrior  = %-8u   maxSLTtry = %-8u\n",
                    LinkCfg.msgSize , LinkCfg.msgPrior, LinkCfg.maxSLTtry);
                printf ("p0QLen   = %-8u   p1QLen    = %-8u   p2QLen    = %-8u\n",
                    LinkCfg.p0QLen  , LinkCfg.p1QLen  , LinkCfg.p2QLen   );
                printf ("p3QLen   = %-8u   discPrior = %-8u   disable   = %-8s\n",
                    LinkCfg.p3QLen  , LinkCfg.discPrior,
                    LinkCfg.dis ? "TRUE " : "FALSE"   );
                printf ("lnkTstSLC= %-8u   tstString = ",
                    LinkCfg.lnkTstSLC);
                /* Print test string in hex */
                for (i = 0; i < LinkCfg.tstLen; i++)
                    printf ("%-2X ", LinkCfg.tst[i]);
                printf ("\n");

                if (!bDebug)        /* No room for this header line if debug on */
                    {
                    printf ("         ---- Link Timers (in tenths/sec) ----\n");
                    }
                printf ("t1       = %-8u   t2        = %-8u   t3        = %-8u\n",
                    LinkCfg.tmr.t1.value  , LinkCfg.tmr.t2.value   , LinkCfg.tmr.t3.value   );
                printf ("t4       = %-8u   t5        = %-8u   t6        = %-8u\n",
                    LinkCfg.tmr.t4.value  , LinkCfg.tmr.t5.value   , LinkCfg.tmr.t6.value   );
                printf ("t7       = %-8u   t11       = %-8u   t12       = %-8u\n",
                    LinkCfg.tmr.t7.value  , LinkCfg.tmr.t11.value  , LinkCfg.tmr.t12.value  );
                printf ("t13      = %-8u   t14       = %-8u   t17       = %-8u\n",
                    LinkCfg.tmr.t13.value , LinkCfg.tmr.t14.value  , LinkCfg.tmr.t17.value  );
                printf ("t22      = %-8u   t23       = %-8u   t24       = %-8u\n",
                    LinkCfg.tmr.t22.value , LinkCfg.tmr.t23.value  , LinkCfg.tmr.t24.value  );
                printf ("t31      = %-8u   t32       = %-8u   t33       = %-8u\n",
                    LinkCfg.tmr.t31.value , LinkCfg.tmr.t32.value  , LinkCfg.tmr.t33.value  );
                printf ("t34      = %-8u   t40       = %-8u   t41       = %-8u\n",
                    LinkCfg.tmr.t34.value , LinkCfg.tmr.t40.value  , LinkCfg.tmr.t41.value  );
                printf ("t42      = %-8u   t43       = %-8u   t44       = %-8u\n",
                    LinkCfg.tmr.t42.value , LinkCfg.tmr.t43.value  , LinkCfg.tmr.t44.value  );

                if (bDebug)
                    {
                    printf ("                 ----Non-configurable values----\n");
                    printf ("dstEnt   = %-8u   dstInst   = %-8u   selector  = %-8u\n",
                        LinkCfg.dstEnt, LinkCfg.dstInst, LinkCfg.selector);
                    printf ("prior    = %-8u   route     = %-8u   spId      = %-8u\n",
                        LinkCfg.prior , LinkCfg.route  , LinkCfg.spId    );
                    printf ("lnkId    = %-8u   portType  = %-8u\n",
                        LinkCfg.lnkId, LinkCfg.portType);
                    }
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf ("Mtp3GetLinkCfg returned %s\n", pErrstr);
                }


            if (LinkCfg.portType == PORT_TYPE_REMOTE)
            {
                printf ("\nRemote Link.  MTP2 status unavailable from this board.\n");
                return;
            }

            printf ("\nPress [Enter] for MTP2 status\n");
            getch();

            status = Mtp2LinkStatus( (U8) cp, entity, &sta.link2Status );
            if (status == MTP2_SUCCESS)
                {
                printf( "\nBoard %d, Link %d MTP2 Status:\n\n", cp, entity );

                printf ("High lvl state = %-8s   Frames out    = %-5d Frames dropped = %-8d\n",
                    hlStates[sta.link2Status.hlSt], sta.link2Status.psOutsFrm, sta.link2Status.cntMaDrop);
                printf ("Low  lvl state = %-20s\n",
                    llStates[sta.link2Status.llSt]);
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Status Request failed: status = %s\n",
                        cp, pErrstr );
                }

            printf ("\n---- Current Link (%d) Layer 2 configuration ----\n\n", entity);
            status = Mtp2GetLinkCfg( (U8) cp, &Link2Cfg, entity );
            if (status == MTP2_SUCCESS)
                {
                /* Display configured link name, ie. S2, T1, etc. */
                text[0] = (Link2Cfg.portType == 0) ? 'S' : 'T';
                sprintf (&text[1], "%d", Link2Cfg.portNum);
                printf ("Link Name  = %-8s   bandwidth   = %-8s   seqNum      = %-8s\n", text,
                    (Link2Cfg.hsBits & HS_LINK) ? "HSL" : "LSL",
                    (Link2Cfg.hsBits & HS_EXT_SEQ) ? "Extended" : "Normal");
                printf ("maxFrmLen  = %-8u   congDisc    = %-8s\n",
                    Link2Cfg.maxFrmLen    ,
                    Link2Cfg.congDisc ? "TRUE " : "FALSE"   );
                printf ("errType    = %-8s   lssuLen     = %-8u   SUERMD rate = %-8u\n",
                    Link2Cfg.errType ? "CYCLICAL" : "NORMAL " ,
                    Link2Cfg.lssuLen, Link2Cfg.sdD );
                printf ("SUERM Thrsh= %-8u   AERM Thrsh N= %-8u   AERM Thrsh E= %-8u\n",
                    Link2Cfg.sdT          , Link2Cfg.sdTin     , Link2Cfg.sdTie     );
                printf ("Max RTB Msg= %-8u   Max RTB Oct = %-8u   Max Prov Abt= %-8u\n\n",
                    Link2Cfg.sdN1         , Link2Cfg.sdN2      , Link2Cfg.sdCp      );
                printf ("IsoThresh  = %-8u   TxqThresh1  = %-8u   TxqThresh1A = %-8u\n",
                    Link2Cfg.isoThresh    , Link2Cfg.txqThresh1, Link2Cfg.txqThresh1Abate);
                printf ("TxqThresh2 = %-8u   TxqThresh2A = %-8u   SapThresh   = %-8u\n",
                    Link2Cfg.txqThresh2, Link2Cfg.txqThresh2Abate, Link2Cfg.sapThresh);
                printf ("SapThreshA = %-8u   IdleFreq    = %-8u   RtFreq      = %-8u\n\n",
                    Link2Cfg.sapThreshAbate, Link2Cfg.idleFreq, Link2Cfg.rtFreq);

                printf ("\n       ----- Layer 2 Timers (in tenths/sec) -----\n");
                printf ("L2 t1      = %-8u   L2 t2       = %-8u   L2 t3       = %-8u\n",
                    Link2Cfg.t1.value     , Link2Cfg.t2.value  , Link2Cfg.t3.value  );
                printf ("L2 t4 norm = %-8u   L2 t4 emrg  = %-8u   L2 t5       = %-8u\n",
                    Link2Cfg.provNormal   , Link2Cfg.provEmrgcy, Link2Cfg.t5.value  );
                printf ("L2 t6      = %-8u   L2 t7       = %-8u   L2 t10      = %-8u\n",
                    Link2Cfg.t6.value     , Link2Cfg.t7.value,   Link2Cfg.t10.value);
                printf ("L2 t11     = %-8u   L2 t12      = %-8u   L2 t13      = %-8u\n\n",
                    Link2Cfg.t11.value     , Link2Cfg.t12.value,   Link2Cfg.t13.value);
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Get Link Config Request failed: status = %s\n",
                        cp, pErrstr );
                }
            }
			break;

		case MTP3_LNKSET_STATUS:
            {
            MTP3LinkSetCfg      LinksetCfg;
            short               i;

            printf( "\nBoard %d, Linkset %d:\n", cp, entity );
            printf( "  Active Links: %d\n", sta.linkSetStatus.nmbActLnks );
            printf( "  Congested Links: %d\n", sta.linkSetStatus.nmbCongLnks );
			printf( "  State: %s, %s\n\n",
                    ((sta.linkSetStatus.state == 0) ? "Active" : "Inactive"),
                    ((sta.linkSetStatus.cong) ? "Congested" : "Not Congested"));

            printf ("---- Current Linkset (%d) configuration ----\n", entity);
            status = Mtp3GetLinkSetCfg( (U8) cp, &LinksetCfg, entity );
            if (status == MTP3_SUCCESS)
                {
                printf ("                          opc          = %s\n",
                    CfgFmtPc( LinksetCfg.opc, PCLEN_DEFAULT, pcDispFmt ) );
                printf ("lnkSetId     = %-8u   adjDpc       = %s\n",
                    LinksetCfg.lnkSetId,
                    CfgFmtPc( LinksetCfg.adjDpc, PCLEN_DEFAULT, pcDispFmt ) );
                printf ("minNmbActLnk = %-8u   nmbCmbLnkSet = %-8u\n",
                    LinksetCfg.minNmbActLnk, LinksetCfg.nmbCmbLnkSet);
                printf ("      ---- Combined Linksets ----\n");
                for (i = 0; i < LinksetCfg.nmbCmbLnkSet; i++)
                    {
                    printf ("cmbLnkSetId  = %-8u   lnkSetPrior  = %-8u\n",
                        LinksetCfg.cmbLnkSet[i].cmbLnkSetId, LinksetCfg.cmbLnkSet[i].lnkSetPrior);
                    }
                }
            else
                {
                Mtp2Geterror(status, &pErrstr);
                printf ("Mtp3GetLinkSetCfg returned %s\n", pErrstr);
                }
            }
			break;

		case MTP3_ROUTE_STATUS:
            {
            MTP3RouteCfg    RouteCfg;

            /* Note:  Had to break up the following printf because calling CfgFmtPc twice in the same printf returns
             *        the first value even on the second call.  Some caching or static variable must be going on */
            printf( "\nBoard %d, Route DPC %s, ", cp,
                    CfgFmtPc( sta.routeStatus.dpc, PCLEN_DEFAULT, pcDispFmt ) );
            printf("OPC %s:\n", CfgFmtPc( sta.routeStatus.opc, PCLEN_DEFAULT, pcDispFmt ) );
            printf( "  Active Linksets: %d\n", sta.routeStatus.nmbActvLnkSets );
            printf( "  Congested Linksets: %d\n", sta.routeStatus.nmbCongLnkSets );
			printf( "  State: %s, %s\n\n",
                    ((sta.routeStatus.state == 0) ? "Unavailable" : "Available"),
                    ((sta.routeStatus.cong) ? "Congested" : "Not Congested"));

            printf ("---- Current Route (0x%x, 0x%x) configuration ----\n", sta.routeStatus.dpc, sta.routeStatus.opc);
            status = Mtp3GetRouteCfg( (U8) cp, &RouteCfg, sta.routeStatus.dpc, sta.routeStatus.opc);
            if (status == MTP3_SUCCESS)
                {
                char text[20];

                if (RouteCfg.swtchType == MTP3LNK_CCITT)
                    strcpy (text, "CCITT");
                else if (RouteCfg.swtchType == MTP3LNK_ANSI88)
                    strcpy (text, "ANSI88");
                else if (RouteCfg.swtchType == MTP3LNK_ANSI)
                    strcpy (text, "ANSI");
                else if (RouteCfg.swtchType == MTP3LNK_JNTT)
                    strcpy (text, "JNTT");
                else if (RouteCfg.swtchType == MTP3LNK_JTTC)
                    strcpy (text, "JTTC");
                else
                    strcpy (text, "UNKNOWN");
                printf ("RouteId     = %-8u  spType       = %-8s   dpc      = %s\n",
                    RouteCfg.cmbLnkSetId,
                    (RouteCfg.spType == MTP3TYPE_STP) ? "STP" : "SP",
                    CfgFmtPc( RouteCfg.dpc, PCLEN_DEFAULT, pcDispFmt ) );
                printf ("swtchType   = %-8s  direction    = %-8s   opc      = %s\n",
                    text , (RouteCfg.dir == MTP3_RTE_DN) ? "DOWN" : "UP",
                    CfgFmtPc( RouteCfg.opc, PCLEN_DEFAULT, pcDispFmt ) );
                if (RouteCfg.ssf == MTP3SSF_INTL || RouteCfg.ssf == MTP3SSF_NAT)
                    printf ("rteToAdjSp  = %-8s  rteToCluster = %-8s   ssf      = %-8s\n",
                        RouteCfg.rteToAdjSp ? "TRUE " : "FALSE",
                        RouteCfg.rteToCluster ? "TRUE " : "FALSE",
                        (RouteCfg.ssf == MTP3SSF_INTL) ? "INTL" : "NATL");
                else
                    printf ("rteToAdjSp  = %-8s  rteToCluster = %-8s   ssf      = %-8u\n",
                        RouteCfg.rteToAdjSp ? "TRUE " : "FALSE",
                        RouteCfg.rteToCluster ? "TRUE " : "FALSE",
                        RouteCfg.ssf);
                printf ("              ---- Route Timers (in tenths/sec) ----\n");
                printf ("t8          = %-8u  t10          = %-8u\n",
                    RouteCfg.tmr.t8.value , RouteCfg.tmr.t10.value  );
                printf ("t19_itu     = %-8u  t21_itu      = %-8u\n",
                    RouteCfg.tmr.t19_itu.value , RouteCfg.tmr.t21_itu.value);
                printf ("t25_ansi    = %-8u  t28_ansi     = %-8u  t29_ansi     = %-8u\n\n",
                    RouteCfg.tmr.t25_ansi.value, RouteCfg.tmr.t28_ansi.value, RouteCfg.tmr.t29_ansi.value);
                /* Note we don't use T30 so don't display */
                }
            else
                printf ("Mtp3GetRouteCfg returned %d\n", status);
            }
			break;

		default:
			break;
	}
#endif      /* Not MTP2_ONLY */
}

/*
 * statsReq command handler; command syntax:
 *      STATS   LINK    <link number> [ RESET ]
 *      STATS   LINKSET <linkset number> [ RESET ]
 *      STATS   ROUTE   <point code> [ RESET ]  ! point code is x.y.z or a number
 *      STATS   MTP3    [ RESET ]
 */
void statsReq( U32 *parms )
{
    MTP2_STATUS status;
    U8          action;
    Bool        reset;
    S16         entity;
#ifndef MTP2_ONLY
    U32         dpc;
    U32         opc;
#endif

    union
    {
#ifndef MTP2_ONLY
        MTP3GenStats        genStats;       /* MTP 3 General/Global stats */
        MTP3LinkStats       linkStats;      /* MTP 3 Link stats */
        MTP3LinkSetStats    linkSetStats;   /* MTP 3 Linkset stats */
        MTP3RouteStats      routeStats;     /* MTP 3 Route stats */
#endif
        MTP2LinkStats       link2Stats;     /* MTP 2 Link stats */
    } stats;

	action = (U8) parms[0];
	entity = (S16) parms[1];
    reset  = (Bool) parms[2];

    switch( action )
    {
#ifndef MTP2_ONLY
        case MTP3_GEN_STATS:
            reset  = (Bool) parms[1];
            status = Mtp3GenStats( (U8) cp, &stats.genStats, reset );
            break;

        case MTP3_DLSAP_STATS:
            status = Mtp3LinkStats( (U8) cp, entity, &stats.linkStats, reset );
            break;

        case MTP3_ROUTE_STATS:
            dpc = (U32) parms[1];
            opc = (U32) parms[2];
            status = Mtp3RouteStats( (U8) cp, dpc, opc, &stats.routeStats, reset );
            break;

        case MTP3_LNKSET_STATS:
            status = Mtp3LinkSetStats( (U8) cp, entity, &stats.linkSetStats, reset );
            break;
#endif

#ifdef MTP2_ONLY
        case MTP3_DLSAP_STATS:
            status = Mtp2LinkStats( (U8) cp, entity, &stats.link2Stats, reset );
            if( status != MTP2_SUCCESS )
            {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Statistics Request failed: status = %s\n",
                        cp, pErrstr );
                return;
            }

            printf( "Level 2 Stats:\n", cp, entity );
            printf ("inService  = %-8ld  lclBusy     = %-8ld  slFailAll   = %-8ld\n",
                stats.link2Stats.inService    , stats.link2Stats.lclBusy   , stats.link2Stats.slFailAll );
            printf ("slFailAb   = %-8ld  slFailAck   = %-8ld  slFailErr   = %-8ld\n",
                stats.link2Stats.slFailAb     , stats.link2Stats.slFailAck , stats.link2Stats.slFailErr );
            printf ("slFailCong = %-8ld  slFailAlign = %-8ld  slNSUErr    = %-8ld\n",
                stats.link2Stats.slFailCong   , stats.link2Stats.slFailAlign , stats.link2Stats.slNSUErr );
            printf ("slNegAck   = %-8ld  nRetrans    = %-8ld\n",
                stats.link2Stats.slNegAck     , stats.link2Stats.nRetrans );
			printf( "Msg:       Rx        Tx          Msg:       Rx        Tx\n" );
			printf( "----       --        --          ----       --        --\n" );
            printf( "SIF/SIO    %-8u  %-8u    MSU        %-8u  %-8u\n",
                    stats.link2Stats.nSifSioRx, stats.link2Stats.nSifSioTx,
                    stats.link2Stats.mRx, stats.link2Stats.mTx);
            printf( "LSSU*      %-8u  %-8u    FISU*      %-8u  %-8u\n\n",
                    stats.link2Stats.lRx, stats.link2Stats.lTx,
                    stats.link2Stats.fRx, stats.link2Stats.fTx);
            printf ("* LSSU & FISU counts do not include those filtered by the firmware\n\n");
            return;
#endif

        default:
#ifdef MTP2_ONLY
            printf( "Unknown statistics request: LINK expectd\n" );
#else
            printf( "Unknown statistics request: LINK, LINKSET, ROUTE, or MTP3 expectd\n" );
#endif
            return;
    }

#ifndef MTP2_ONLY
    if( status != MTP3_SUCCESS )
	{
        Mtp2Geterror(status, &pErrstr);
        printf( "Board %d MTP3 Statistics Request failed: status = %s\n",
                cp, pErrstr );
		return;
	}

	switch( action )
	{
		case MTP3_GEN_STATS:
			printf( "Board %d Stats\n", cp );
			printf( "Msg:       Rx        Tx\n" );
			printf( "----       --        --\n" );
			printf( "UPU        %-8u  %-8u\n",
                    stats.genStats.usrUnavailRx, stats.genStats.usrUnavailTx);
			printf( "TRA        %-8u  %-8u\n",
                    stats.genStats.traRx, stats.genStats.traTx);
			printf( "TRW        %-8u  %-8u\n\n",
                    stats.genStats.trwRx, stats.genStats.trwTx);
			printf( "Packets dropped due to routing errors: %-8u\n\n",
                    stats.genStats.msuDropRteErr );
 			break;

		case MTP3_DLSAP_STATS:
        {
            MTP3LinkCfg LinkCfg;

            printf( "\nBoard %d, Link %d MTP3 Stats:\n\n", cp, entity );
			printf( "Msg:       Rx        Tx          Msg:       Rx        Tx\n" );
			printf( "----       --        --          ----       --        --\n" );
			printf( "COO        %-8u  %-8u    COA        %-8u  %-8u\n",
                    stats.linkStats.changeOverRx, stats.linkStats.changeOverTx,
                    stats.linkStats.changeOverAckRx, stats.linkStats.changeOverAckTx);
			printf( "CBD        %-8u  %-8u    CBA        %-8u  %-8u\n",
                    stats.linkStats.changeBackRx, stats.linkStats.changeBackTx,
                    stats.linkStats.changeBackAckRx, stats.linkStats.changeBackAckTx);
			printf( "ECO        %-8u  %-8u    ECA        %-8u  %-8u\n",
                    stats.linkStats.emChangeOverRx, stats.linkStats.emChangeOverTx,
                    stats.linkStats.emChangeOverAckRx, stats.linkStats.emChangeOverAckTx);
			printf( "LIN        %-8u  %-8u    LIA        %-8u  %-8u\n",
                    stats.linkStats.lnkInhRx, stats.linkStats.lnkInhTx,
                    stats.linkStats.lnkInhAckRx, stats.linkStats.lnkInhAckTx);
			printf( "LUN        %-8u  %-8u    LUA        %-8u  %-8u\n",
                    stats.linkStats.lnkUninhRx, stats.linkStats.lnkUninhTx,
                    stats.linkStats.lnkUninhAckRx, stats.linkStats.lnkUninhAckTx);
			printf( "LID        %-8u  %-8u    LFU        %-8u  %-8u\n",
                    stats.linkStats.lnkInhDenRx, stats.linkStats.lnkInhDenTx,
                    stats.linkStats.lnkForceUninhRx, stats.linkStats.lnkForceUninhTx);
			printf( "LLI        %-8u  %-8u    LRI        %-8u  %-8u\n",
                    stats.linkStats.lnkLocInhTstRx, stats.linkStats.lnkLocInhTstTx,
                    stats.linkStats.lnkRmtInhTstRx, stats.linkStats.lnkRmtInhTstTx);
			printf( "DLC        %-8u  %-8u    CSS        %-8u  %-8u\n",
                    stats.linkStats.lnkConOrdRx, stats.linkStats.lnkConOrdTx,
                    stats.linkStats.lnkConAckRx, stats.linkStats.lnkConAckTx);
			printf( "SLTM       %-8u  %-8u    SLTA       %-8u  %-8u\n",
                    stats.linkStats.lnkTstRx, stats.linkStats.lnkTstTx,
                    stats.linkStats.lnkTstAckRx, stats.linkStats.lnkTstAckTx);
			printf( "TFC        %-8u  %-8u\n",
                    stats.linkStats.txCntrlRx, stats.linkStats.txCntrlTx);
			printf( "MSU        %-8u  %-8u\n",
                    stats.linkStats.msuRx, stats.linkStats.msuTx);
			printf( "SIF        %-8u  %-8u    Octets\n",
                    stats.linkStats.sifOctRx, stats.linkStats.sifOctTx);
			printf( "SIO        %-8u  %-8u    Octets\n\n",
                    stats.linkStats.sioOctRx, stats.linkStats.sioOctTx);
            printf( "MSUs Dropped:      Congestion %5d    Total  %5d\n",
                    stats.linkStats.txCongDrop, stats.linkStats.txDrop);
            printf( "Transmit Queue:       Current %5d     High  %5d\n", stats.linkStats.tqCnt, stats.linkStats.hiTqCnt );
            printf( "Retransmit Queue:     Current %5d     High  %5d\n\n", stats.linkStats.rtqCnt, stats.linkStats.hiRtqCnt );
			printf( "Link Unavailable:     %10d secs   Link Congested:  %10d secs\n\n",
                    stats.linkStats.durLnkUnav / 10, stats.linkStats.durLnkCong / 10 );
            status = Mtp3GetLinkCfg( (U8) cp, &LinkCfg, entity );
            if (status == MTP3_SUCCESS)
            {
                if (LinkCfg.portType == PORT_TYPE_REMOTE)
                {
                    printf ("Remote Link.  MTP2 statistics unavailable from this board\n\n");
                    return;
                }
            }

            status = Mtp2LinkStats( (U8) cp, entity, &stats.link2Stats, reset );
            if( status != MTP2_SUCCESS )
            {
                Mtp2Geterror(status, &pErrstr);
                printf( "Board %d MTP2 Statistics Request failed: status = %s\n",
                        cp, pErrstr );
                return;
            }

            printf ("Press [Enter] for MTP2 statistics\n\n");
            getch();

            printf( "Board %d, Link %d MTP2 Stats:\n\n", cp, entity );

            printf ("inService  = %-8ld  lclBusy     = %-8ld  slFailAll   = %-8ld\n",
                stats.link2Stats.inService    , stats.link2Stats.lclBusy   , stats.link2Stats.slFailAll );
            printf ("slFailAb   = %-8ld  slFailAck   = %-8ld  slFailErr   = %-8ld\n",
                stats.link2Stats.slFailAb     , stats.link2Stats.slFailAck , stats.link2Stats.slFailErr );
            printf ("slFailCong = %-8ld  slFailAlign = %-8ld  slNSUErr    = %-8ld\n",
                stats.link2Stats.slFailCong   , stats.link2Stats.slFailAlign , stats.link2Stats.slNSUErr );
            printf ("slNegAck   = %-8ld  nRetrans    = %-8ld\n\n",
                stats.link2Stats.slNegAck     , stats.link2Stats.nRetrans );

			printf( "Msg:       Rx        Tx          Msg:       Rx        Tx\n" );
			printf( "----       --        --          ----       --        --\n" );
            printf( "SIF/SIO    %-8ld  %-8ld    MSU        %-8ld  %-8ld\n",
                    stats.link2Stats.nSifSioRx, stats.link2Stats.nSifSioTx,
                    stats.link2Stats.mRx, stats.link2Stats.mTx);
            printf( "LSSU*      %-8ld  %-8ld    FISU*      %-8ld  %-8ld\n\n",
                    stats.link2Stats.lRx, stats.link2Stats.lTx,
                    stats.link2Stats.fRx, stats.link2Stats.fTx);
            printf ("* LSSU & FISU counts do not include those filtered by the firmware\n\n");
            printf ("               Current    High\n");
            printf ("               -------    ----\n");
            printf (" Tx Queue:     %-8u   %-8u\n", stats.link2Stats.txCnt, stats.link2Stats.hiTxCnt);
            printf ("Rtb Queue:     %-8u   %-8u\n", stats.link2Stats.rtbCnt, stats.link2Stats.hiRtbCnt);
            printf ("lTx Queue:     %-8u   %-8u\n", stats.link2Stats.ltxCnt, stats.link2Stats.hiLtxCnt);
            printf ("Sap Queue:     %-8u   %-8u\n\n", stats.link2Stats.sapCnt, stats.link2Stats.hiSapCnt);

 			break;
        }

		case MTP3_ROUTE_STATS:
            printf( "Board %d, Route %s Stats:\n", cp,
                    CfgFmtPc( stats.routeStats.dpc, PCLEN_DEFAULT, pcDispFmt ) );
			printf( "Msg:       Rx        Tx          Msg:       Rx        Tx\n" );
			printf( "----       --        --          ----       --        --\n" );
			printf( "RST        %-8u  %-8u    RCT        %-8u  %-8u\n",
                    stats.routeStats.routeTstRx, stats.routeStats.routeTstTx,
                    stats.routeStats.congTstRx, stats.routeStats.congTstTx);
			printf( "TFP        %-8u  %-8u    TFA        %-8u  %-8u\n",
                    stats.routeStats.txProhibRx, stats.routeStats.txProhibTx,
                    stats.routeStats.txAllowRx, stats.routeStats.txAllowTx);
            printf( "TFC        %-8u\n\n", stats.routeStats.txRestrictRx);
            printf( "SIF Octets Tx: %d\n", stats.routeStats.sifOctTx);
            printf( "SIO Octets Tx: %d\n", stats.routeStats.sioOctTx);
            printf( "Route Unavailable Count:  %d\n\n", stats.routeStats.rteUnavCnt);
 			break;

		case MTP3_LNKSET_STATS:
			printf( "Board %d, Linkset %d Stats:\n", cp, entity );
			printf( "Linkset Failure  Start: %d  End: %d\n",
                    stats.linkSetStats.strtLnkSetFail,
                    stats.linkSetStats.stopLnkSetFail);
            printf( "Linkset Unavailable Duration: %d\n\n",
                    stats.linkSetStats.durLnkSetUnav);
			break;

		default:
			break;
	}
#endif      /* MTP2_ONLY */
}

void StatusAllLinks (void)
{
    short           i;
    short           status = MTP2_SUCCESS;
    char            text[10];
    short           portType, portNum;
    MTP2LinkStatus  link2Status;    /* MTP 2 Link status */
    MTP2LinkCfg     link2cfg;       /* MTP 2 Link config */
#ifndef MTP2_ONLY
    char            blkdText[4], inhbtText[4];
    MTP3LinkStatus  linkStatus;     /* MTP 3 Link status */
    MTP3LinkCfg     link3cfg;       /* MTP 3 Link config */

    printf ("Num    Name   MTP3 State     MTP2 Hi State   Low State           Block   Inhbt\n");
    printf ("---    ----   ----------     -------------   ---------           -----   -----\n");
#else
    printf ("Num    Name   MTP2 Hi State    MTP2 Low State\n");
    printf ("---    ----   -------------    --------------\n");
#endif
    for (i = 0; i < MTP_MAX_LINKS; i++)
    {
#ifndef MTP2_ONLY
        status = Mtp3LinkStatus( (U8) cp, i, &linkStatus );
#endif
        if (status == MTP2_SUCCESS)
        {
            status = Mtp2LinkStatus( (U8) cp, i, &link2Status );
            if (status == MTP2_SUCCESS)
            {
#ifndef MTP2_ONLY
                status = Mtp3GetLinkCfg( (U8) cp, &link3cfg, i );
                portType = link3cfg.portType;
                status = Mtp2GetLinkCfg( (U8) cp, &link2cfg, i );
                portNum = link2cfg.portNum;
#else
                status = Mtp2GetLinkCfg( (U8) cp, &link2cfg, i );
                portType = link2cfg.portType;
                portNum = link2cfg.portNum;
#endif
                if (status == MTP2_SUCCESS)
                {
                    /* Display configured link name, ie. S2, T1, etc. */
                    text[0] = (portType == PORT_TYPE_SERIAL) ? 'S' :
                        (portType == PORT_TYPE_TDM) ? 'T' : 'R';
                    if (portType != PORT_TYPE_REMOTE)
                        sprintf (&text[1], "%d", portNum);
                    else
                        strcpy (&text[1], "  ");
#ifndef MTP2_ONLY
                    /* Display blocked status */
                    strcpy (blkdText, "   ");       /* Clear initially */
                    if (linkStatus.locBlkd)
                        blkdText[0] = 'L';
                    if (linkStatus.rmtBlkd)
                        blkdText[2] = 'R';

                    /* Display inhibited status */
                    strcpy (inhbtText, "   ");       /* Clear initially */
                    if (linkStatus.locInhbt)
                        inhbtText[0] = 'L';
                    if (linkStatus.rmtInhbt)
                        inhbtText[2] = 'R';
#endif

#ifndef MTP2_ONLY
                    if (portType == PORT_TYPE_REMOTE)
                        printf ("%3d     %-4s  %-12s   REMOTE          REMOTE               %-3s     %3s\n", i, text,
                            getname( LinkStatesTbl, (short) linkStatus.state ), blkdText, inhbtText);
                    else
                        printf ("%3d     %-4s  %-12s   %-8s        %-17s    %-3s     %-3s\n", i, text,
                            getname( LinkStatesTbl, (short) linkStatus.state ),
                            hlStates[link2Status.hlSt], llStates[link2Status.llSt], blkdText, inhbtText);
#else
                    printf ("%3d     %-4s  %-8s         %-20s\n", i, text,
                        hlStates[link2Status.hlSt], llStates[link2Status.llSt]);
#endif
                }
                else
                {
                    continue;
                }
            }
            else
            {
                /* Since we no longer auto-configure all links, a remote link will be configured at layer 3 but
                 * not layer 2.  Assume this is a remote link */
                status = Mtp3GetLinkCfg( (U8) cp, &link3cfg, i );
                portType = link3cfg.portType;

                if (status == MTP2_SUCCESS)
                {
                    /* Display configured link name, ie. S2, T1, etc. */
                    text[0] = (portType == PORT_TYPE_SERIAL) ? 'S' :
                        (portType == PORT_TYPE_TDM) ? 'T' : 'R';
                    strcpy (&text[1], "  ");

                    /* Display blocked status */
                    strcpy (blkdText, "   ");       /* Clear initially */
                    if (linkStatus.locBlkd)
                        blkdText[0] = 'L';
                    if (linkStatus.rmtBlkd)
                        blkdText[2] = 'R';

                    /* Display inhibited status */
                    strcpy (inhbtText, "   ");       /* Clear initially */
                    if (linkStatus.locInhbt)
                        inhbtText[0] = 'L';
                    if (linkStatus.rmtInhbt)
                        inhbtText[2] = 'R';

                    if (portType == PORT_TYPE_REMOTE)
                    {
                        printf ("%3d     %-4s  %-12s   REMOTE          REMOTE               %-3s     %3s\n", i, text,
                            getname( LinkStatesTbl, (short) linkStatus.state ), blkdText, inhbtText);
                    }
                }
            }
        }
        else
        {
            continue;                   /* keep on checking rest of the links */
        }
    }

}

void HelpCommand( U32 *parms )
{
	printf( "%s", helpTbl[*parms] );
}

void setCP( U32 *parms )
{
    short       newcp, oldcp;
    MTP2_STATUS status;

#ifndef MTP2_ONLY
    MTP3GenCfg  genCfg;     /* mtp3 general config for retrieving PC display format */
#endif

    oldcp = cp;
	newcp = (short) parms[0];

	if( newcp < 1 || newcp > 8 )
	{
		printf( "CP must be in range 1..8\n" );
	}

#ifndef MTP2_ONLY
    /* open/init the MTP3 mgmt API for new board*/
    status = Mtp3MgmtInit( (U8) newcp, myEntId, myInstId  );
    if( status != MTP3_SUCCESS )
	{
		printf( "Board %d mtp3MgmtInit failed [%d]\n", newcp, status );
        return;
	}
	else
	{
		/* successful, close old cp & set new cp as the current cp */
        status = Mtp3MgmtTerm( (U8) oldcp );
		cp = newcp;
	}

    /* read MTP3 gen config to find the configured point code display format */
    status = Mtp3GetGenCfg( (U8) newcp, &genCfg );
    if (status == MTP3_SUCCESS)
    {
        pcDispFmt = genCfg.pcDispFmt;
    }
    else
    {
        Mtp2Geterror(status, &pErrstr);
        printf ("Warning: Unable to read MTP3 general configuration [%s]\n", pErrstr);
    }
#endif

    /* open/init the MTP2 mgmt API for new board*/
    status = Mtp2MgmtInit( (U8) newcp, myEntId, myInstId  );
    if( status != MTP2_SUCCESS )
	{
        printf( "Board %d mtp2MgmtInit failed [%d]\n", newcp, status );
	}
	else
	{
		/* successful, close old cp & set new cp as the current cp */
        status = Mtp2MgmtTerm( (U8) oldcp );
		cp = newcp;
	}
}

void quit( U32 *parms )
{
    UNREFERENCED_PARAMETER(parms);
    quittinTime = 1;
}


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
    U32                 params[MAX_PARMS];  /* numeric parameters array */
	short 				i, j;
	short				found;
	struct lookupEnt	*lp;
    short               retval;
#ifndef MTP2_ONLY                       /* Combined MTP2 & MTP3 version */
#else                                   /* MTP2 only version */
    unsigned int        scan_param;
#endif

	/* first search command table for command match */
	for( i = 0; CmdTable[i] != 0; i++ )
	{
		if( strcmp( tokens[0], CmdTable[i]->command ) == 0 )
		{
			break;		/* command matched */
		}
	}	/* end for */

	/* if no match found display error message and return */
	if( CmdTable[i] == 0 )
	{
		printf("Unrecognized command: %s\n", tokens[0] );
		return;
	}

    /* convert remaining tokens (i.e., parameters) to long integers,
     * using lookup tables where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
	for( j = 0; j < MAX_PARMS; j++ )
		params[j] = 0;					/* first initialize to defaults */

	for( j = 1; j < cnt; j++, lp++ )
	{
		lp = CmdTable[i]->parmTbl[j - 1];
		if( lp )
		{	/* lookup table for this param ? */
			found = getval( lp, tokens[j], &params[j - 1] );
			if( found )
			{
				continue;				/* parameter succesfully converted */
			}
        }

#ifndef MTP2_ONLY                       /* Combined MTP2 & MTP3 version */
        /* no lookup table or lookup failed - see if this is a point code in x.y.z format
         * or a number in hex or decimal
         */
        retval = CfgScanPc( tokens[j], &params[j - 1], pcDispFmt );
        if( retval == 0 )
        {
            /* successful conversion */
            continue;
        }
        else if( retval == -2 )
        {
            printf("Point code: %s out of range\n", tokens[j] );
			return;
        }
        else
#else                                   /* MTP2 only version */
        /* no lookup table - try to convert as a number */
        retval = sscanf( tokens[j], "%i", &scan_param );
        params[j - 1] = (U32)scan_param;
        if (retval != 1)
#endif
        {
            /* If it's not a number, see if it's an asterisk */
            if (*tokens[j] == '*')
                {
                params[j-1] = ALL_LINKS;            /* Unique value to denote '*' */
                continue;
                }

			/* conversion failed */
			printf("Unrecognized parameter: %s\n", tokens[j] );
			return;
		}
	}	/* end for */

	/* call command handler function */

	if( CmdTable[i]->func != 0 )
	{
		CmdTable[i]->func( params );
	}
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}

void usage( char *pName )
{
    printf("\n usage: %s [ -b <board>] \n", pName );
    printf("\n\t<board>\tboard number to attach to [default 1]\n");
	exit(1);
}

int main( int argc, char *argv[] )
{

	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	short	cnt;
	short	i;
	char	*p;
    MTP2_STATUS status;
#ifndef MTP2_ONLY
    MTP3GenCfg  genCfg;     /* mtp3 general config for retrieving PC display format */
#endif

#ifndef MTP2_ONLY
    printf("\n\n%s: sample MTP3 management application version %s %s\n\n",
            argv[0], MTP3MGR_VERSION, __DATE__);
#else
    printf("\n\n%s: sample MTP2 management application version %s %s\n\n",
            argv[0], MTP2MGR_VERSION, __DATE__);
#endif

	/* parse command line for interrupt & cp # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
			case 'b' :    /* Set Board number */
				i++;
				if((argv[i] == NULL) || (sscanf(argv[i], "%hi", &cp) != 1) || (cp < 1) || (cp > 8) )
				{
                    usage( argv[0] );
				}
				break;

            case 'd' :      /* Set debug mode on */
                bDebug = TRUE;
                break;

			default:
                usage( argv[0] );
				break;
			}	/* end switch */
		}
		else
		{
            usage( argv[0] );
		}

	}	/* end for */

#ifndef MTP2_ONLY
    /* open/init the MTP3 mgmt API */
    status = Mtp3MgmtInit( (U8) cp, myEntId, myInstId  );
    if( status != MTP3_SUCCESS )
	{
		printf( "mtp3MgmtInit failed [%d]\n", status );
		exit(1);
	}

    /* read MTP3 gen config to find the configured point code display format */
    status = Mtp3GetGenCfg( (U8) cp, &genCfg );
    if (status == MTP3_SUCCESS)
    {
        pcDispFmt = genCfg.pcDispFmt;
    }
    else
    {
        Mtp2Geterror(status, &pErrstr);
        printf ("Warning: Unable to read MTP3 general configuration [%s]\n", pErrstr);
    }
#endif

    /* open/init the MTP2 mgmt API */
    status = Mtp2MgmtInit( (U8) cp, myEntId, myInstId  );
    if( status != MTP2_SUCCESS )
	{
        printf( "mtp2MgmtInit failed [%d]\n", status );
		exit(1);
	}

	/* command processing loop */
#ifndef MTP2_ONLY
    printf( "mtpmgr[%-1u]>", cp );
#else
    printf( "mtp2mgr[%-1u]>", cp );
#endif
    while( (quittinTime == 0) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
		cnt = tokenize( buffer, tokens );
		if( cnt > 0 )
			processCmd( tokens, cnt );
#ifndef MTP2_ONLY
            printf( "mtpmgr[%-1u]>", cp );
#else
            printf( "mtp2mgr[%-1u]>", cp );
#endif
	}	/* end while */

#ifndef MTP2_ONLY
    status = Mtp3MgmtTerm( (U8) cp );
#endif
    status = Mtp2MgmtTerm( (U8) cp );
}	/* end main() */

