/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1998 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/* --------------------------------------------------------------------------
 * sccpcfg.c
 *
 * Scans an ASCII text configuration file and downloads configuration to
 * SCCP task on specified board.
 *
 * $Log: /winnt/ss7/sccp/sccpcfg/sccpcfg.c $
*
* 14    2/01/99 4:34p Tedd
* Make changes to print different point code formats. n.n.n notation
* should now print correctly for ITU point codes.
*
* 13    1/08/99 10:15a Tedd
* Fixed a problem with ANSI point code checking. Code didn't allow the
* full range of addresses.
* Allowed Address Masks to be zero(no 0xF required).
*
* 12    8/07/98 1:46p Dodren
* Completely new version of SCCPCFG.C, which uses the SCCP Management
* API.
 * $Nokeywords$
 * -------------------------------------------------------------------------- */

/*
 * Things we don't check for:
 * 1. Two routes with the same route number.
 *    (ROUTE 1.0.1 ... END; ROUTE 1.0.1 ... END)
 * 2. Two identical address configurations.
 */

/*
 * The following pragma can be used with Microsoft's
 * Visual C++ compiler to eliminate warning messages:
 *
 *      #pragma warning( disable: 4514 )
 */

#if defined( UNIX )                     /* System header files for UNIX       */

#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <sys/types.h>
/* #include <sys/stream.h> */
#include <stdlib.h>
#include <ctype.h>
#define TRUE    1                       /* TRUE/FALSE need defining for UNIX  */
#define FALSE   0

#elif defined( __OS2__ )                /* System header files for OS/2       */

#include <stdio.h>
#include <stddef.h>
#include <ctype.h>
#include <string.h>

#elif defined( WIN32 )                  /* System header files Windows NT     */

#include <stdio.h>
#include <stddef.h>

#endif
#include <txnuser.h>
#include "sccpmapi.h"   /* SCCP Management API interfce                       */
#include "sccpver.h"    /* Version information (SCCPxxxx                      */
#include "cfgscan.h"    /* For initcfg(), endcfg(), getcfgval()               */

/* --------------------------------------------------------------------------
 * Macro Definitions
 * -------------------------------------------------------------------------- */
#define SCCPWRN             0x01
#define SCCPERR             0x02

#define SCCP_DBG_USER       0x0001  /* Turns on helpful messages for user     */
#define SCCP_DBG_CFG        0x0002  /* Turns on configuration debugging msgs  */
#define SCCP_DBG_SCAN       0x0004  /* Turns on scanning debugging messages   */

#define SCCP_DBG_DUMP       0x0100  /* Turns on hex dumps                     */
#define SCCP_DBG_NXMT       0x0200  /* Disables actual transmission           */

#define LENU32              11      /* Str len for U32 (10 digits + '\0' )    */
#define LENU16              6       /* Str len for U16 (5 digits + '\0')      */
#define LENU8               4       /* Str len for U8 (3 digits + '\0')       */
#define LENPC               27      /* Str len pc: **xxx.xxx.xxx [0xhhhhhhhh] */
                                    /*    (note: 2 extra for when pc > PCMAX) */

static char *StatusName[] = {
    "SCCP_SUCCESS",
    "SCCP_OVERRUN",
    "SCCP_UNDERRUN",
    "SCCP_UNBOUND",
    "SCCP_BOARD",
    "SCCP_DRIVER",
    "SCCP_NOMSG",
    "SCCP_NOTBOUND",
    "SCCP_RESOURCES",
    "SCCP_UNINIT",
    "SCCP_TOOMANY",
    "SCCP_INSTANCE",
    "SCCP_TIMEOUT",
    "SCCP_SWTYPE",
    "SCCP_PARAM",
    "SCCP_NOGENCFG",
    "SCCP_NOTFOUND",
    "SCCP_EXCEEDMAXCFG",
    "SCCP_INTERNAL",
    "SCCP_BUFLEN",
    "SCCP_BADDIGIT",
    "SCCP_MAXROUTES",
    "SCCP_NULLPTR",
    "SCCP_TIMERVALUE",
    "SCCP_HOPCOUNT",
    "SCCP_POINTCODE",
    "SCCP_RANGE",
    "<unknown>"
};
#define OFFSET_TO_UNKNOWN   0x1b

/* Note that this macro could have possible side effects */
#define STA2STR( status )       StatusName[                                 \
    ((status) == 0) ||                                                      \
    (((status) >= 0x1001) && ((status) <= 0x1000 + OFFSET_TO_UNKNOWN - 1))  \
    ? (status ) & 0x00ff : OFFSET_TO_UNKNOWN ]

/* Warning/Error message text:                                                */
static char *FmtBanner      = "\nTX Series SCCP Configuration Utility   Version %s %s\n";
static char *FmtUsage       = "\nUsage: sccpcfg [ -d[mask]] [ -b <board number> ] [-f <filename>]\n";
static char *FmtBoard       = "Error, board number must be between 1 and %d inclusive.\n";
static char *FmtMgmtErr     = "Error, %s [%#6x] returned by %s.\n";

static char *FmtSecAdrLen   = "Warning, %s exceeds %d characters, section ignored.\n";
static char *FmtSecAdrSyntax= "Warning, %s contains invalid digits, section ignored.\n";
static char *FmtSecEnd      = "Error, %s is missing an END statement.\n";
static char *FmtSecEndPst   = "Warning, %s definition %s %lu found after end of section, value ignored.\n";
static char *FmtSecMax      = "Warning, %s exceeds maximum [%u], section ignored.\n";
static char *FmtSecOor      = "Warning, %s out of range, section ignored.\n";
static char *FmtSecScnErrLp = "Error, %s syntax error, cannot continue.\n";

static char *FmtDefAdrNul   = "Warning, %s definition %s is all zeros, definition ignored.\n";
static char *FmtDefAdrSyn   = "Warning, %s definition %s contains invalid digits, definition ignored.\n";
static char *FmtDefInvIS    = "Warning, %s definition %s %s is invalid, definition ignored.\n";
static char *FmtDefInvDS    = "Warning, %s definition %s %s is invalid, defaulting to %s.\n";
static char *FmtDefLen      = "Warning, %s definition %s is too long, truncating to first %d characters.\n";
static char *FmtDefMax      = "Warning, %s definition %s %s exceeds maximum [%u], definition ignored.\n";
static char *FmtDefOorIN    = "Warning, %s definition %s %lu is out of range, definition ignored.\n";
static char *FmtDefOorIS    = "Warning, %s definition %s %s is out of range, definition ignored.\n";
static char *FmtDefOorDN    = "Warning, %s definition %s %lu is out of range, defaulting to %d.\n";
static char *FmtDefScnErr   = "Warning, %s definition %s syntax error, definition ignored.\n";
static char *FmtDefScnErrLp = "Warning, %s definition %s, one or more definitions ignored due to prior syntax error.\n";

static char *FmtReplFalse   = "Warning, %s definition %s is inapplicable when REPLACE_GLT is FALSE, definition ignored.\n";
static char *FmtReplFormat  = "Warning, %s definition %s is inapplicable for GT_FORMAT %u, definition ignored.\n";

static char *FmtDpc         = "Error, %s requires a DPC, section ignored.\n";
static char *FmtDpcLen      = "Warning, %s definition DPC_LEN %lu ignored, value must be 14 or 24.\n";
static char *FmtGtFormat    = "Error, %s GT_FORMAT %lu requires an outgoing global title, section ignored.\n";

static char *FmtNone        = "<None>\n";
static char *FmtNotUsed     = "<Not used>\n";

static char *FmtTrue        = "TRUE";
static char *FmtFalse       = "FALSE";

#define PRT_BANNER              { printf( FmtBanner, SCCPCFG_VERSION, __DATE__ ); }
#define PRT_USAGE               { fprintf( stderr, FmtUsage ); }
#define PRT_BOARD               { fprintf( stderr, FmtBoard, MAXBOARD ); }
#define PRT_MGMTERR( n, f )     { fprintf( stderr, FmtMgmtErr, STA2STR( n ), n, f ); }

#define PRT_SEC_ADRLEN( s, l )  { fprintf( stderr, FmtSecAdrLen, s, l ); }
#define PRT_SEC_ADRSYNTAX( s )  { fprintf( stderr, FmtSecAdrSyntax, s ); }
#define PRT_SEC_END( s )        { fprintf( stderr, FmtSecEnd, s ); }
#define PRT_SEC_ENDPST(s,d,n)   { fprintf( stderr, FmtSecEndPst, s, d, n ); }
#define PRT_SEC_MAX( s, n )     { fprintf( stderr, FmtSecMax, s, n ); }
#define PRT_SEC_OOR( s )        { fprintf( stderr, FmtSecOor, s ); }
#define PRT_SEC_SCANERRLP( s )  { fprintf( stderr, FmtSecScnErrLp, s ); }

#define PRT_DEF_ADRNUL( s, d )  { fprintf( stderr, FmtDefAdrNul, s, d ); }
#define PRT_DEF_ADRSYN( s, d )  { fprintf( stderr, FmtDefAdrSyn, s, d ); }
#define PRT_DEF_INV_IS(s,d,n)   { fprintf( stderr, FmtDefInvIS, s, d, n ); }
#define PRT_DEF_INV_DS(s,d,n,m) { fprintf( stderr, FmtDefInvDS, s, d, n, m ); }
#define PRT_DEF_LEN( s, d, n )  { fprintf( stderr, FmtDefLen, s, d, n ); }
#define PRT_DEF_MAX(s,d,n,m)    { fprintf( stderr, FmtDefMax, s, d, n, m ); }
#define PRT_DEF_OOR_IN(s,d,n)   { fprintf( stderr, FmtDefOorIN, s, d, n ); }
#define PRT_DEF_OOR_IS(s,d,v)   { fprintf( stderr, FmtDefOorIS, s, d, v ); }
#define PRT_DEF_OOR_DN(s,d,n,m) { fprintf( stderr, FmtDefOorDN, s, d, n, m ); }
#define PRT_DEF_SCANERR( s, d ) { fprintf( stderr, FmtDefScnErr, s, d ); }
#define PRT_DEF_SCANERRLP(s,d)  { fprintf( stderr, FmtDefScnErrLp, s, d ); }

#define PRT_REPL_FALSE(s,d)     { fprintf( stderr, FmtReplFalse, s, d ); }
#define PRT_REPL_FORMAT(s,d,v)  { fprintf( stderr, FmtReplFormat, s, d, v ); }

#define PRT_DPC( s )            { fprintf( stderr, FmtDpc, s ); }
#define PRT_DPCLEN( s, n )      { fprintf( stderr, FmtDpcLen, s, n ); }
#define PRT_GTFORMAT( s, n )    { fprintf( stderr, FmtGtFormat, s, n ); }

#define RETURN_SCCPERR( stmt )  { stmt; return( SCCPERR ); }

/* Note that the following macro could have side effects */
#define BCD_BYTES( nDigits )    ( ( ( nDigits ) /2 ) + ( ( nDigits ) % 2 ) )

#define CHK_POS( sId, pos )                                                 \
{                                                                           \
    if ( ( ( pos ) > 0L ) && ( ( pos ) < EndGenCfg ) ) {                    \
       RETURN_SCCPERR( PRT_SEC_END( sId ) );                                \
    }                                                                       \
}

#define OK_TO_TRANSMIT      ( !( DbgFlgsXmt & SCCP_DBG_NXMT ) )

/* Note that the following macro could have side effects */
#define PCMAX( type )                                                          \
( (U32) ( ( type == PCLEN_ANSI ) ? 0xffffff : ( ( type == PCLEN_ITU ) ? 0x3fff : 0 ) ) )

#define PC1( pc )             ( ( (U32) ( pc ) ) >> 16 )
#define PC2( pc )           ( ( ( (U32) ( pc ) ) >> 8  ) & 0xff )
#define PC3( pc )             ( ( (U32) ( pc ) )         & 0xff )

/* TRP modify to handle different display formats */
#define PCPRTF( stream, pc )                                                    \
{                                                                               \
    U32 _pc = (U32) ( pc );                                                     \
    char *fmt;                                                                  \
    fmt = CfgFmtPc(_pc, pcLen, pcDispFmt);                                      \
    fprintf( stream, fmt );                                                     \
}

#define STRPCPRTF( str, pc )                                                    \
{                                                                               \
    U32 _pc = (U32) ( pc );                                                     \
    char *fmt;                                                                  \
    fmt = CfgFmtPc(_pc, pcLen, pcDispFmt);                                      \
    sprintf( str, fmt );                                                     \
}

/*    fprintf( stream, "%d.%d.%d [%#lx]\n", PC1(_pc), PC2(_pc), PC3(_pc), _pc );  */

/* TRP modify to handle different display formats */
#define PCPRTS( dest, pc )                                                      \
{                                                                               \
    U32     _pc = (U32) ( pc );                                                 \
    char *fmt;                                                                  \
    fmt = CfgFmtPc (_pc, pcLen, pcDispFmt);                                     \
    strcpy(dest,fmt);                                                           \
}
    /*sprintf( dest,   "%d.%d.%d [%#1x]",   PC1(_pc), PC2(_pc), PC3(_pc), _pc );   */

#define WARN_PC_OOR( sId, parm, pc, flg )                                       \
{                                                                               \
    char _str[LENPC];                                                           \
    PCPRTS( _str, ( pc ) );                                                     \
    PRT_DEF_OOR_IS( sId, parm, _str );                                          \
    flg |= SCCPWRN;                                                             \
}

#define WARN_PC_TOOMANY( sId, parm, pc, flg )                                  \
{                                                                              \
    char _str[LENPC];                                                          \
    PCPRTS( _str, pc );                                                        \
    PRT_DEF_MAX( sId, parm, _str, SCCP_MAXCONPC );                             \
    flg |= SCCPWRN;                                                            \
}

#define WARN_DEFINV_DEFS( sId, parm, str, def, flg )                           \
{                                                                              \
    PRT_DEF_INV_DS( sId, parm, str, def );                                     \
    flg |= SCCPWRN;                                                            \
}


/*
 * ANSI supports Type 1 and Type 2 global titles.  However, Type 1 global
 * titles are stored in the GT_FORMAT 3 section of the SccpGlbTitleCfg gt
 * union.  This macro returns the format used for accessing the union.
 */
#define GT_UNION_FMT( pAdr )                                                \
    ((U8)((((pAdr)->gt.format == (U8)1) &&((pAdr)->swType == SCCP_SW_ANS))  \
    ? 3 : (pAdr)->gt.format))

/*
 * Handy macros to get a boolean value.  If formatted OK, 'dest' is set to
 * the retrieved value.  GETCHK_BG is for the General Configuration Parameters
 * section.  GETCHK_B is for all other sections.
 */
#define GETCHK_B( sId, keywd, dest, flg, from )                         \
{                                                                       \
    INT32 _val;                                                          \
    INT32 _fp = getcfgval( keywd, CFGT_BOOL, &_val, from );              \
    if ( _fp == 0L ) {                                                  \
        ;                                                               \
    } else if ( _fp < 0L ) {                                            \
        PRT_DEF_SCANERR( sId, keywd );                                  \
        flg |= SCCPWRN;                                                 \
    } else {                                                            \
        dest = (Bool) _val;                                             \
    }                                                                   \
}

#define GETCHK_BG( sId, keywd, dest, flg )                              \
{                                                                       \
    INT32 _val;                                                          \
    INT32 _fp = getcfgval( keywd, CFGT_BOOL, &_val, 0L );                \
    if ( _fp == 0L ) {                                                  \
        ;                                                               \
    } else if ( _fp < 0L ) {                                            \
        PRT_DEF_SCANERR( sId, keywd );                                  \
        flg |= SCCPWRN;                                                 \
    } else if ( _fp > EndGenCfg ) {                                     \
        PRT_SEC_ENDPST( sId, keywd, _val );                             \
        flg |= SCCPWRN;                                                 \
    } else {                                                            \
        dest = (Bool) _val;                                             \
    }                                                                   \
}

/*
 * Handy macros to get & range check a long value.  If out of range, a warning
 * message is printed and "dest" is not changed.  Otherwise, 'dest' is set to
 * the retrieved value.  GETCHK_IG is for the General Configuration Parameters
 * section.  GETCHK_I is for all other sections.
 */
#define GETCHK_IG( sId, keywd, min, max, dest, cast, flg )              \
{                                                                       \
    INT32 _val;                                                          \
    INT32 _fp = getcfgval( keywd, CFGT_INT, &_val, 0L );                 \
    if ( _fp == 0 ) {                                                   \
        ;                                                               \
    } else if ( _fp < 0L ) {                                            \
        PRT_DEF_SCANERR( sId, keywd );                                  \
        flg |= SCCPWRN;                                                 \
    } else if ( _fp > EndGenCfg ) {                                     \
        PRT_SEC_ENDPST( sId, keywd, _val );                             \
        flg |= SCCPWRN;                                                 \
    } else if ( ( _val < ( min ) ) || ( _val > ( max ) ) ) {            \
        PRT_DEF_OOR_DN( sId, keywd, _val, dest );                       \
        flg |= SCCPWRN;                                                 \
    } else {                                                            \
        dest = (cast) _val;                                             \
    }                                                                   \
}

#define GETCHK_I( sId, keywd, min, max, dest, cast, flg, from )         \
{                                                                       \
    INT32 _val;                                                          \
    INT32 _fp = getcfgval( keywd, CFGT_INT, &_val, from );               \
    if ( _fp == 0L ) {                                                  \
        ;                                                               \
    } else if ( _fp < 0L ) {                                            \
        PRT_DEF_SCANERR( sId, keywd );                                  \
        flg |= SCCPWRN;                                                 \
    } else if ( ( _val < ( min ) ) || ( _val > ( max ) ) ) {            \
        PRT_DEF_OOR_DN( sId, keywd, _val, dest );                       \
        flg |= SCCPWRN;                                                 \
    } else {                                                            \
        dest = (cast) _val;                                             \
    }                                                                   \
}

/*
 * This macro seeks to the end of the current section and updates 'next';
 * or warns the user if "END" is missing and sets 'next' to 'from'.
 */
#define SEEK_TO_EOS( sId, from, next )                                  \
{                                                                       \
    INT32 _pos, _value;                                                  \
    if ( ( _pos = getcfgval( "END", CFGT_INT, &_value, from ) ) > 0L ) {\
        next = _pos;                                                    \
    } else {                                                            \
        if ( DbgFlgsUsr ) fprintf( stderr, "\n" );                      \
        RETURN_SCCPERR( PRT_SEC_END( sId ) );                            \
    }                                                                   \
}

#define SKIP_SECTION( sId, pos, stmt )                                  \
{                                                                       \
    stmt;                                                               \
    SEEK_TO_EOS( sId, pos, pos );                                       \
    continue;                                                           \
}


/*
 * Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER( sId, keywd, timer, flg )                             \
{                                                                       \
    INT32 _val;                                                          \
    INT32 _fp = getcfgval( keywd, CFGT_INT, &_val, 0L );                 \
    if ( _fp == 0L ) {                                                  \
        ;                                                               \
    } else if ( _fp < 0L ) {                                            \
        flg = TRUE;                                                     \
        PRT_DEF_SCANERR( sId, keywd );                                  \
    } else if ( _fp > EndGenCfg ) {                                     \
        flg = TRUE;                                                     \
        PRT_SEC_ENDPST( sId, keywd, _val );                             \
    } else {                                                            \
        timer.value = (U16) _val;                                       \
        timer.enable = (U8) ( ( _val != 0L ) ? 1 : 0 );                 \
    }                                                                   \
}

#define TIMER_VALUE( ptr )          ((U16)(((ptr)->enable) ? (ptr)->value : 0 ))

#define PRT_TIMERCFG( name, ptr )                               \
{                                                               \
    printf( "    %s.enable = %u\n", name, (ptr)->enable );      \
    if ( (ptr)->enable ) {                                      \
        printf( "    %s.value  = %u\n", name, (ptr)->value  );  \
    }                                                           \
}

/* --------------------------------------------------------------------------
 * Global Variables
 * -------------------------------------------------------------------------- */

static char *DecDigits      = "0123456789";
static char *MskDigits      = "0Ff";

static U16 DbgFlgsUsr = 0;              /* User Debugging flags               */
static U16 DbgFlgsXmt = 0;              /* Transmit Flags                     */
static char CfgFile[80] = "sccp.cfg";   /* Configuration file name            */
DllImport int bSccpMgmtDebug;

/* Limits from General Configuration section:                                 */

static U16 MaxUSaps;                    /* Max nbr user (upper) SAPs          */
static U16 MaxNSaps;                    /* Max nbr network (lower) SAPs       */
static U16 MaxRoutes;                   /* Max nbr of Routes                  */
static U16 MaxAddrs;                    /* Max nbr of Addresses               */
static INT32 EndGenCfg;                  /* Position of End of Gen Cfg Section */

/* Add Point Code type for printing:                                          */
static U8 pcLen;                       /* Point Code length                   */
static U8 pcDispFmt;                   /* Point Code Display Format          */

/* --------------------------------------------------------------------------
 * Function Definitions
 * -------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------
 * getVal
 *   Interface to getcfgval() that reports error conditions.  This allows the
 *   user to be notified when getcfgval() returns a negztive number.  In the
 *   configuration routine that this code replaces, no distinction was made
 *   between an error and a keywork simply not being present.
 *
 * Parameters:
 *   pSecId     Pointer to a string that uniquely identifies the section.
 *   pKeywd     Pointer to the keyword to search for.
 *   nType      Type of search (CFGT_INT, etc.).
 *   pValue     Address where value associated with keywork is returned.
 *   fpStrt     File position from which to begin search
 *   pFlags     Address of error flags bit field.
 *
 * Returns:
 *   1      Keyword found.
 *   0      No keyword present.
 *  -1      Error(s) detected.
 * -------------------------------------------------------------------------- */
static int getVal( char *pSecId, char *pKeywd, int nType, void *pValue, INT32 fpStrt, U16 *pFlags )
{
    INT32 fpNext = getcfgval( pKeywd, nType, pValue, fpStrt );
    if ( fpNext == 0L ) {
        return( 0 );
    } else if ( fpNext > 0L ) {
        return( 1 );
    } else {
        *pFlags |= SCCPWRN;
        PRT_DEF_SCANERR( pSecId, pKeywd );
        return( -1 );
    }
}

/* --------------------------------------------------------------------------
 * getAddr
 *   Scans the current section of the configuration file for an optional
 *   ADDR_MASK or GL_TITLE.
 *
 * Parameters:
 *   pSecId     Pointer to a string that uniquely identifies the section.
 *   pDef       Pointer to a string containing the definition sought.
 *   fpStrt     Position in configuration file from which to start scan.
 *   pAdr       Address of an SccpAddrName/SccpAddrMask structure.
 *   pnDigits   Number of digits in address.
 *   pFlags     Address of error flags bit field.
 *
 * Returns:
 *   1      Keyword found.
 *   0      No keyword present.
 *  -1      Error(s) detected.
 * -------------------------------------------------------------------------- */
static int getAddr( char *pSecId, char *pDef, INT32 fpStrt, SccpAddrMask *pAdr, U32 *pnDigits, U16 *pFlags )
{
    char        buf[CFGT_MAXSTR];   /* Holds string returned by getcfgval     */
    size_t      len;
    /* char        *p; */
    int         rtn;
    SCCP_STATUS status = -1;


    pAdr->length = 0;

    if ( fpStrt < 0L ) {
        *pFlags |= SCCPERR;
        return( -1 );
    }

    rtn = getVal( pSecId, pDef, CFGT_STRING, buf, fpStrt, pFlags );
    if ( rtn == 0 ) {
        return( 0 );
    } else if ( rtn < 0 ) {
        /* *pFlags modified by getVal() */
        return( -1 );
    }

    /* Check for a string that is too long. */
    len = strlen( buf );
    if ( len > SCCP_LENADDR * 2 ) {
        PRT_DEF_LEN( pSecId, pDef, SCCP_LENADDR * 2 );
        *pFlags |= SCCPWRN;
        len = SCCP_LENADDR * 2;
        buf[len] = '\0';
    }
	/* TRP - Patch 5365 - do not strip trailing zeros from the GL_TITLE entry in the ADDRESS section
	 * GL_TITLE is the only parameter that uses the "getAddr" routine */
    /* Strip trailing zeros (this breaks routing). */
    /*p = &buf[ ( len > 1 ) ? len - 1 : 0 ];
    while ( ( p >= buf ) && ( *p == '0' ) ) {
        *p-- = '\0';
    } */

    if ( ( len = strlen( buf ) ) == 0L ) {
        buf[0] = 0x30;                  /* allow an ADDR_MASK of "0", len = 1 */
        len = 1;
   /*      Zero length address
        PRT_DEF_ADRNUL( pSecId, pDef );
        *pFlags |= SCCPWRN;
        return( -1 ); */
    }

    /* Convert addresses to BCD. */
    if ( strcmp( pDef, "ADDR_MASK" ) == 0 ) {
        status = SccpAsciiMaskToBcd( buf, pAdr->strg, SCCP_LENADDR, pnDigits );
    } else if ( strcmp( pDef, "GL_TITLE"  ) == 0 ) {
        status = SccpAsciiNumToBcd( buf, pAdr->strg, SCCP_LENADDR, pnDigits );
    }

    if ( status != SCCP_SUCCESS ) {
        PRT_DEF_ADRSYN( pSecId, pDef );
        *pFlags |= SCCPWRN;
        return( -1 );
    }

    pAdr->length = (U8) BCD_BYTES( len );

    return( 1 );
} /* End of getAddr */

/* --------------------------------------------------------------------------
 * getAddrMasks
 *   Scans the current section of the configuration file for an optional
 *   ADDR_MASK.
 *
 * Parameters:
 *   pSecId     Pointer to a string that uniquely identifies the section.
 *   pDef       Pointer to a string containing the definition sought.
 *   fpStrt     Position in configuration file from which to start scan.
 *   pAdr       Address of an SccpAddrName/SccpAddrMask structure.
 *   pnDigits   Number of digits in address.
 *   pFlags     Address of error flags bit field.
 *
 * Returns:
 *          number of address masks parsed.
 * -------------------------------------------------------------------------- */
static U8 getAddrMasks( char *pSecId, char *pDef, INT32 fpStrt, SccpAddrMask *pAdr, U32 *pnDigits, U16 *pFlags )
{
    char        buf[CFGT_MAXSTR];   /* Holds string returned by getcfgval     */
    size_t      len;
    char        *p;
    U8          num_masks;
    INT32       fpNext;
    SCCP_STATUS status;


    num_masks = 0;

    pAdr->length = 0;

    if ( fpStrt < 0L ) {
        *pFlags |= SCCPERR;
        return( 0 );
    }

    while ((fpNext = getcfgval( pDef, CFGT_STRING, buf, fpStrt )) > 0L)
    {
        /* Check for a string that is too long. */
        len = strlen( buf );
        if ( len > SCCP_LENADDR * 2 ) {
            PRT_DEF_LEN( pSecId, pDef, SCCP_LENADDR * 2 );
            *pFlags |= SCCPWRN;
            len = SCCP_LENADDR * 2;
            buf[len] = '\0';
        }

        /* Strip trailing zeros (this breaks routing). */
        p = &buf[ ( len > 1 ) ? len - 1 : 0 ];
        while ( ( p >= buf ) && ( *p == '0' ) ) {
            *p-- = '\0';
        }

        if ( ( len = strlen( buf ) ) == 0L ) {
            buf[0] = 0x30;                  /* allow an ADDR_MASK of "0", len = 1 */
            len = 1;
    /*      Zero length address
            PRT_DEF_ADRNUL( pSecId, pDef );
            *pFlags |= SCCPWRN;
            return( 0 ); */
        }

        /* Convert addresses to BCD. */
        status = SccpAsciiMaskToBcd( buf, pAdr->strg, SCCP_LENADDR, pnDigits );

        if ( status != SCCP_SUCCESS ) {
            PRT_DEF_ADRSYN( pSecId, pDef );
            *pFlags |= SCCPWRN;
            return( 0 );
        }

        pAdr->length = (U8) BCD_BYTES( len );

        pAdr++;
        num_masks++;
        fpStrt = fpNext;

    } /* end while */

    if ( fpNext < 0L)
    {
        PRT_DEF_SCANERR(pSecId, pDef);
        *pFlags |= SCCPWRN;
        return( 0 );
    }

    return( num_masks );
} /* End of getAddrMasks */

/* --------------------------------------------------------------------------
 * scanConPCs
 *   Build a list of concerned point codes by scanning the current section
 *   of the configuration file.
 *
 * Parameters:
 *   pCpc       Pointer to start of list where concerned point codes are placed
 *   pNbrCPC    Address where number of CPCs scanned is returned
 *   fpStrt     Position in configuration file from which to start scan
 *   pSecId     Pointer to a string that uniquely identifies the section
 *   lenPc      Maximum length of point code (PCLEN_ANSI or PCLEN_ITU)
 *
 * Returns:
 *   0          Success
 * or bitfiled of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 scanConPCs( U32 *pCpc, U16 *pNbrCPC, INT32 fpStrt, char* pSecId, U8 lenPc )
{
    U16      flags = 0;
    INT32    fpNext;
    U32      pc;

    /* Validate file position pointer */
    if ( fpStrt < 0L ) {
        return( SCCPERR );
    }

    /* Scan for up to SCCP_MAXCONPC concerned point codes */
    *pNbrCPC = 0;
    while ( ( fpNext = getcfgval( "CONC_PC", CFGT_PC, &pc, fpStrt ) ) > 0L ) {
        if ( *pNbrCPC >= SCCP_MAXCONPC ) {
            /* Number of CONC_PC exceeds array size. */
            WARN_PC_TOOMANY( pSecId, "CONC_PC", pc, flags );
        } else if  ( ( pc == 0L ) || ( pc > PCMAX( lenPc ) ) ) {
            /* The point code is either 0 or too large. */
            WARN_PC_OOR( pSecId, "CONC_PC", pc, flags );
        } else {
            (*pNbrCPC)++;
            *pCpc++ = pc;
        }
        fpStrt = fpNext;
    } /* while */

    if ( fpNext < 0L  ) {
        PRT_DEF_SCANERRLP( pSecId, "CONC_PC" );
        flags |= SCCPWRN;
    }

    return( flags );
} /* End of scanConPCs */

/* --------------------------------------------------------------------------
 * scanUsapAltOPCs
 *   Build a list of alternate point codes by scanning the current section
 *   of the configuration file.
 *
 * Parameters:
 *   pOpcInfo   Pointer to start of list where alternate point codes are placed
 *   pNbrOPC    Address where number of OPCs scanned is returned
 *   pfp        Position in configuration file from which to start scan
 *   pSecId     Pointer to a string that uniquely identifies the section
 *   lenPc      Maximum length of point code (PCLEN_ANSI or PCLEN_ITU)
 *
 * Returns:
 *   0          Success
 * or bitfiled of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 scanUsapAltOPCs( UsapOpcInfo *pOpcInfo, U16 *pNbrOPC, INT32 *pfp, char* pSecId, U8 lenPc )
{
    U16         flags = 0;
    INT32       fpNext;
    U32         pc;
    char        opcId[256];
    size_t      len;

    /* Validate file position pointer. */
    if ( *pfp < 0L ) {
        return( SCCPERR );
    }

    strcpy(opcId, pSecId);
    strcat(opcId, " ALT_OPC ");
    len = strlen(opcId);

    *pNbrOPC = 0;
    while( ( fpNext = getcfgval( "ALT_OPC", CFGT_PC, &pc, *pfp ) ) != 0 ) {
        /* Check for file pointer error. */
        if ( fpNext < 0L ) {
            flags |= SCCPWRN;
            SKIP_SECTION( opcId, *pfp, PRT_DEF_SCANERR( opcId, "ALT_OPC" ) );
        }

        if ( *pNbrOPC >= SCCP_MAXALTOPC ) {
            /* Number of ALT_OPC exceeds array size. */
            flags |= SCCPWRN;
            SKIP_SECTION( opcId, *pfp, PRT_SEC_MAX( opcId, SCCP_MAXALTOPC ) );
        } else if  ( ( pc == 0L ) || ( pc > PCMAX( lenPc ) ) ) {
            /* The point code is either 0 or too large. */
            flags |= SCCPWRN;
            SKIP_SECTION( opcId, *pfp, PRT_SEC_OOR( opcId ) );
        } else {
            (*pNbrOPC)++;
            pOpcInfo->usapOpc = pc;
        }

        STRPCPRTF(opcId+len,pc);

        *pfp = fpNext;

        /* Scan for up to SCCP_MAXCONPC concerned point codes */
        flags |= scanConPCs( pOpcInfo->conPcList, &pOpcInfo->numConPc, *pfp, opcId, lenPc );

        pOpcInfo++;

        /* Skip to end of this subsystem section, return fpStrt if not found */
        SEEK_TO_EOS( opcId, *pfp, *pfp );

    }   /* while */

    return( flags );

} /* End of scanUsapAltOPCs */

/* --------------------------------------------------------------------------
 * scanRouteAltOPCs
 *   Build a list of alternate point codes by scanning the current section
 *   of the configuration file.
 *
 * Parameters:
 *   pOpc       Pointer to start of list where alternate point codes are placed
 *   pNbrOPC    Address where number of OPCs scanned is returned
 *   fpStrt     Position in configuration file from which to start scan
 *   pSecId     Pointer to a string that uniquely identifies the section
 *   lenPc      Maximum length of point code (PCLEN_ANSI or PCLEN_ITU)
 *
 * Returns:
 *   0          Success
 * or bitfiled of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 scanRouteAltOPCs( RouteOpcInfo *pRteOpcInfo, U16 *pNbrOPC, INT32 fpStrt, char* pSecId, U8 lenPc )
{
    U16      flags = 0;
    INT32    fpNext;
    U32      pc;

    /* Validate file position pointer */
    if ( fpStrt < 0L ) {
        return( SCCPERR );
    }

    /* Scan for up to SCCP_MAXALTPC point codes */
    *pNbrOPC = 0;
    while ( ( fpNext = getcfgval( "ALT_OPC", CFGT_PC, &pc, fpStrt ) ) > 0L ) {
        if ( *pNbrOPC >= SCCP_MAXALTOPC ) {
            /* Number of ALT_OPC exceeds array size. */
            WARN_PC_TOOMANY( pSecId, "ALT_OPC", pc, flags );
        } else if  ( ( pc == 0L ) || ( pc > PCMAX( lenPc ) ) ) {
            /* The point code is either 0 or too large. */
            WARN_PC_OOR( pSecId, "ALT_OPC", pc, flags );
        } else {
            (*pNbrOPC)++;
            pRteOpcInfo->rteOpc = pc;
        }
        fpStrt = fpNext;
        pRteOpcInfo++;

    } /* while */

    if ( fpNext < 0L  ) {
        PRT_DEF_SCANERRLP( pSecId, "ALT_OPC" );
        flags |= SCCPWRN;
    }

    return( flags );
} /* End of scanRouteAltOPCs */

/* --------------------------------------------------------------------------
 * scanSSNs
 *   Build a list of subsystems for a dpc by scanning the current section
 *   of the configuration file.
 *
 * Parameters:
 *   pCfg       Pointer to an SccpRouteCfg structure.
 *   pfp        Pointer to file position in configuration file.  The next
 *              position is returned here.
 *   pSecId     Pointer to a string that uniquely identifies the section.
 *
 * Returns:
 *   0      Success.
 *  !0      Error(s) detected.
 *
 *   The position pointer is also updated.
 */
static U16 scanSSNs( SccpRouteCfg *pCfg, INT32 *pfp, char* pSecId )
{
    U16         flags = 0;
    INT32       fpNext;
    size_t      len;
    SccpSsnCfg  *pSsn = pCfg->ssnList;
    char        ssnId[256];
    INT32       value;

    /* Validate file position pointer. */
    if ( *pfp < 0L ) {
        return( SCCPERR );
    }

    /* Scan for up to SCCP_MAXSSN subsystem definitions */
    strcpy( ssnId, pSecId );
    strcat( ssnId, " SSN " );
    len = strlen( ssnId );
    pCfg->numSsns = 0;
    while( ( fpNext = getcfgval( "SSN", CFGT_INT, &value, *pfp ) ) != 0 ) {
        /* Check for an SSN definition syntax error. */
        if ( fpNext < 0L ) {
            flags |= SCCPWRN;
            SKIP_SECTION( pSecId, *pfp, PRT_DEF_SCANERR( pSecId, "SSN" ) );
        }

        *pfp = fpNext;

        /* Save of SSN definition as string for error reporting. */
        sprintf( ssnId + len, "%lu", value );

        /* Make sure SSN is within range */
        if ( value > 255 ) {
            flags |= SCCPWRN;
            SKIP_SECTION( ssnId, *pfp, PRT_SEC_OOR( ssnId ) );
        }

        /* Make sure there is room in the array to store this SSN */
        if ( pCfg->numSsns >= SCCP_MAXSSNS ) {
            flags |= SCCPWRN;
            SKIP_SECTION( ssnId, *pfp, PRT_SEC_MAX( ssnId, SCCP_MAXSSNS ) );
        }

        /* Valid SSN, check for SSN_xxx definitions */
        pSsn->ssn = (U8) value;
        /* pSsn->swType = pCfg->swType;  (not used!) */

        if ( getVal( pSecId, "SSN_SNR", CFGT_BOOL, &value, *pfp, &flags ) <= 0 ) ;
        else if ( value == 0L ) pSsn->status &= ~SCCP_SNR;
        else                    pSsn->status |= SCCP_SNR;

        if ( getVal( pSecId, "SSN_ACC", CFGT_BOOL, &value, *pfp, &flags ) <= 0 ) ;
        else if ( value == 0L ) pSsn->status &= ~SCCP_ACC;
        else                    pSsn->status |= SCCP_ACC;

        if ( getVal( pSecId, "SSN_BPC", CFGT_PC, &value, *pfp, &flags ) > 0 ) {
            if ( ( value == 0L ) || ( ( (U32) ( value ) ) > PCMAX( PCLEN_ANSI ) ) ) {
                WARN_PC_OOR( ssnId, "SSN_BPC", value, flags );
            } else {
                pSsn->bkupPcInd = 1;
                pSsn->bkupPc = (U32) value;
            }
        }

        GETCHK_I( pSecId, "SSN_UP_ON_RESUME", 0, 1, pSsn->upOnResume,  U8,  flags , fpNext);

        /* Scan for up to SCCP_MAXCONPC concerned point codes */
        flags |= scanConPCs( pSsn->conPcList, &pSsn->numConPc, *pfp, ssnId, PCLEN_ANSI );

        pCfg->numSsns++;
        pSsn++;

        /* Skip to end of this subsystem section, return fpStrt if not found */
        SEEK_TO_EOS( ssnId, *pfp, *pfp );

    }   /* while */

    return( flags );

} /* End of scanSSNs */

/* --------------------------------------------------------------------------
 * prtSccpGenCfg
 *   This function prints out the values of the SccpGenCfg structure for
 *   debugging purposes.
 *
 * Parameters:
 *   cfg        A pointer to an SccpGenCfg structure.
 *
 * Returns:
 *   none
 * -------------------------------------------------------------------------- */
static void prtSccpGenCfg( SccpGenCfg *pCfg )
{
    if ( CDebug ) {
        printf( "\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "GENERAL CONFIGURATION:\n" );
        printf( "    MAX_USERS      %u\n", pCfg->maxSaps );
        printf( "    MAX_NSAPS      %u\n", pCfg->maxNSaps );
        printf( "    MAX_SCLI       %u\n", pCfg->maxScli );
        printf( "    MAX_ADDRS      %u\n", pCfg->maxAddrs );
        printf( "    MAX_ROUTES     %u\n", pCfg->maxRtes );
        printf( "    MAX_ADJDPC     %u\n", pCfg->maxAdjDpc );
        printf( "    MAX_MSGDRN     %u\n", pCfg->maxMsgDrn );
        printf( "    MAX_XUDT       %u\n", pCfg->maxXUDTs );
        printf( "    MAX_XUDTXREF   %u\n", pCfg->maxXUDTref );
        printf( "    MAX_CONN       %u\n", pCfg->maxConn );
        printf( "    DEF_ROUTING    %s\n", pCfg->DefRouting ? FmtTrue : FmtFalse );
        printf( "    SAVE_CONNS     %s\n", pCfg->ConnDrop ? FmtTrue : FmtFalse );
        printf( "    ALARM_LEVEL    %u\n", pCfg->alarmLevel );
        printf( "    TRACE_DATA     %s\n", pCfg->traceFlags & SCCP_TRACE_DATA ? FmtTrue : FmtFalse );
        printf( "    CONN_THRESH    %d\n", pCfg->connThresh );
        printf( "    QUEUE_THRESH   %d\n", pCfg->queueThresh );
        printf( "    SOG_THRESH     %u\n", pCfg->sogThresh );
        printf( "    SCLI_TIMER     %u\n", TIMER_VALUE( &pCfg->scliTimer ) );
        printf( "    SST_TIMER      %u\n", TIMER_VALUE( &pCfg->sstTimer ) );
        printf( "    NSAP_TIMER     %u\n", TIMER_VALUE( &pCfg->nsapTimer ) );
        printf( "    SRT_TIMER      %u\n", TIMER_VALUE( &pCfg->srtTimer ) );
        printf( "    IGNORE_TIMER   %u\n", TIMER_VALUE( &pCfg->ignoreTimer ) );
        printf( "    COORD_TIMER    %u\n", TIMER_VALUE( &pCfg->coordTimer ) );
        printf( "    XREFFRZ_TIMER  %u\n", TIMER_VALUE( &pCfg->xrefFrzTimer ) );
        printf( "    ASMB_TIMER     %u\n", TIMER_VALUE( &pCfg->defAsmbTmr ) );
        printf( "    FREEZE_TIMER   %u\n", TIMER_VALUE( &pCfg->freezeTimer ) );
        printf( "    CONN_TIMER     %u\n", TIMER_VALUE( &pCfg->connTimer ) );
        printf( "    TXINACT_TIMER  %u\n", TIMER_VALUE( &pCfg->txInactTimer ) );
        printf( "    RXINACT_TIMER  %u\n", TIMER_VALUE( &pCfg->rxInactTimer ) );
        printf( "    REL_TIMER      %u\n", TIMER_VALUE( &pCfg->relTimer ) );
        printf( "    REPREL_TIMER   %u\n", TIMER_VALUE( &pCfg->repRelTimer ) );
        printf( "    INTERVAL_TIMER %u\n", TIMER_VALUE( &pCfg->intvalTimer ) );
        printf( "    GUARD_TIMER    %u\n", TIMER_VALUE( &pCfg->guardTimer ) );
        printf( "    RESET_TIMER    %u\n", TIMER_VALUE( &pCfg->resetTimer ) );
        printf( "END\n\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_CFG ) {
        printf( "GEN CONFIG SccpGenCfg structure:\n" );
        printf( "    maxSaps = %u\n",       pCfg->maxSaps );
        printf( "    maxNSaps = %u\n",      pCfg->maxNSaps );
        printf( "    maxScli = %u\n",       pCfg->maxScli );
        printf( "    maxAddrs = %u\n",      pCfg->maxAddrs );
        printf( "    maxRtes = %u\n",       pCfg->maxRtes );
        printf( "    DefRouting = %u\n",    pCfg->DefRouting );
        printf( "    ConnDrop = %u\n",      pCfg->ConnDrop );
        printf( "    maxAdjDpc = %u\n",     pCfg->maxAdjDpc );
        printf( "    maxMsgDrn = %u\n",     pCfg->maxMsgDrn );
        printf( "    maxXUDTs = %u\n",      pCfg->maxXUDTs );
        printf( "    maxXUDTref = %u\n",    pCfg->maxXUDTref );
        printf( "    status = %u\n",        pCfg->status );
        printf( "    sogThresh = %i\n",     pCfg->sogThresh );
        printf( "    scliTimeRes = %i\n",   pCfg->scliTimeRes );
        printf( "    rteTimeRes = %i\n",    pCfg->rteTimeRes );
        printf( "    sapTimeRes = %i\n",    pCfg->sapTimeRes );
        printf( "    nsapTimeRes = %i\n",   pCfg->nsapTimeRes );
        printf( "    xrefTimeRes = %i\n",   pCfg->xrefTimeRes );
        printf( "    asmbTimeRes = %i\n",   pCfg->asmbTimeRes );

        PRT_TIMERCFG( "scliTimer",      &pCfg->scliTimer );
        PRT_TIMERCFG( "sstTimer",       &pCfg->sstTimer );
        PRT_TIMERCFG( "nsapTimer",      &pCfg->nsapTimer );
        PRT_TIMERCFG( "srtTimer",       &pCfg->srtTimer );
        PRT_TIMERCFG( "ignoreTimer",    &pCfg->ignoreTimer );
        PRT_TIMERCFG( "coordTimer",     &pCfg->coordTimer );
        PRT_TIMERCFG( "xrefFrzTimer",   &pCfg->xrefFrzTimer );
        PRT_TIMERCFG( "defAsmbTmr",     &pCfg->defAsmbTmr );

        printf( "    maxConn = %u\n",       pCfg->maxConn );
        printf( "    connThresh = %i\n",    pCfg->connThresh );
        printf( "    queueThresh = %i\n",   pCfg->queueThresh );
        printf( "    conTimeRes = %i\n",    pCfg->conTimeRes );
        printf( "    frzTimeRes = %i\n",    pCfg->frzTimeRes );

        PRT_TIMERCFG( "freezeTimer",    &pCfg->freezeTimer );
        PRT_TIMERCFG( "connTimer",      &pCfg->connTimer );
        PRT_TIMERCFG( "txInactTimer",   &pCfg->txInactTimer );
        PRT_TIMERCFG( "rxInactTimer",   &pCfg->rxInactTimer );
        PRT_TIMERCFG( "relTimer",       &pCfg->relTimer );
        PRT_TIMERCFG( "repRelTimer",    &pCfg->repRelTimer );
        PRT_TIMERCFG( "intvalTimer",    &pCfg->intvalTimer );
        PRT_TIMERCFG( "guardTimer",     &pCfg->guardTimer );
        PRT_TIMERCFG( "resetTimer",     &pCfg->resetTimer );
        printf( "    alarmLevel = %u\n",    pCfg->alarmLevel );
        printf( "    traceFlags = %u\n",    pCfg->traceFlags );
        printf( "    memThresh1 = %u\n",    pCfg->memThresh1 );
        printf( "    memThresh2 = %u\n",    pCfg->memThresh2 );
        printf( "    memThresh3 = %u\n",    pCfg->memThresh3 );
        printf( "\n" );
    }

    return;
} /* End of prtSccpGenCfg */

/* --------------------------------------------------------------------------
 * prtSccpUSapCfg
 *   This function prints out the values of the SccpUSapCfg structure for
 *   debugging purposes.
 *
 * Parameters:
 *   cfg        A pointer to an SccpUSapCfg structure.
 *   sapId      Service Access Point Identifier
 *
 * Returns:
 *   none
 * -------------------------------------------------------------------------- */
static void prtSccpUSapCfg( SccpUSapCfg *pCfg, U16 sapId )
{
    size_t  i, j, num;
    char    buf[(SCCP_LENADDR*2)+1];
    char    *p;


    if ( CDebug ) {
        printf( "\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "USER_SAP %u\n", sapId );
        printf( "    SWITCH_TYPE    " );

        switch ( pCfg->swType ) {
        case SCCP_SW_ANS96 :
            printf( "%s\n", "ANSI96" );
            break;
        case SCCP_SW_ANS92 :
            printf( "%s\n", "ANSI92" );
            break;
        case SCCP_SW_ANS88 :
            printf( "%s\n", "ANSI88" );
            break;
        case SCCP_SW_ITU96 :
            printf( "%s\n", "ITU96" );
            break;
        case SCCP_SW_ITU92 :
            printf( "%s\n", "ITU92" );
            break;
        case SCCP_SW_ITU88 :
            printf( "%s\n", "ITU88" );
            break;
        }


        printf( "    BACKUP_PC      " );
        if ( pCfg->bkupPcInd == 0 ) {
            printf( FmtNone );
        } else {
            PCPRTF( stdout, pCfg->bkupPc );
        }

        num = pCfg->numAltOpc;
        if ( num == 0 ) {
            /* if multiple OPCs not defined, print concerned pcs */
            num = pCfg->usapAltOpc[0].numConPc;
            if ( num == 0 ) {
                printf( "    CONC_PC        %s", FmtNone );
            } else {
                for ( i = 0; i < num; i++ ) {
                    printf( "    CONC_PC        " );
                    PCPRTF( stdout, pCfg->usapAltOpc[0].conPcList[i] );
                    printf("\n");
                }
            }

        } else {
            /* print multiple OPCs and their concerned PCs */
            for(j=0; j<pCfg->numAltOpc; j++)
            {
                printf( "    ALT_OPC        " );
                PCPRTF( stdout, pCfg->usapAltOpc[j].usapOpc );
                printf("\n");
                num = pCfg->usapAltOpc[j].numConPc;
                if ( num == 0 ) {
                    printf( "    CONC_PC        %s", FmtNone );
                } else {
                    for ( i = 0; i < num; i++ ) {
                        printf( "    CONC_PC        " );
                        PCPRTF( stdout, pCfg->usapAltOpc[j].conPcList[i] );
                        printf("\n");
                    }
                }
            }
        }

        printf( "    ADDR_MASK      " );
        if ( pCfg->useMsk == 0 ) {
            printf( FmtNone );
        } else {
            for ( i = 0; i < pCfg->useMsk; i++ ) {
               SccpBcdMaskToAscii( pCfg->addrMsk[i].strg, (U8)(pCfg->addrMsk[i].length * 2), buf, sizeof( buf ) );
               p = buf + strlen( buf ) - 1;
               if ( *p == '0' ) *p = '\0';
               printf( "%s\n", buf );
            }
        }

        printf( "    MAX_HOPS       %u\n", pCfg->hopCnt );
        printf( "END\n\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_CFG ) {
        printf( "USER_SAP %u SccpUSapCfg structure:\n", sapId );
        printf( "    swType = %i\n", pCfg->swType );
        printf( "    selector = %u\n", pCfg->selector );

        printf( "    bkupPcInd = %u\n", pCfg->bkupPcInd );
        if ( pCfg->bkupPcInd ) {
            printf( "    bkupPc = " );
            PCPRTF( stdout, pCfg->bkupPc );
        }

        /* Print numConPc and each element in conPcList array. */
        if(pCfg->numAltOpc == 0){
            num = pCfg->usapAltOpc[0].numConPc;
            printf( "    numConPc = %u\n", num );

            for ( i = 0; i < num; i++ ) {
                printf( "    conPcList[%u] = ", i );
                PCPRTF( stdout, pCfg->usapAltOpc[0].conPcList[i] );
            }
        }
        else {
            printf( "    numAltPc = %u\n", pCfg->numAltOpc );
            for(j=0; j<pCfg->numAltOpc; j++) {
                printf( "    AltOpc = " );
                PCPRTF( stdout, pCfg->usapAltOpc[j].usapOpc );

                num = pCfg->usapAltOpc[j].numConPc;
                printf( "    numConPc = %u\n", num );

                for ( i = 0; i < num; i++ ) {
                    printf( "    conPcList[%u] = ", i );
                    PCPRTF( stdout, pCfg->usapAltOpc[j].conPcList[i] );
                }
            }
        }

        printf( "    useMsk = %u\n", pCfg->useMsk );
        for ( i = 0; i < pCfg->useMsk; i++ ) {
            num = pCfg->addrMsk[i].length;

            printf( "    addrMsk.length = %u\n", num );
            SccpBcdMaskToAscii( pCfg->addrMsk[i].strg, (U8)( num * 2 ), buf, sizeof( buf ) );
            printf( "    addrMsk.strg = %s\n", buf );
        }
        printf( "    hopCnt = %u\n", pCfg->hopCnt );
        printf( "    qThresh1 = %u\n", pCfg->qThresh1 );
        printf( "    qThresh2 = %u\n", pCfg->qThresh2 );
        printf( "    qThresh3 = %u\n", pCfg->qThresh3 );
        printf( "\n" );
    }

    return;
} /* End of prtSccpUSapCfg */

/* --------------------------------------------------------------------------
 * prtSccpNSapCfg
 *   This function prints out the values of the SccpNSapCfg structure for
 *   debugging purposes.
 *
 * Parameters:
 *   cfg        A pointer to an SccpNSapCfg structure.
 *   sapId      Service Access Point Identifier
 *
 * Returns:
 *   none
 * -------------------------------------------------------------------------- */
static void prtSccpNSapCfg( SccpNSapCfg *pCfg, U16 sapId )
{
    char    buf[(SCCP_LENADDR*2)+1];
    U8      len, i;
    char    *p;

    if ( CDebug ) {
        printf( "\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "NSAP %u\n", sapId );

        printf( "    SWITCH_TYPE    " );
        switch ( pCfg->swType ) {
        case SCCP_SW_ANS :
            printf( "ANSI\n" );
            break;
        case SCCP_SW_ITU :
            printf( "ITU\n" );
            break;
        }

        printf( "    SSF            " );
        switch ( pCfg->ssf ) {
        case SCCP_SSF_INTL :
            printf( "INTERNATIONAL\n" );
            break;
        case SCCP_SSF_NAT :
            printf( "NATIONAL\n" );
            break;
        case SCCP_SSF_RES :
            printf( "RESERVED\n" );
            break;
        case SCCP_SSF_SPARE :
            printf( "SPARE\n" );
            break;
        }

        printf( "    DPC            " );
        PCPRTF( stdout, pCfg->dpc );

        printf( "    DPC_LEN        " );
        switch ( pCfg->dpcLen ) {
        case PCLEN_ANSI :
            printf( "24\n" );
            break;
        case PCLEN_ITU :
            printf( "14\n" );
            break;
        }

        printf( "    MSG_LEN        %d\n", pCfg->maxMsgLen );
        printf( "    TXQ_THRESH     %d\n", pCfg->txQThr );

        printf( "    ADDR_MASK      " );
        if ( pCfg->useMsk == 0 ) {
            printf( FmtNone );
        } else {
           for ( i = 0; i < pCfg->useMsk; i++ ) {
              SccpBcdMaskToAscii( pCfg->addrMsk[i].strg, (U8)( pCfg->addrMsk[i].length * 2 ), buf, sizeof( buf ) );
              p = buf + strlen( buf ) - 1;
              if ( *p == '0' ) *p = '\0';
              printf( "%s\n", buf );
           }
        }

        printf( "    MAX_HOPS       %u\n", pCfg->hopCnt );
        printf( "END\n\n" );
    }
    if ( DbgFlgsUsr & SCCP_DBG_CFG ) {

        printf( "NSAP %u SccpNSapCfg structure:\n", sapId );
        printf( "    swType = %d\n", pCfg->swType );
        printf( "    dpcLen = %u\n", pCfg->dpcLen );
        printf( "    dpc = " );
        PCPRTF( stdout, pCfg->dpc );
        printf( "    maxMsgLen = %d\n", pCfg->maxMsgLen );
        printf( "    txQThr = %d\n", pCfg->txQThr );
        printf( "    useMsk = %u\n", pCfg->useMsk );
        for ( i = 0; i < pCfg->useMsk; i++ ) {
            len = pCfg->addrMsk[i].length;
            printf( "    addrMsk.length = %u\n", len );
            SccpBcdMaskToAscii( pCfg->addrMsk[i].strg, (U8)( len * 2 ), buf, sizeof( buf ) );
            printf( "    addrMsk.strg = %s\n", buf );
        }
        printf( "    mem = %s", FmtNotUsed );
        printf( "    selector = %u\n", pCfg->selector );
        printf( "    dstProcId = %s", FmtNotUsed );
        printf( "    priority = %s", FmtNotUsed );
        printf( "    route = %s", FmtNotUsed );
        printf( "    dstEnt = %u\n", pCfg->dstEnt );
        printf( "    dstInst = %u\n", pCfg->dstInst );
        printf( "    spId = %d\n", pCfg->spId );
        printf( "    hopCnt = %u\n", pCfg->hopCnt );
        printf( "    ssf = %u\n", pCfg->ssf );
        printf( "\n" );
    }
    return;
} /* prtSccpNSapCfg */

/* --------------------------------------------------------------------------
 * prtSccpAddrCfg
 *   This function prints out the values of the SccpAddrMapCfg structure for
 *   debugging purposes.
 *
 * Parameters:
 *   cfg        A pointer to an SccpAddrMapCfg structure.
 *   bOddIn     TRUE when Incoming Global Title is an odd number of digits.
 *   bOddOut    TRUE when Outgoing Global Title is an odd number of digits.
 *
 * Returns:
 *   none
 * -------------------------------------------------------------------------- */
static void prtSccpAddrCfg( SccpAddrMapCfg *pCfg, Bool bOddIn, Bool bOddOut )
{
    char            *fmtNatAddr = "    NAT_ADDR       ";
    char            *fmtNumPlan = "    NUM_PLAN       ";
    char            *fmtXltType = "    TRANS_TYPE     ";
    char            gtIn[(SCCP_LENADDR*2)+1];
    char            gtOut[(SCCP_LENADDR*2)+1];
    size_t          len;
    SccpAddrCfg     *pAdr = &pCfg->addr;
    SccpGlbTitleCfg *pGt  = &pCfg->addr.gt;

    if ( CDebug ) {
        printf( "\n" );
    }

    len = ( bOddIn ) ? ( pCfg->gtI.length * 2 ) - 1 : pCfg->gtI.length * 2;
    SccpBcdNumToAscii( pCfg->gtI.strg, (U8)len, gtIn, sizeof( gtIn ) );

    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "ADDRESS %s\n", gtIn );

        printf( "    REPLACE_GLT    %s\n", pCfg->replGt ? FmtTrue : FmtFalse );

        printf( "    SWITCH_TYPE    " );
        switch ( pAdr->swType ) {
        case SCCP_SW_ANS :
            printf( "ANSI\n" );
            break;
        case SCCP_SW_ITU :
            printf( "ITU\n" );
            break;
        }

        printf( "    NI_IND         " );
        switch ( pAdr->niInd ) {
        case ADDRIND_INT :
            printf( "INTERNATIONAL\n" );
            break;
        case ADDRIND_NAT :
            printf( "NATIONAL\n" );
            break;
        }

        printf( "    ROUTING_IND    " );
        switch ( pAdr->rtgInd ) {
        case ROUTE_PC_SN :
            printf( "PC_SSN\n" );
            break;
        case ROUTE_GLT :
            printf( "GLT\n" );
            break;
        }

        printf( "    SSN            " );
        if ( pAdr->ssnInd == 0 ) {
            printf( FmtNone );
        } else {
            printf( "%u\n", pAdr->ssn );
        }

        printf( "    DPC            " );
        if ( pAdr->pcInd == 0 ) {
            printf( FmtNone );
        } else {
            PCPRTF( stdout, pAdr->pc );
        }

        printf( "    GT_FORMAT      " );
        if ( pCfg->replGt == 0 ) {
            printf( FmtNotUsed );
        } else {
            printf( "%u\n", pGt->format );
        }
        printf( "    GL_TITLE       " );
        if ( (pCfg->replGt == 0 ) || ( pGt->format == 0 ) ) {
            printf( FmtNotUsed );
        } else if ( pGt->addr.length == 0 ) {
            printf( FmtNone );
        } else {
            len = ( bOddOut ) ? ( pGt->addr.length * 2 ) - 1 : pGt->addr.length * 2;
            SccpBcdNumToAscii( pGt->addr.strg, (U8) len, gtOut, sizeof( gtOut ) );
            printf( "%s\n", gtOut );
        }

        if ( pCfg->replGt == 0 ) {
            printf( fmtXltType );  printf( FmtNotUsed );
            printf( fmtNatAddr );  printf( FmtNotUsed );
            printf( fmtNumPlan );  printf( FmtNotUsed );
        } else {
            switch ( GT_UNION_FMT( pAdr ) ) {
            case 0 :
                printf( fmtXltType );  printf( FmtNotUsed );
                printf( fmtNatAddr );  printf( FmtNotUsed );
                printf( fmtNumPlan );  printf( FmtNotUsed );
                break;
            case 1 :
                printf( fmtXltType );  printf( FmtNotUsed );
                printf( fmtNatAddr );  printf( "%u\n", pGt->gt.f1.natAddr );
                printf( fmtNumPlan );  printf( FmtNotUsed );
                break;
            case 2 :
                printf( fmtXltType ); printf( "%u\n", pGt->gt.f2.tType );
                printf( fmtNatAddr ); printf( FmtNotUsed );
                printf( fmtNumPlan ); printf( FmtNotUsed );
                break;
            case 3 :
                printf( fmtXltType ); printf( "%u\n", pGt->gt.f3.tType );
                printf( fmtNatAddr ); printf( FmtNotUsed );
                printf( fmtNumPlan ); printf( "%u\n", pGt->gt.f3.numPlan );
                break;
            case 4 :
                printf( fmtXltType ); printf( "%u\n", pGt->gt.f4.tType );
                printf( fmtNatAddr ); printf( "%u\n", pGt->gt.f4.natAddr );
                printf( fmtNumPlan ); printf( "%u\n", pGt->gt.f4.numPlan );
                break;
            }
        }

        printf( "END\n\n" );
    }
    if ( DbgFlgsUsr & SCCP_DBG_CFG ) {
        printf( "ADDRESS %s SccpAddrMapCfg structure:\n", gtIn );
        printf( "    replGt = %s\n", pCfg->replGt ? FmtTrue : FmtFalse );
        printf( "    gtI.length = %u\n", pCfg->gtI.length );
        printf( "    gtI.strg = %s\n", gtIn );
        printf( "    addr.pres = %u\n", pAdr->pres );
        printf( "    addr.swType = %d\n", pAdr->swType );
        printf( "    addr.niInd = %u\n", pAdr->niInd );
        printf( "    addr.rtgInd = %u\n", pAdr->rtgInd );
        printf( "    addr.ssnInd = %u\n", pAdr->ssnInd );
        printf( "    addr.pcInd = %u\n", pAdr->pcInd );
        printf( "    addr.ssnInd = " );
        if ( pAdr->ssnInd == 0 ) {
            printf( FmtNone );
        } else {
            printf( "%u\n", pAdr->ssn );
        }
        printf( "    addr.pc = " );
        if ( pAdr->pcInd == 0 ) {
            printf( FmtNone );
        } else {
            PCPRTF( stdout, pAdr->pc );
        }

        if ( pCfg->replGt == FALSE ) {
            printf( "    addr.gt = %s", FmtNotUsed );
        } else {
            printf( "    addr.gt.format = %u\n", pGt->format );

            switch ( GT_UNION_FMT( pAdr ) ) {
            case 0 :
                printf( "    addr.gt.gt = %s", FmtNotUsed );
                break;
            case 1 :
                printf( "    addr.gt.gt.f1.oddEven = %u\n", pGt->gt.f1.oddEven );
                printf( "    addr.gt.gt.f1.natAddr = %u\n", pGt->gt.f1.natAddr );
                break;
            case 2 :
                printf( "    addr.gt.gt.f2.tType = %u\n", pGt->gt.f2.tType );
                break;
            case 3 :
                printf( "    addr.gt.gt.f3.tType = %u\n", pGt->gt.f3.tType );
                printf( "    addr.gt.gt.f3.numPlan = %u\n", pGt->gt.f3.numPlan );
                printf( "    addr.gt.gt.f3.encSch = %u\n", pGt->gt.f3.encSch );
                break;
            case 4 :
                printf( "    addr.gt.gt.f4.tType = %u\n", pGt->gt.f4.tType );
                printf( "    addr.gt.gt.f4.numPlan = %u\n", pGt->gt.f4.numPlan );
                printf( "    addr.gt.gt.f4.encSch = %u\n", pGt->gt.f4.encSch );
                printf( "    addr.gt.gt.f4.natAddr = %u\n", pGt->gt.f4.natAddr );
                break;
            }


            if ( pGt->format == 0 ) {
                printf( "    addr.gt.addr = %s", FmtNotUsed );
            } else {
                len = pGt->addr.length;
                printf( "    addr.gt.addr.length = %u\n", len );
                if ( len > 0 ) {
                    printf( "    addr.gt.addr.strg = %s\n", gtOut );
                }
            }
        }

        printf( "\n" );
    }
    return;
} /* End of prtSccpAddrCfg */

/* --------------------------------------------------------------------------
 * prtSccpRouteCfg
 *   This function prints out the values of the SccpRouteCfg structure for
 *   debugging purposes.
 *
 * Parameters:
 *   cfg        A pointer to an SccpRouteCfg structure.
 *
 * Returns:
 *   none
 * -------------------------------------------------------------------------- */
static void prtSccpRouteCfg( SccpRouteCfg *pCfg )
{
    size_t      i, j;
    size_t      nCpc;
    size_t      nSs;
    U32         *pCpc;
    SccpSsnCfg  *pSs;

    if ( CDebug ) {
        printf( "\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "ROUTE " );
        PCPRTF( stdout, pCfg->dpc );
        printf( "    SWITCH_TYPE    " );
        switch ( pCfg->swType ) {
        case SCCP_SW_ANS :
            printf( "ANSI\n" );
            break;
        case SCCP_SW_ITU :
            printf( "ITU\n" );
            break;
        }
        printf( "    TRANSLATOR     %s\n", ( pCfg->rteAltOpc[0].status & SCCP_TRANS ) ? FmtTrue : FmtFalse );
        printf( "    ADJACENT       %s\n", ( pCfg->rteAltOpc[0].status & SCCP_ADJACENT ) ? FmtTrue : FmtFalse );

        if ( pCfg->numAltOpc != 0 ) {
            for(i=0; i<pCfg->numAltOpc; i++) {
                printf( "    ALT_OPC        " );
                PCPRTF( stdout, pCfg->rteAltOpc[i].rteOpc );
                printf("\n");
            }
        }

        printf( "    BACKUP_PC      " );
        if ( pCfg->bkupPcInd == 0 ) {
            printf( FmtNone );
        } else {
            PCPRTF( stdout, pCfg->bkupPc );
        }

        nSs = pCfg->numSsns;
        pSs = pCfg->ssnList;
        for ( i = 0; i < nSs; i++ ) {
            printf( "    SSN %u\n", pSs->ssn );
            printf( "        SSN_SNR    %s\n", ( pSs->status & SCCP_SNR ) ? FmtTrue : FmtFalse );
            printf( "        SSN_ACC    %s\n", ( pSs->status & SCCP_ACC ) ? FmtTrue : FmtFalse );

            printf( "        SSN_BPC    " );
            if ( pSs->bkupPcInd == 0 ) {
                printf( FmtNone );
            } else {
                PCPRTF( stdout, pSs->bkupPc );
            }

            nCpc = pSs->numConPc;
            pCpc = pSs->conPcList;
            for ( j = 0; j < nCpc; j++ ) {
                printf( "        CONC_PC    " );
                PCPRTF( stdout, *pCpc++ );
            }

            printf( "    END\n" );
            pSs++;
        }

        printf( "END\n\n" );
    }

    if ( DbgFlgsUsr & SCCP_DBG_CFG ) {
        U32 pc = pCfg->dpc;

        printf( "ROUTE %d.%d.%d [%#lx] SccpRouteCfg structure:\n",
                PC1( pc ), PC2( pc ), PC3( pc ), pc );
        printf( "    swType = %d\n", pCfg->swType );
        printf( "    dpc = " );
        PCPRTF( stdout, pCfg->dpc );
        printf( "    status = %u\n", pCfg->rteAltOpc[0].status );
        printf( "    bkupPcInd = %u\n", pCfg->bkupPcInd );
        printf( "    bkupPc = " );
        if ( pCfg->bkupPcInd == 0 ) {
            printf( FmtNone );
        } else {
            PCPRTF( stdout, pCfg->bkupPc );
        }
        if ( pCfg->numAltOpc == 0 ) {
            printf( "    AltOpc = " );
            printf( FmtNone );
        } else {
            for(i=0; i<pCfg->numAltOpc; i++) {
                printf( "    AltOpc = " );
                PCPRTF( stdout, pCfg->rteAltOpc[i].rteOpc );
            }
        }
        nSs = pCfg->numSsns;
        pSs = pCfg->ssnList;
        printf( "    numSsns = %u\n", nSs );
        for ( i = 0; i < nSs; i++ ) {
            printf( "    ssnList[%u].swType = %d\n", i, pSs->swType );
            printf( "    ssnList[%u].ssn = %u\n", i, pSs->ssn );
            printf( "    ssnList[%u].status = %u\n", i, pSs->status );
            printf( "    ssnList[%u].bkupPcInd = %u\n", i, pSs->bkupPcInd );
            printf( "    ssnList[%u].bkupPc = ", i );
            if ( pSs->bkupPcInd == 0 ) {
                printf( FmtNone );
            } else {
                PCPRTF( stdout, pSs->bkupPc );
            }

            nCpc = pSs->numConPc;
            pCpc = pSs->conPcList;
            printf( "    ssnList[%u].numConPc = %u\n", i, pSs->numConPc );
            for ( j = 0; j < nCpc; j++ ) {
                printf( "    ssnList[%u].conPcList[%u] = ", i, j );
                PCPRTF( stdout, *pCpc++ );
            }

            pSs++;
        }

        printf( "\n" );
    }
    return;
} /* prtSccpRouteCfg */


/* --------------------------------------------------------------------------
 * sccpGenCfg
 *   Configure the general parameters for SCCP.
 *
 * Parameters:
 *   board      Board number [1..8]
 *
 * Returns:
 *   0          Success
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpGenCfg( U8 board )
{
    U16             flags = 0;
    SccpGenCfg      cfg;
    U16             errors;
    Bool            isYes;
    char            *pSecId = "GEN CONFIG";
    SCCP_STATUS     status;
    char            str[CFGT_MAXSTR];   /* Holds string returned by getcfgval */
    INT32           val;                /* Holds int/bool rtn'd by getcfgval  */

    /* Initialize the SccpGenCfg structure to its default values. */
    if ( ( status = SccpInitGenCfg( &cfg ) ) != SCCP_SUCCESS ) {
        RETURN_SCCPERR( PRT_MGMTERR( status, "SccpInitGenCfg()" ) );
    }

    if ( CDebug ) printf( "\n" );

    /* Return an error status if section has no END statement.  Check that
     * there is at least one END statement in file.  Then make sure the first
     * END found isn't really for a USER_SAP, NSAP, ADDRESS or ROUTE section
     * instead.
     */
    SEEK_TO_EOS( pSecId, 0L, EndGenCfg );
    CHK_POS( pSecId, getcfgval( "USER_SAP", CFGT_INT,    &val, 0L ) );
    CHK_POS( pSecId, getcfgval( "NSAP",     CFGT_INT,    &val, 0L ) );
    CHK_POS( pSecId, getcfgval( "ADDRESS",  CFGT_STRING, str,  0L ) );
    CHK_POS( pSecId, getcfgval( "ROUTE",    CFGT_PC,     &val, 0L ) );

    /* Scan the configuration file for overrides of the default values. */
    GETCHK_IG( pSecId, "MAX_USERS",    1,   255, cfg.maxSaps,     U8,  flags );
    GETCHK_IG( pSecId, "MAX_NSAPS",    1,   255, cfg.maxNSaps,    U8,  flags );
    GETCHK_IG( pSecId, "MAX_SCLI",     0, 65535, cfg.maxScli,     U16, flags );
    GETCHK_IG( pSecId, "MAX_ADDRS",    0, 65535, cfg.maxAddrs,    U16, flags );
    GETCHK_IG( pSecId, "MAX_ROUTES",   1, 65535, cfg.maxRtes,     U16, flags );

    GETCHK_BG( pSecId, "DEF_ROUTING",            cfg.DefRouting,       flags );
    GETCHK_BG( pSecId, "SAVE_CONNS",             cfg.ConnDrop,         flags );

    GETCHK_IG( pSecId, "MAX_ADJDPC",   0, 65535, cfg.maxAdjDpc,   U16, flags );
    GETCHK_IG( pSecId, "MAX_MSGDRN",   0, 65535, cfg.maxMsgDrn,   U16, flags );
    GETCHK_IG( pSecId, "MAX_XUDT",     0, 65535, cfg.maxXUDTs,    U16, flags );
    GETCHK_IG( pSecId, "MAX_XUDTXREF", 0, 65535, cfg.maxXUDTref,  U16, flags );
    GETCHK_IG( pSecId, "MAX_CONN",     0, 65535, cfg.maxConn,     U16, flags );
    GETCHK_IG( pSecId, "CONN_THRESH",  0,     9, cfg.connThresh,  S16, flags );
    GETCHK_IG( pSecId, "QUEUE_THRESH", 0, 32766, cfg.queueThresh, S16, flags );
    GETCHK_IG( pSecId, "SOG_THRESH",   0,     9, cfg.sogThresh,   U8,  flags );
    GETCHK_IG( pSecId, "ALARM_LEVEL",   0,    3, cfg.alarmLevel,  U8,  flags );  /* allow both,  */
    GETCHK_IG( pSecId, "SCCP_ALARM_LEVEL", 0, 3, cfg.alarmLevel,  U8,  flags );  /* SCCP_ALARM_LEVEL overrides */
    GETCHK_IG( pSecId, "MEM_THRESH_1",   1,  99, cfg.memThresh1,   U32,  flags );
    GETCHK_IG( pSecId, "MEM_THRESH_2",   1,  99, cfg.memThresh2,   U32,  flags );
    GETCHK_IG( pSecId, "MEM_THRESH_3",   1,  99, cfg.memThresh3,   U32,  flags );

    isYes = FALSE;
    errors = 0;
    GETCHK_BG( pSecId, "TRACE_DATA", isYes, errors );
    if ( errors )     flags |= errors;
    else if ( isYes ) cfg.traceFlags |=  SCCP_TRACE_DATA;
    else              cfg.traceFlags &= ~SCCP_TRACE_DATA;

    isYes = TRUE;
    errors = 0;
    GETCHK_BG( pSecId, "SCCP_ONLINE", isYes, errors );
    if ( errors )    flags |= errors;
    else if ( isYes) cfg.status = (U8) SCCP_ONLINE;
    else             cfg.status = (U8) SCCP_OFFLINE;

    /* Timers */
    GET_TIMER( pSecId, "XREFFRZ_TIMER",  cfg.xrefFrzTimer,  flags );
    GET_TIMER( pSecId, "ASMB_TIMER",     cfg.defAsmbTmr,    flags );
    GET_TIMER( pSecId, "FREEZE_TIMER",   cfg.freezeTimer,   flags );
    GET_TIMER( pSecId, "CONN_TIMER",     cfg.connTimer,     flags );
    GET_TIMER( pSecId, "TXINACT_TIMER",  cfg.txInactTimer,  flags );
    GET_TIMER( pSecId, "RXINACT_TIMER",  cfg.rxInactTimer,  flags );
    GET_TIMER( pSecId, "REL_TIMER",      cfg.relTimer,      flags );
    GET_TIMER( pSecId, "REPREL_TIMER",   cfg.repRelTimer,   flags );
    GET_TIMER( pSecId, "INTERVAL_TIMER", cfg.intvalTimer,   flags );
    GET_TIMER( pSecId, "GUARD_TIMER",    cfg.guardTimer,    flags );
    GET_TIMER( pSecId, "RESET_TIMER",    cfg.resetTimer,    flags );
    GET_TIMER( pSecId, "SCLI_TIMER",     cfg.scliTimer,     flags );
    GET_TIMER( pSecId, "SST_TIMER",      cfg.sstTimer,      flags );
    GET_TIMER( pSecId, "NSAP_TIMER",     cfg.nsapTimer,     flags );
    GET_TIMER( pSecId, "SRT_TIMER",      cfg.srtTimer,      flags );
    GET_TIMER( pSecId, "IGNORE_TIMER",   cfg.ignoreTimer,   flags );
    GET_TIMER( pSecId, "COORD_TIMER",    cfg.coordTimer,    flags );
    GET_TIMER( pSecId, "AIC_TIMER",      cfg.aicTimer,      flags );
    GET_TIMER( pSecId, "AIC_RESP_TIMER", cfg.aicRespTimer,  flags );
    GET_TIMER( pSecId, "ACR_TIMER",      cfg.conRespTimer,  flags );

    /* get the point code display format from the config scanner global setting */
    cfg.pcDispFmt = CfgPcDispFmt;

    /* Save some general configuration dimensions for checks in other sections.*/
    MaxUSaps  = cfg.maxSaps;
    MaxNSaps  = cfg.maxNSaps;
    MaxRoutes = cfg.maxRtes;
    MaxAddrs  = cfg.maxAddrs;

    if ( DbgFlgsUsr ) {
        prtSccpGenCfg( &cfg );
    }
    if ( OK_TO_TRANSMIT ) {
        if ( ( status = SccpSetGenCfg( board, &cfg ) ) != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpSetGenCfg()" ) );
        }
    }

    return( flags );
} /* sccpGenCfg */

/* --------------------------------------------------------------------------
 * sccpUSapCfg
 *   Configure the SCCP User (Upper) SAPs
 *
 * Parameters:
 *   board  Board number [1..8]
 *
 * Returns:
 *   0          Success
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpUSapCfg( U8 board )
{
    U16         flags = FALSE;
    SccpUSapCfg cfg;
    INT32       fpCur = 0L;             /* Current position in config file    */
    U16         maxId;
    U32         nDigits;                /* Number of digits in address mask   */
    U32         pc;                     /* Point code                         */
    U16         sapId;                  /* User SAP ID                        */
    SCCP_STATUS status;
    char        secId[LENU32 + 9];      /* Holds "USER_SAP " + SapId          */
    char        str[CFGT_MAXSTR];       /* Holds string returned by getcfgval */
    S16         swType;                 /* Switch type (i.e., which protocol) */
    INT32       value;                  /* Holds integer returned by getcfgval*/

    strcpy( secId, "USER_SAP " );
    maxId = (U16) ( ( MaxUSaps > 1 ) ? ( MaxUSaps - 1 ) : 0 );

    /* Scan each USER_SAP section.                                            */
    while ( ( fpCur = getcfgval( "USER_SAP", CFGT_INT, &value, fpCur ) ) > 0L ) {
        /* Create section identifier for error messages */
        sprintf( secId + 9, "%lu", (U32) value );

        /*
         *  Make sure that the general configuration limit for number of
         *  User SAPs was set large enough to handle this User SAP number.
         *  If not, warn user and discard.  But keep scanning for others,
         *  since they need not be defined in any order.
         */
        sapId = (U16) value;
        if ( sapId > maxId ) {
            SKIP_SECTION( secId, fpCur, PRT_SEC_MAX( secId, MaxUSaps ) );
        }

        /* Determine protocol to be used */
        swType = SCCP_SW_ANS92; /* default */
        if ( getVal( secId, "SWITCH_TYPE", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "ANSI96" ) == 0 ) swType = SCCP_SW_ANS96;
        else if ( strcmp( str, "ANSI92" ) == 0 ) swType = SCCP_SW_ANS92;
        else if ( strcmp( str, "ANSI88" ) == 0 ) swType = SCCP_SW_ANS88;
        else if ( strcmp( str, "ITU96"  ) == 0 ) swType = SCCP_SW_ITU96;
        else if ( strcmp( str, "ITU92"  ) == 0 ) swType = SCCP_SW_ITU92;
        else if ( strcmp( str, "ITU88"  ) == 0 ) swType = SCCP_SW_ITU88;
        else WARN_DEFINV_DEFS( secId, "SWITCH_TYPE", str, "ANSI92", flags );

        /* Initialize User SAP parameters to their default values */
        status = SccpInitUSapCfg( &cfg, swType );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpInitUSagCfg()" ) );
        }

        /* Scan for Backup Point Code */
        if ( getVal( secId, "BACKUP_PC", CFGT_PC, &pc, fpCur, &flags ) > 0 ) {
            if ( ( pc == 0L ) || ( pc > PCMAX( PCLEN_ANSI ) ) ) {
                WARN_PC_OOR( secId, "BACKUP_PC", pc, flags );
            } else {
                cfg.bkupPcInd = 1;
                cfg.bkupPc = pc;
            }
        }

        cfg.useMsk = getAddrMasks( secId, "ADDR_MASK", fpCur, cfg.addrMsk, &nDigits, &flags );

        /* Scan for Maximum Hops */
        GETCHK_I( secId, "MAX_HOPS", 1, 15, cfg.hopCnt, U8, flags, fpCur );

        /* Scan for Queue Congestion Thresholds */
        GETCHK_I( secId, "CONG_THRESH_1", 1, 2000, cfg.qThresh1, U32, flags, fpCur );
        GETCHK_I( secId, "CONG_THRESH_2", 1, 2000, cfg.qThresh2, U32, flags, fpCur );
        GETCHK_I( secId, "CONG_THRESH_3", 1, 2000, cfg.qThresh3, U32, flags, fpCur );

        /* scan for optional application inactivity timing on connections */
        GETCHK_B( secId, "INACT_CONTROL", cfg.aicEnabled, flags, fpCur )

        /* Scan for up to MAXALTOPC Multiple OPCs */
        flags |= scanUsapAltOPCs( cfg.usapAltOpc, &cfg.numAltOpc, &fpCur, secId, PCLEN_ANSI);

        /* if multiple OPCs are not specified, the concerned point codes for the default OPC
         * have to be scanned */
        if ( cfg.numAltOpc == 0) {
            /* Scan the concered point codes */
            flags |= scanConPCs( cfg.usapAltOpc[0].conPcList, &cfg.usapAltOpc[0].numConPc, fpCur, secId, PCLEN_ANSI );

        }

        if ( DbgFlgsUsr ) {
            prtSccpUSapCfg( &cfg, sapId );
        }

        if ( OK_TO_TRANSMIT ) {
            /* Send the configuration parameters to the board */
            status = SccpSetUSapCfg( board, &cfg, sapId );
            if ( status != SCCP_SUCCESS ) {
                RETURN_SCCPERR( PRT_MGMTERR( status, "SccpSetUSapCfg()" ) );
            }
        }

        /* Skip to end section, return error if END not found */
        SEEK_TO_EOS( secId, fpCur, fpCur );
    } /* End of while loop for scanning each USER_SAP section */

    if ( fpCur < 0L ) {
        PRT_SEC_SCANERRLP( "USER_SAP" );
        flags |= SCCPERR;
    }

    if ( CDebug ) printf( "\n" );

    return( flags );
} /* End of sccpUSapCfg */

/* --------------------------------------------------------------------------
 * sccpPCLenCfg
 *   Configure Point Code length for messages
 *
 * Parameters:
 *   board  Board number [1..8]
 *
 * Returns:
 *   0          Success
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpPCLenCfg( void )
{
    U16         flags = 0;
    INT32       fpCur = 0L;            /* Current position in config file    */
    char        secId[LENU32 + 5];   /* Holds "NSAP " + sapId              */
    INT32       value;                  /* Holds integer returned by getcfgval*/

    strcpy( secId, "PCLEN " );

    /* Scan each NSAP section                                                 */
    while ( ( fpCur = getcfgval( "NSAP", CFGT_INT, &value, fpCur ) ) > 0L ) {
        /* Create section identifier for error messages */
        sprintf( secId + 5, "%lu", value );

        /* Check for optional DPC length override */
        if ( getVal( secId, "DPC_LEN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
            switch ( value ) {
            case 14 :
                pcLen = PCLEN_ITU;
                pcDispFmt = SS7_PCDISP_INTL;
                break;
            case 24 :
                pcLen = PCLEN_ANSI;
                pcDispFmt = SS7_PCDISP_DFLT;
                break;
            default:
                PRT_DPCLEN( secId, value );
                flags |= SCCPWRN;
                break;
            }
        }

        /* Skip to end of section, return error if END not found. */
        SEEK_TO_EOS( secId, fpCur, fpCur );

    } /* End while loop for scanning each NSAP section */

    if ( fpCur < 0L ) {
        flags |= SCCPERR;
        PRT_SEC_SCANERRLP( "NSAP" );
    }

    if ( CDebug ) printf( "\n" );

    return( flags );
} /* End of sccpPCLenCfg */

/* --------------------------------------------------------------------------
 * sccpNSapCfg
 *   Configure lower SAPs with MTP3 level
 *
 * Parameters:
 *   board  Board number [1..8]
 *
 * Returns:
 *   0          Success
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpNSapCfg( U8 board )
{
    U16         flags = 0;
    SccpNSapCfg cfg;
    INT32       fpCur = 0L;            /* Current position in config file    */
    U32         dpc;
    U16         maxId;
    U32         nDigits;
    U16         sapId;                  /* Network SAP ID                     */
    SCCP_STATUS status;
    char        str[CFGT_MAXSTR];       /* Holds string returned by getcfgval */
    char        secId[LENU32 + 5];   /* Holds "NSAP " + sapId              */
    S16         swType;                 /* Switch type (i.e., which protocol) */
    INT32       value;                  /* Holds integer returned by getcfgval*/

    strcpy( secId, "NSAP " );
    maxId = (U16) ( ( MaxNSaps > 1 ) ? ( MaxNSaps - 1 ) : 0 );

    /* Scan each NSAP section                                                 */
    while ( ( fpCur = getcfgval( "NSAP", CFGT_INT, &value, fpCur ) ) > 0L ) {
        /* Create section identifier for error messages */
        sprintf( secId + 5, "%lu", value );

        /*
         * Make sure that the general configuration limit for number of Network
         * SAPs was set large enough to handle this Network SAP.  If not, warn
         * user and discard.  But keep scanning for others, since they need not
         * be defined in any order. */
        sapId = (U16) value;
        if ( sapId > maxId ) {
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_MAX( secId, MaxNSaps ) );
        }

        /* Get the DPC (required) */
        if ( getcfgval( "DPC", CFGT_PC, &dpc, fpCur ) <= 0L ) {
            flags |= SCCPERR;
            SKIP_SECTION( secId, fpCur, PRT_DPC( secId ) );
        }

        /* Determine protocol to be used */
        swType = SCCP_SW_ANS; /* default */
        if ( getVal( secId, "SWITCH_TYPE", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "ANSI" ) == 0 ) swType = SCCP_SW_ANS;
        else if ( strcmp( str, "ITU"  ) == 0 ) swType = SCCP_SW_ITU;
        else WARN_DEFINV_DEFS( secId, "SWITCH_TYPE", str, "ANSI", flags );

        /* Initialize Network SAP parameters to their default values. */
        status = SccpInitNSapCfg( &cfg, sapId, dpc, swType );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpInitNSapCfg()" ) );
        }

        /* Check for optional DPC length override */
        if ( getVal( secId, "DPC_LEN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
            switch ( value ) {
            case 14 :
                cfg.dpcLen = PCLEN_ITU;
                pcLen = PCLEN_ITU;
                pcDispFmt = SS7_PCDISP_INTL;
                break;
            case 24 :
                cfg.dpcLen = PCLEN_ANSI;
                pcLen = PCLEN_ANSI;
                pcDispFmt = SS7_PCDISP_DFLT;
                break;
            default:
                PRT_DPCLEN( secId, value );
                flags |= SCCPWRN;
                break;
            }
        }

        /* Validate length of DPC */
        if ( ( dpc == 0L ) || ( dpc > PCMAX( cfg.dpcLen ) ) ) {
            WARN_PC_OOR( secId, "DPC", dpc, flags );
            flags |= SCCPERR;
            SKIP_SECTION( secId, fpCur, PRT_DPC( secId ) );
        }

        GETCHK_I( secId, "MSG_LEN",  32, 1500, cfg.maxMsgLen, S16, flags, fpCur );
        GETCHK_I( secId, "TXQ_THRESH", 0, 32766, cfg.txQThr, S16, flags, fpCur );

        /* Scan for optional address mask */
        cfg.useMsk = getAddrMasks( secId, "ADDR_MASK", fpCur, cfg.addrMsk, &nDigits, &flags );

        /* Scan for Maximum Hops */
        GETCHK_I( secId, "MAX_HOPS", 1, 15, cfg.hopCnt, U8, flags, fpCur );

        /* Get optional Subsystem field (of SIO) override */
        if ( getVal( secId, "SSF", CFGT_STRING, (void *) str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "INTERNATIONAL" ) == 0 ) cfg.ssf = SCCP_SSF_INTL;
        else if ( strcmp( str, "INTL"          ) == 0 ) cfg.ssf = SCCP_SSF_INTL;
        else if ( strcmp( str, "SPARE"         ) == 0 ) cfg.ssf = SCCP_SSF_SPARE;
        else if ( strcmp( str, "NATIONAL"      ) == 0 ) cfg.ssf = SCCP_SSF_NAT;
        else if ( strcmp( str, "NATL"          ) == 0 ) cfg.ssf = SCCP_SSF_NAT;
        else if ( strcmp( str, "RESERVED"      ) == 0 ) cfg.ssf = SCCP_SSF_RES;
        else if ( strcmp( str, "RES"           ) == 0 ) cfg.ssf = SCCP_SSF_RES;
        else {
            PRT_DEF_INV_IS( secId, "SSF", str );
            flags |= SCCPWRN;
        }

        /* scan for optional national/intl address indicator value for mgmt msgs */
        GETCHK_I( secId, "SCCP_NI_IND", 0, 1, cfg.niInd, U8, flags, fpCur );

        if ( DbgFlgsUsr ) {
            prtSccpNSapCfg( &cfg, sapId );
        }
        if ( OK_TO_TRANSMIT ) {
            /* Send the configuration parameters to the board. */
            status = SccpSetNSapCfg( board, &cfg, sapId );
            if ( status != SCCP_SUCCESS ) {
                RETURN_SCCPERR( PRT_MGMTERR( status, "SccpSetNSapCfg()" ) );
            }
        }

        /* Skip to end of section, return error if END not found. */
        SEEK_TO_EOS( secId, fpCur, fpCur );

    } /* End while loop for scanning each NSAP section */

    if ( fpCur < 0L ) {
        flags |= SCCPERR;
        PRT_SEC_SCANERRLP( "NSAP" );
    }

    if ( CDebug ) printf( "\n" );

    return( flags );
} /* End of sccpNSapCfg */

/* --------------------------------------------------------------------------
 * sccpAddrCfg
 *     Configure the address table.
 *
 * Parameters:
 *   board  Board number [1..8]
 *
 * Returns:
 *   0      Success
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpAddrCfg( U8 board )
{
    Bool        bOddIn;
    Bool        bOddOut;
    U16         flags = 0;
    SccpAddrMapCfg  cfg;
    SccpAddrCfg     *pAdr = &cfg.addr;
    SccpGlbTitleCfg *pGt = &cfg.addr.gt;
    INT32       fpCur = 0L;             /* Current position in config file    */
    U8          encSch;                 /* Encoding scheme                    */
    char        secId[CFGT_MAXSTR + 8]; /* Holds "ADDRESS " + Incoming GLT    */
    U16         nAddr = 0;              /* Nbr of addresses specified         */
    U8          natAddr;                /* Nature of address                  */
    U8          numPlan;                /* Numbering Plan                     */
    int         rtn;
    SCCP_STATUS status;
    char        str[CFGT_MAXSTR];       /* Holds string returned by getcfgval */
    U8          tType;                  /* Translation type                   */
    INT32       value;                  /* Holds integer returned by getcfgval*/
    U8          bcd[SCCP_LENADDR];      /* Holds BCD digits                   */
    U32         nDigits;                /* Number of BCD digits in BCD buffer */

    strcpy( secId, "ADDRESS " );
    /*
     * Addresses have two components:
     * 1. An incoming address
     *      This is the global title string to translate.
     * 2. The translation
     *      This is the fully qualified SCCP address the
     *      incoming address will be translated into.
     */

    /* Scan each ADDRESS section */
    while ( ( fpCur = getcfgval( "ADDRESS", CFGT_STRING, str, fpCur ) ) > 0L ) {
        /* Create section identifier for error messages */
        strcpy( secId + 8, str );

        /* Check if number exceeds maximum specified in general configuration */
        if ( ++nAddr > MaxAddrs ) {
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_MAX( secId, MaxAddrs ) );
        }

        /* Convert ascii address to bcd */
        status = SccpAsciiNumToBcd( str, bcd, SCCP_LENADDR, &nDigits );
        switch ( status ) {
        case SCCP_SUCCESS:
            break;
        case SCCP_BUFLEN:
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_ADRLEN( secId, SCCP_LENADDR * 2 ) );
            break;
        case SCCP_BADDIGIT:
        default:
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_ADRSYNTAX( secId  ) );
            break;
        }
        bOddIn = (Bool) ( nDigits % 2 );

        /* Initialize Address parameters to their default values. */
        status = SccpInitAddrCfg( &cfg, bcd, (U8) BCD_BYTES( nDigits ) );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpInitAddrCfg()" ) );
        }

        /* Scan for replacement flag */
        if ( getVal( secId, "REPLACE_GLT", CFGT_BOOL, &value, fpCur, &flags ) > 0 ) {
            cfg.replGt = (U8) value;
        }

        /* Scan for Switch Type  */
        pAdr->swType = SCCP_SW_ANS; /* default */
        if ( getVal( secId, "SWITCH_TYPE", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "ANSI" ) == 0 ) pAdr->swType = SCCP_SW_ANS;
        else if ( strcmp( str, "ITU"  ) == 0 ) pAdr->swType = SCCP_SW_ITU;
        else WARN_DEFINV_DEFS( secId, "SWITCH_TYPE", str, "ANSI", flags );

        /*
         * A global title translation entry may have any combination of another
         * global title and/or subsystem number and/or destination point code.
         */

        /* Scan for Routing Indicator */
        pAdr->rtgInd = ROUTE_PC_SN;
        if ( getVal( secId, "ROUTING_IND", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "PC_SSN" ) == 0 ) pAdr->rtgInd = ROUTE_PC_SN;
        else if ( strcmp( str, "GLT"    ) == 0 ) pAdr->rtgInd = ROUTE_GLT;
        else WARN_DEFINV_DEFS( secId, "ROUTING_IND", str, "PC_SSN", flags );

        /* Scan for National/International Indicator */
        pAdr->niInd = ADDRIND_NAT;
        if ( getVal( secId, "NI_IND", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "NATIONAL"      ) == 0 ) pAdr->niInd = ADDRIND_NAT;
        else if ( strcmp( str, "NAT"           ) == 0 ) pAdr->niInd = ADDRIND_NAT;
        else if ( strcmp( str, "INTERNATIONAL" ) == 0 ) pAdr->niInd = ADDRIND_INT;
        else if ( strcmp( str, "INTL"          ) == 0 ) pAdr->niInd = ADDRIND_INT;
        else WARN_DEFINV_DEFS( secId, "NI_IND", str, "NATIONAL", flags );

        /* Scan for Subsystem Number and validate */
        if ( getVal( secId, "SSN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
            if ( ( value >= 0L ) && ( value <= 255L ) ) {
                pAdr->ssnInd = 1;
                pAdr->ssn = (U8) value;
            } else {
                PRT_DEF_OOR_IN( secId, "SSN", value );
            }
        }

        /* Scan for destination point code */
        if ( getVal( secId, "DPC", CFGT_PC, &value, fpCur, &flags ) > 0 ) {
            if ( ( value == 0L ) || ( ( (U32) value ) > PCMAX( PCLEN_ANSI ) ) ) {
                WARN_PC_OOR( secId, "DPC", value, flags );
            } else {
                pAdr->pcInd = 1;
                pAdr->pc = (U32) value;
            }
        }

        bOddOut = FALSE;
        if ( cfg.replGt == FALSE ) {
            /* When REPLACE_GLT is FALSE, GT_FORMAT, GT_TILE, NAT_ADDR,
             * TRANS_TYPE and NUM_PLAN are ignored.  Warn the user if
             * they are present.
             */
            if ( getVal( secId, "GT_FORMAT", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                PRT_REPL_FALSE( secId, "GT_FORMAT" );
            }
            if ( getVal( secId, "GT_TITLE", CFGT_STRING, str, fpCur, &flags ) > 0 ) {
                PRT_REPL_FALSE( secId, "GT_TITLE" );
            }
            if ( getVal( secId, "NAT_ADDR", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                PRT_REPL_FALSE( secId, "NAT_ADDR" );
            }
            if ( getVal( secId, "NUM_PLAN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                PRT_REPL_FALSE( secId, "NUM_PLAN" );
            }
            if ( getVal( secId, "TRANS_TYPE", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                PRT_REPL_FALSE( secId, "TRANS_TYPE" );
            }
        } else {
           /*
            * ANSI Supports Type 1 and Type 2 global titles.  However, Type 1
            * global titles are stored in the GTFRMT_3 part of the global title
            * union
            */
            if ( pAdr->swType == SCCP_SW_ANS ) {
                GETCHK_I( secId, "GT_FORMAT", 0, 2, pGt->format , U8, flags, fpCur );
            } else {
                GETCHK_I( secId, "GT_FORMAT", 0, 4, pGt->format , U8, flags, fpCur );
            }

            if ( pGt->format == 0 ) {
                /* When GT_FORMAT is 0, GT_TILE, NAT_ADDR, TRANS_TYPE and
                 * NUM_PLAN are ignored.  Warn the user if they are present.
                 */
                if ( getVal( secId, "GT_TITLE", CFGT_STRING, str, fpCur, &flags ) > 0 ) {
                    PRT_REPL_FORMAT( secId, "GT_TITLE", pGt->format );
                }
                if ( getVal( secId, "NAT_ADDR", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                    PRT_REPL_FORMAT( secId, "NAT_ADDR", pGt->format );
                }
                if ( getVal( secId, "NUM_PLAN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                    PRT_REPL_FORMAT( secId, "NUM_PLAN", pGt->format );
                }
                if ( getVal( secId, "TRANS_TYPE", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                    PRT_REPL_FORMAT( secId, "TRANS_TYPE", pGt->format );
                }
            } else {
                /* Get outgoing global title string */
                rtn = getAddr( secId, "GL_TITLE", fpCur, &pGt->addr, &nDigits, &flags );
                if ( rtn <= 0 ) {
                    flags |= SCCPERR;
                    SKIP_SECTION( secId, fpCur, PRT_GTFORMAT( secId, pGt->format ) );
                }

                /*
                * Set default values for outgoing global title.  (Which of these are
                * actually used depends on the format type).
                */
                bOddOut = (Bool) ( nDigits % 2 );
                encSch = (U8) ( ( nDigits % 2 ) ? ENC_BCD_ODD : ENC_BCD_EVEN );
                natAddr = NATIND_NATL;
                numPlan = NP_ISDN;
                tType   = 0;

                switch ( GT_UNION_FMT( pAdr ) ) {
                case 1 :
                    pGt->gt.f1.oddEven = (U8)( nDigits % 2 );
                    GETCHK_I( secId, "NAT_ADDR",   0, 4,   natAddr, U8, flags, fpCur );
                    pGt->gt.f1.natAddr = natAddr;

                    if ( getVal( secId, "NUM_PLAN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                        PRT_REPL_FORMAT( secId, "NUM_PLAN", pGt->format );
                    }
                    if ( getVal( secId, "TRANS_TYPE", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                        PRT_REPL_FORMAT( secId, "TRANS_TYPE", pGt->format );
                    }
                    break;
                case 2 :
                    GETCHK_I( secId, "TRANS_TYPE", 0, 255, tType, U8, flags, fpCur );
                    pGt->gt.f2.tType = tType;
                    if ( getVal( secId, "NAT_ADDR", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                        PRT_REPL_FORMAT( secId, "NAT_ADDR", pGt->format );
                    }
                    if ( getVal( secId, "NUM_PLAN", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                        PRT_REPL_FORMAT( secId, "NUM_PLAN", pGt->format );
                    }
                    break;
                case 3 :
                    pGt->gt.f3.encSch = encSch;
                    GETCHK_I( secId, "TRANS_TYPE", 0, 255, tType, U8, flags, fpCur );
                    pGt->gt.f3.tType = tType;
                    GETCHK_I( secId, "NUM_PLAN",   0, 15,  numPlan, U8, flags, fpCur );
                    pGt->gt.f3.numPlan = numPlan;
                    if ( getVal( secId, "NAT_ADDR", CFGT_INT, &value, fpCur, &flags ) > 0 ) {
                        PRT_REPL_FORMAT( secId, "NAT_ADDR", pGt->format );
                    }
                    break;
                case 4 :
                    pGt->gt.f4.encSch = encSch;
                    GETCHK_I( secId, "TRANS_TYPE", 0, 255, tType, U8, flags, fpCur );
                    pGt->gt.f4.tType = tType;
                    GETCHK_I( secId, "NUM_PLAN",   0, 15,  numPlan, U8, flags, fpCur );
                    pGt->gt.f4.numPlan = numPlan;
                    GETCHK_I( secId, "NAT_ADDR",   0, 4,   natAddr, U8, flags, fpCur );
                    pGt->gt.f4.natAddr = natAddr;
                    break;
                } /* switch */
            }
        }

        if ( DbgFlgsUsr ) {
            prtSccpAddrCfg( &cfg, bOddIn, bOddOut );
        }
        if ( OK_TO_TRANSMIT ) {
            /* Send the configuration parameters to the board */
            status = SccpSetAddrCfg( board, &cfg );
            if ( status != SCCP_SUCCESS ) {
                RETURN_SCCPERR( PRT_MGMTERR( status, "SccpSetAddrCfg()" ) );
            }
        }

        /* Skip to end of section, return error if END not found. */
        SEEK_TO_EOS( secId, fpCur, fpCur );

    } /* End while scanning for ADDRESS sections */

    if ( fpCur < 0L ) {
        PRT_SEC_SCANERRLP( "ADDRESS" );
        flags |= SCCPERR;
    }

    if ( CDebug ) printf( "\n" );

    return( flags );

} /* End of sccpAddrCfg */

/* --------------------------------------------------------------------------
 * sccpRteCfg
 *
 * Parameters:
 *   board      Board number [1..8]
 *
 * Returns:
 *   0          Success.
 * or bitfield of
 *   SCCPWRN    Warning(s) issued.
 *   SCCPERR    Error(s) detected.
 * -------------------------------------------------------------------------- */
static U16 sccpRteCfg( U8 board )
{
    U16             flags = 0;
    SccpRouteCfg    cfg;
    INT32           fpCur = 0L;        /* Current position in config file    */
    U32             dpc;
    U16             nRoutes = 0;        /* Number of routes specified         */
    SCCP_STATUS     status;
    char            str[CFGT_MAXSTR];   /* Holds string returned by getcfgval */
    char            secId[LENPC + 6];   /* "ROUTE xxx.xxx.xxx"                */
    INT32           value;              /* Holds integer returned by getcfgval*/
    U8              rteStatus=0, i;
    U16             count;

    strcpy( secId, "ROUTE " );

    /* Scan each ROUTE section */
    while ( ( fpCur = getcfgval( "ROUTE", CFGT_PC, &dpc, fpCur ) ) > 0L ) {
        nRoutes++;

        /* Convert dpc to string for error messages */
        PCPRTS( secId + 6, dpc );

        /* Save off route and verify value. */
        if ( ( dpc == 0 ) || ( dpc > PCMAX( PCLEN_ANSI ) ) ) {
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_OOR( secId ) );
        }

        if ( nRoutes > MaxRoutes ) {
            flags |= SCCPWRN;
            SKIP_SECTION( secId, fpCur, PRT_SEC_MAX( secId, MaxRoutes ) );
        }

        /* Initialize Route Configuration parameters structure. */
        status = SccpInitRteCfg( &cfg, dpc );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpInitRteCfg()" ) );
        }

        /*
         * Fetch overides from config file
         */

        if ( getVal( secId, "SWITCH_TYPE", CFGT_STRING, str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "ITU"  ) == 0 ) cfg.swType = SCCP_SW_ITU;
        else if ( strcmp( str, "ANSI" ) == 0 ) cfg.swType = SCCP_SW_ANS;
        else WARN_DEFINV_DEFS( secId, "SWITCH_TYPE", str, "ANSI", flags );

        if ( getVal( secId, "ADJACENT", CFGT_BOOL, &value, fpCur, &flags ) <= 0 ) ;
        else if ( value == 0 ) rteStatus &= ~SCCP_ADJACENT;
        else                   rteStatus |=  SCCP_ADJACENT;

        if ( getVal( secId, "TRANSLATOR", CFGT_BOOL, &value, fpCur, &flags ) <= 0 ) ;
        else if ( value == 0 ) rteStatus &= ~SCCP_TRANS;
        else                   rteStatus |=  SCCP_TRANS;

         /* Get optional Subsystem field (of SIO) override */
        if ( getVal( secId, "SSF", CFGT_STRING, (void *) str, fpCur, &flags ) <= 0 ) ;
        else if ( strcmp( str, "INTERNATIONAL" ) == 0 ) cfg.ssf = SCCP_SSF_INTL;
        else if ( strcmp( str, "INTL"          ) == 0 ) cfg.ssf = SCCP_SSF_INTL;
        else if ( strcmp( str, "SPARE"         ) == 0 ) cfg.ssf = SCCP_SSF_SPARE;
        else if ( strcmp( str, "NATIONAL"      ) == 0 ) cfg.ssf = SCCP_SSF_NAT;
        else if ( strcmp( str, "NATL"          ) == 0 ) cfg.ssf = SCCP_SSF_NAT;
        else if ( strcmp( str, "RESERVED"      ) == 0 ) cfg.ssf = SCCP_SSF_RES;
        else if ( strcmp( str, "RES"           ) == 0 ) cfg.ssf = SCCP_SSF_RES;
        else                                            cfg.ssf = SCCP_SSF_DEF;

        /* scan for the multiple OPCs for this route */
        flags |= scanRouteAltOPCs( cfg.rteAltOpc, &cfg.numAltOpc, fpCur, secId, PCLEN_ANSI);

        if ( getVal( secId, "BACKUP_PC", CFGT_PC, &value, fpCur, &flags ) > 0 ) {
            if ( ( value == 0L ) || ( ( (U32) value ) > PCMAX( PCLEN_ANSI ) ) ) {
                WARN_PC_OOR( secId, "BACKUP_PC", value, flags );
            } else {
                cfg.bkupPcInd = 1;
                cfg.bkupPc = (U32) value;
            }
        }

        if(cfg.numAltOpc == 0)          /* if multiple opcs are not specified, save the status at the index 0 */
                count = 1;
        else
                count = cfg.numAltOpc;

        /* save the same value of status ( ADJACENT/TRANSLATOR) in the status
         * for all the opcs. */
        for(i = 0; i < count; i++)
                cfg.rteAltOpc[i].status = rteStatus;

        /* Scan for up to SCCP_MAXSNS subsystem definitions */
        flags |= scanSSNs( &cfg, &fpCur, secId );


        if ( DbgFlgsUsr ) {
            prtSccpRouteCfg( &cfg );
        }
        if ( OK_TO_TRANSMIT ) {
            status = SccpSetRteCfg( board, &cfg );
            if ( status != SCCP_SUCCESS ) {
                RETURN_SCCPERR( PRT_MGMTERR( status, "SccpSetRteCfg()" ) );
            }
        }

        /* Skip to end of section, return error if END not found. */
        SEEK_TO_EOS( secId, fpCur, fpCur );

    } /* End while scannig for ROUTE sections */

    if ( fpCur < 0L ) {
        PRT_SEC_SCANERRLP( "ROUTE" );
        flags |= SCCPERR;
    }

    if ( CDebug ) printf( "\n" );

    return( flags );
} /* End of sccpRteCfg */

/* --------------------------------------------------------------------------
 * sccpCfg
 *   Main SCCP configuration routine.
 *
 * Parameters:
 *   board      Board number [1..8]
 *
 * Returns:
 *   0          Success.
 *   2          Warning(s) issued.
 *   3          Error(s) detected.
 * -------------------------------------------------------------------------- */
static int sccpCfg( U8 board )
{
    U16  flags = 0;
    int  rtn = 2;
    SCCP_STATUS status;


    if ( OK_TO_TRANSMIT ) {
        /* Initialize the SCCP API Management API */
        status = SccpMgmtInit( board, SCCP_ENT, 0 );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpMgmtInit()" ) );
        }
    }

    printf("--- Reading file %s\n", CfgFile);

    /* Note that errors are reported in the routine in which they occur. */
    if ( initcfg( CfgFile ) == 0 ) {
        flags |= sccpGenCfg(  board );
        /* TRP - The PCLenCfg routine checks the NSAP section for point code length for correct
         * display of error/debug msgs for later sections */
        /* The NSAP section must be configured last because the NSAP configuration message
         * results in the SCCP task sending a bind message to the MTP task. All Routes must be
         * configured before any MTP messages are handled by the SCCP task */
        if ( !( flags & SCCPERR ) ) flags |= sccpPCLenCfg( );
        if ( !( flags & SCCPERR ) ) flags |= sccpUSapCfg( board );
        if ( !( flags & SCCPERR ) ) flags |= sccpAddrCfg( board );
        if ( !( flags & SCCPERR ) ) flags |= sccpRteCfg(  board );
        if ( !( flags & SCCPERR ) ) flags |= sccpNSapCfg( board );

        if      ( flags & SCCPERR ) rtn = 2;
        else if ( flags & SCCPWRN ) rtn = 1;
        else                        rtn = 0;

        endcfg();
    }

    if ( OK_TO_TRANSMIT ) {
        status = SccpMgmtTerm( board );
        if ( status != SCCP_SUCCESS ) {
            RETURN_SCCPERR( PRT_MGMTERR( status, "SccpMgmtTerm()" ) );
        }
    }

    return( rtn );
} /* End of sccpCfg() */

/* --------------------------------------------------------------------------
 * main
 *   This function parses the command line for optional parameters and then
 *   invokes the configuration routines.
 *
 * Parameters:
 *
 * Returns:
 *      0   Successful.
 *      1   Warning(s) issued.
 *      2   Error(s) detected.
 * -------------------------------------------------------------------------- */
int main( int argc, char *argv[] )
{
    U8       board = 1;             /* Default board is 1                     */
    U16      flags;                 /* Debug flags                            */
    int      i;
    int      opt;
    char     *p;
    char     *pEnd;
    S16      tmp;

    PRT_BANNER;

    /* Parse the command line for configuration file name and board number */
    for ( i = 1; i < argc; i++ ) {
        /* Check the next argument for the options indicator '-' */
        p = argv[i];
        if ( *p++ != '-' ) RETURN_SCCPERR( PRT_USAGE );

        /* Check for valid option after the hyphen */
        if ( strpbrk( p, "bBdDfF" ) == NULL ) RETURN_SCCPERR( PRT_USAGE );
        opt = tolower( *p++ );

        /*
         * Position p at start of option value, which is either immediately
         * following the letter or separated by a space.  Note that the
         * debug option does not require a value to be specified.
         */
        if      ( *p != '\0' )  ;
        else if ( ++i < argc )  p = argv[i];
        else if ( opt != 'd' )  RETURN_SCCPERR( PRT_USAGE );

        switch ( opt ) {
        case 'b' :                      /* Board number                       */
            if ( sscanf( p, "%hi", &tmp ) != 1 ) {
                RETURN_SCCPERR( PRT_USAGE );
            } else if ( ( tmp < 1 ) || ( tmp > MAXBOARD ) ) {
                RETURN_SCCPERR( PRT_BOARD );
            }
            board = (U8)tmp;
            break;
        case 'd' :                      /* Turn on debugging                  */
            flags = (U16) ( ( *p ) ? strtoul( p, &pEnd, 0 ) : SCCP_DBG_USER );
            DbgFlgsUsr |= ( flags & 0x00ff );
            DbgFlgsXmt |= ( flags & 0xff00 );
            break;
        case 'f' :                      /* Name of configuration file         */
            strncpy( CfgFile, p, sizeof( CfgFile ) - 1 );
            CfgFile[sizeof( CfgFile ) - 1] = '\0';
            break;
        }
    } /* End for ( i = 1; i < argc; i++ ) */

    if ( DbgFlgsUsr & SCCP_DBG_SCAN ) CDebug  = TRUE;
    if ( DbgFlgsXmt & SCCP_DBG_DUMP ) bSccpMgmtDebug = TRUE;
    if ( DbgFlgsUsr & SCCP_DBG_USER ) {
        printf( "    Board number:       %d\n",   board   );
        printf( "    Configuration file: %s\n\n", CfgFile );
    }

    return( sccpCfg( board ) );
} /* End of main() */
