/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation.                         **
**                                                                      **
*************************************************************************/

/* --------------------------------------------------------------------------
 *  SCCP Management Sample Application
 *
 *  This sample application illustrates the use of the SCCP Management API
 *  primitives.  The user can issue commands to retrieve statistics and
 *  status, set alarm levels and enable tracing.
 *
 *  To Run:
 *
 *      SCCPMGR [-b <board>] [-p ITU | ANSI ]
 *
 *  where:
 *
 *      board    Board number to open (defaults to 1).
 *
 *      -p       Optional, controls point code format for input & display;
 *               I[TU]  scans/displays x.y.z point codes as international 14-bit
 *                      3-8-3 format;
 *               A[NSI] (Default) scans/displays x.y.z point codes as 24-bit
 *                      8-8-8 format.
 *
 *
 *  User commands:
 *
 *      ALARMLEVEL <0..3>
 *          Sets alarm level to number specified.
 *      GENERAL CONFIG
 *          Displays general configuration
 *      GENERAL STATS
 *          Displays general statistics.
 *      USAP <sap nbr> CONFIG
 *          Displays User SAP Configuration for specified SAP.
 *      USAP <sap nbr> STATS [RESET]
 *          Displays User SAP Statistics for specified SAP, with optional reset.
 *      USAP <sap nbr> STATUS
 *          Displays User SAP Status for specified SAP.
 *      NSAP <sap nbr> CONFIG
 *          Displays Network SAP Configuration for the specified SAP.
 *      NSAP <sap nbr> STATS [RESET]
 *          Displays Network SAP Statistics for specified SAP, with opt reset.
 *      TRACE DATA <ON | OFF>
 *          Turns data tracing on or off.
 *      BOARD <board>
 *          Switches to a new target board
 *      ? [COMMAND]
 *          Lists available commands or parameters for a specific command.
 *      QUIT
 *          Quits application.
 * -------------------------------------------------------------------------- */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <txnuser.h>
#include "sccpver.h"
#include "sccpmapi.h"
#include "cfgscan.h"

#if !defined( TRUE )
#define TRUE 1
#endif
#if !defined( FALSE )
#define FALSE 0
#endif

#define SCCP_TYPE_CONFIG    1
#define SCCP_TYPE_STATS     2
#define SCCP_TYPE_STATUS    3

#define NOT_USED( a )       { ( a ) = ( a ); }

#define PRT_TIMERCFG( name, ptr )                               \
{                                                               \
    printf( "%s.enable = %u  ", name, (ptr)->enable );          \
    if ( (ptr)->enable ) {                                      \
        printf( "%s.value = %u\n", name, (ptr)->value  );       \
    } else {                                                    \
        printf( "\n" );                                         \
    }                                                           \
}

#define PC1( pc )             ( ( (U32) ( pc ) ) >> 16 )
#define PC2( pc )           ( ( ( (U32) ( pc ) ) >> 8  ) & 0xff )
#define PC3( pc )             ( ( (U32) ( pc ) )         & 0xff )

#define PRT_PC( pc )                                                    \
{                                                                       \
    U32 _pc = (U32) ( pc );                                             \
    printf( "%s\n", CfgFmtPc( _pc, pcLen, pcDispFmt ) );                \
}

#define PAUSE                                                   \
{                                                               \
    printf( "\nPress the Enter key to continue . . .\n" );      \
    getchar();                                                  \
}

/* --------------------------------------------------------------------------
 * Output messages
 * -------------------------------------------------------------------------- */

static char *MsgUsage   = "Usage: sccpmgr [-b <board>] [-p ITU | ANSI ]\n";
static char *MsgPrompt  = "\nSccpMgr[%1u]>";
static char *MsgBanner  = "\nTX Series SCCP Management Application Version %s %s\n\n";
static char *MsgBoard   = "Error, board number must be in range 1..8\n";
static char *MsgTimeout = "Error, no response from board.\n";
static char *MsgMgmtApi = "Error, %s failed with status of %#6x.\n";
static char *MsgAlarm   = "Error, alarm level must be in range 0..3\n";
static char *MsgNotFound= "Error, SAP %u undefined for this board.\n";
static char *MsgGenCfg  = "Error, the board has not recieved its general configuration parameters yet.\n";
static char *MsgCommand = "Error, unrecognized command: %s.\n";
static char *MsgNParms  = "Error, insufficient number of parameters.\n";
static char *MsgParm    = "Error, unrecognized parameter: %s.\n";
static char *MsgPCrange = "Error, point code out of range: %s.\n";
static char *MsgPCNotFound= "Error, point code %s undefined for this board.\n";
static char *MsgNotUsed = "<Not used>\n";

#define RETURN_USAGE    { fprintf( stderr, MsgUsage ); return( 1 ); }

#define RETURN_BOARD    { fprintf( stderr, MsgBoard ); return( 1 ); }

#define RETURN_MAPI(f,v){ fprintf( stderr, MsgMgmtApi, f, v ); return( 1 ); }

/* --------------------------------------------------------------------------
 * Misc. global variables
 * -------------------------------------------------------------------------- */

static U8   BDone = FALSE;                  /* Indicates when 'Quit' issued.  */
static U8   NBoard = 1;                     /* Holds board number (default=1) */
static U8   pcDispFmt = SS7_PCDISP_DFLT;    /* point code display format, defaults to "8.8.8" (+hex) */
static U8   pcLen = PCLEN_ANSI;             /* point code length, defaults to 24 bits */


/* --------------------------------------------------------------------------
 * Forward declarations of funtions used in look up tables
 * -------------------------------------------------------------------------- */

static void alarmCtrl( DWORD *parms );
static void helpReq( DWORD *parms );
static void setBoard( DWORD *parms );
static void genReq( DWORD *parms );
static void nSapReq( DWORD *parms );
static void routeReq( DWORD *parms );
static void uSapReq( DWORD *parms );
static void traceCtrl( DWORD *parms );
static void quit( DWORD *parms );

/* --------------------------------------------------------------------------
 * Help menus
 * -------------------------------------------------------------------------- */

static char AllHelp[] =
    "ALARMLEVEL  Sets alarm level\n"
    "BOARD       Changes target board\n"
    "GENERAL     Displays general information\n"
    "NSAP        Displays network SAP information\n"
    "ROUTE       Displays SCCP route information\n"
    "TRACE       Enables/disables tracing\n"
    "USAP        Displays user SAP information\n"
    "HELP | ?    Displays help information\n"
    "QUIT        Quits program\n"
    "\n"
    "For help with a specific command enter HELP <command>.\n"
    ;

static char AlarmsCtrlHelp[] = \
"Syntax: ALARMLEVEL <0..3>\n";
static char BoardHelp[] = \
"Syntax: BOARD <new target number>\n";
static char GenHelp[] = \
"Syntax: GENERAL CONFIG\n        GENERAL STATS [RESET]\n";
static char NSapHelp[] = \
"Syntax: NSAP <SAP number> CONFIG\n        NSAP <SAP number> STATS [RESET]\n";
static char RouteHelp[] = \
"Syntax: ROUTE <point code> CONFIG\n        ROUTE <point code> STATUS\n";
static char USapHelp[] = \
"Syntax: USAP <SAP number> CONFIG\n        USAP <SAP number> STATS [RESET]\n        USAP <SAP number> STATUS\n";
static char TraceCtrlHelp[] = \
"Syntax: TRACE DATA <ON | OFF>\n";
static char QuitHelp[] = \
"Syntax: QUIT\n";
static char HelpHelp[] = \
"Syntax: HELP [ALARMLEVEL | BOARD | GENERAL | NSAP | TRACE | USAP | HELP | QUIT]\n";

static char* HelpTbl[] =
{
    AllHelp,
    AlarmsCtrlHelp,
    BoardHelp,
    GenHelp,
    NSapHelp,
    RouteHelp,
    TraceCtrlHelp,
    USapHelp,
    HelpHelp,
    HelpHelp,
    QuitHelp
};

/* --------------------------------------------------------------------------
 * Look up tables for translating names into values and vice versa
 * -------------------------------------------------------------------------- */

typedef struct {                        /* Type def for Look up table entity  */
	char 	*name;
    DWORD value;
} LookUp;

static LookUp LupTblCmd[] =
{
    { "ALARMLEVEL",   1 },
    { "BOARD",        2 },
    { "GENERAL",      3 },
    { "NSAP",         4 },
    { "ROUTE",        5 },
    { "TRACE",        6 },
    { "USAP",         7 },
    { "HELP",         8 },
    { "?",            9 },
    { "QUIT",         10 },
    { "Q",            11 },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblType1[] = {
    { "CONFIG", SCCP_TYPE_CONFIG  },
    { "STATS",  SCCP_TYPE_STATS   },
    { NULL,    0                  }
};

static LookUp LupTblType2[] = {
    { "CONFIG", SCCP_TYPE_CONFIG },
    { "STATS",  SCCP_TYPE_STATS  },
    { "STATUS", SCCP_TYPE_STATUS },
    { NULL,     0                }
};

static LookUp LupTblTraceType[] = {     /* Types of SCCP trace control        */
    { "DATA", SCCP_TRACE_DATA },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblType3[] = {
    { "CONFIG", SCCP_TYPE_CONFIG  },
    { "STATUS",  SCCP_TYPE_STATUS },
    { NULL,    0                  }
};


static LookUp LupTblTraceAction[] = {    /* SCCP trace control actions         */
    { "ON",  TRUE },
    { "OFF", FALSE },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblReset[] = {         /* TCAP Stats reset action values  */
    { "RESET", 1 },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblSwType[] = {        /* switch types        */
    { "ITU",    SCCP_SW_ITU },
    { "ANSI",   SCCP_SW_ANS },
    { "ITU-88", SCCP_SW_ITU88 },
    { "ITU-92", SCCP_SW_ITU92 },
    { "ANSI-88",SCCP_SW_ANS88 },
    { "ANSI-92",SCCP_SW_ANS92 },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblDpcLen[] = {        /* point code lengths   */
    { "14", PCLEN_ITU },
    { "24", PCLEN_ANSI },
    { "16", PCLEN_JAPAN },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblRunState[] = {      /* Redundancy states  */
    { "INITIAL", SCCP_INITIAL },
    { "BACKUP", SCCP_BACKUP },
    { "PRIMARY", SCCP_PRIMARY },
    { "STANDALONE", SCCP_STANDALONE },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblMateState[] = {     /* Mate connection states  */
    { "(ISOLATED)", SCCP_MATE_ISOLATED },
    { "(CONNECTED)", SCCP_MATE_CONNECTED },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblNiInd[] = {         /* NI Indicator values  */
    { "INTERNATIONAL", 0 },
    { "NATIONAL", 1 },
    { NULL, 0 } /* End of table */
};

static LookUp LupTblEnDis[] = {         /* Enabled/disabled status  */
    { "DISABLED", 0 },
    { "ENABLED", 1 },
    { NULL, 0 } /* End of table */
};

/* --------------------------------------------------------------------------
 * forward refs for lookup table manipulation routines
 * -------------------------------------------------------------------------- */
static char *getString( LookUp *tbl, DWORD val );
static short getVal( LookUp *tbl, char *str, U32 *val );


/* --------------------------------------------------------------------------
 * Command parse tables
 * -------------------------------------------------------------------------- */
#define MAX_PARMS   8

typedef struct {
    char    *command;
    U8      nParmsReq;
    LookUp  *parmTbl[MAX_PARMS];
    void    (*func)(DWORD *);
} CmdSpec;

static CmdSpec AlarmCtrlCmd = {                     /* Alarm Control          */
    "ALARMLEVEL",                                   /* - command name         */
    1,
    { 0, 0, 0, 0, 0, 0, 0, 0 },                     /* - parameter look ups   */
    alarmCtrl                                       /* - handler function     */
};

static CmdSpec BoardCmd = {                         /* Change Board           */
    "BOARD",                                        /* - command name         */
    1,
    {  0, 0, 0, 0, 0, 0, 0, 0 },                    /* - parameter look ups   */
    setBoard                                        /* - handler function     */
};

static CmdSpec GenCmd = {                           /* General Information    */
    "GENERAL",                                      /* - command name         */
    1,
    { LupTblType1, LupTblReset, 0, 0, 0, 0, 0, 0 }, /* - parameter look ups   */
    genReq                                          /* - handler function     */
};

static CmdSpec NSapCmd = {                          /* Network SAP Information*/
    "NSAP",                                         /* - command name         */
    2,
    { 0, LupTblType1, LupTblReset, 0, 0, 0, 0, 0 }, /* - parameter look ups   */
    nSapReq
};

static CmdSpec RouteCmd = {                         /* Route Information*/
    "ROUTE",                                         /* - command name         */
    2,
    { 0, LupTblType3, 0, 0, 0, 0, 0, 0 },           /* - parameter look ups   */
    routeReq
};

static CmdSpec TraceCtrlCmd = {                     /* Trace Control          */
    "TRACE",                                        /* - command name         */
    1,
    { LupTblTraceAction, 0, 0, 0, 0, 0, 0, 0 }, /* parm look ups*/
    traceCtrl                                       /* - handler function     */
};

static CmdSpec USapCmd = {                          /* User SAP Information   */
    "USAP",                                         /* - command name         */
    2,
    { 0, LupTblType2, LupTblReset, 0, 0, 0, 0, 0 }, /* - parameter look ups   */
    uSapReq
};

static CmdSpec HelpCmd = {                          /* Help Command           */
    "HELP",                                         /* - command name         */
    0,
    { LupTblCmd, 0, 0, 0, 0, 0, 0, 0 },             /* - parameter look ups   */
    helpReq                                         /* - handler function     */
};

static CmdSpec QuestionCmd = {                      /* ? Command              */
    "?",                                            /* - command name         */
    0,
    { LupTblCmd, 0, 0, 0, 0, 0, 0, 0 },             /* - parameter look ups   */
    helpReq                                         /* - handler function     */
};

static CmdSpec QuitCmd = {                          /* Quit command           */
    "QUIT",                                         /* - command name         */
    0,
    {  0, 0, 0, 0, 0, 0, 0, 0 },                    /* - parameter look ups   */
    quit                                            /* - handler function     */
};

static CmdSpec QCmd = {                             /* Quit command "Q"       */
    "Q",                                            /* - command name         */
    0,
    {  0, 0, 0, 0, 0, 0, 0, 0 },                    /* - parameter look ups   */
    quit                                            /* - handler function     */
};

static CmdSpec *CmdTable[] = {
    &AlarmCtrlCmd,
    &BoardCmd,
    &GenCmd,
    &NSapCmd,
    &RouteCmd,
    &TraceCtrlCmd,
    &USapCmd,
    &HelpCmd,
    &QuestionCmd,
    &QuitCmd,
    &QCmd,
    0
};

/* --------------------------------------------------------------------------
 * Command handler routines
 * -------------------------------------------------------------------------- */

/*
 * Alarm Control command handler
 *
 * Sets the alarm level.  First parameter is alarm level entered, which must
 * be in range 0..3.
 */
static void alarmCtrl( DWORD *parms )
{
    U8          level = (U8) parms[0];
    SCCP_STATUS rtnCode;

    if ( level > 3 ) {
        fprintf( stderr, MsgAlarm );
        return;
    }

    printf( "Calling SccpAlarmControl( %u, %u )\n", NBoard, level );
#if defined( NO_TX_BOARD )
    rtnCode = SCCP_SUCCESS;
#else
    rtnCode = SccpAlarmControl( NBoard, level );
#endif
    switch ( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT :
        fprintf( stderr, MsgTimeout );
        break;
    default :
        fprintf(  stderr, MsgMgmtApi, "SccpAlarmControl()", rtnCode );
        break;
    }

    return;
}

/* Board command handler
 *
 * Changes the current target board and closes out access to the old target
 * board.  The first parameter is the board number.
 */
static void setBoard( DWORD *parms )
{
    U8          newBoard = (U8) parms[0];
    SCCP_STATUS rtnCode;

    if ( newBoard == NBoard ) {
        return;
    }

    if ( newBoard < 1 || newBoard > 8 ) {
        fprintf( stderr, MsgBoard );
        return;
    }

    printf( "Calling SccpMgmtInit( %u, SCCPMGR_ENT, 0 )\n", newBoard );
#if defined( NO_TX_BOARD )
    rtnCode = SCCP_SUCCESS;
#else
    rtnCode = SccpMgmtInit( newBoard, SCCPMGR_ENT, 0 );
#endif
    if ( rtnCode != SCCP_SUCCESS ) {
        fprintf( stderr, MsgMgmtApi, "SccpMgmtInit()", rtnCode );
        return;
    }

    printf( "Calling SccpMgmtTerm( %u )\n", NBoard );
    SccpMgmtTerm( NBoard );
    NBoard = newBoard;
    return;
}

/*
 * General command handler
 *
 * First parameter indicates type of information desired (CONFIG or STATS)
 * Second paramter is optional RESET flag
 */
static void genReq( DWORD *parms )
{
    S16             action = (S16) parms[0];
    U8              bReset = (U8) parms[1];
    SccpGenCfg      cfg = {0};
    char            *funcName = "Unknown";
    SCCP_STATUS     rtnCode = 0;
    SccpGenStats    stats = {0};

    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        funcName = "SccpGetGenCfg()";
        printf( "Calling SccpGetGenCfg( %u, &cfg)\n", NBoard );
        #if defined( NO_TX_BOARD )
            memset( &cfg, 0, sizeof( cfg ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetGenCfg( NBoard, &cfg );
        #endif
        break;
    case SCCP_TYPE_STATS :
        funcName = "SccpGetGenStats()";
        printf( "Calling SccpGetGenStats( %u, &stats, %u )\n", NBoard, bReset );
        #if defined( NO_TX_BOARD )
            memset( &stats, 0, sizeof( stats ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetGenStats( NBoard, &stats, bReset );
        #endif
        break;
    }

    switch ( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT:
        fprintf( stderr, MsgTimeout );
        return;
    case SCCP_NOGENCFG :
        fprintf( stderr, MsgGenCfg );
        return;
    default :
        fprintf( stderr, MsgMgmtApi, funcName, rtnCode );
        return;
    }

    /* Request successful, display results */
    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        printf( "Board %d General Configuration:\n", NBoard );
        printf( "maxSaps    = %-5u  maxAdjDpc   = %-5u  rteTimeRes  = %-5i\n",
               cfg.maxSaps, cfg.maxAdjDpc, cfg.rteTimeRes );
        printf( "maxNSaps   = %-5u  maxMsgDrn   = %-5u  sapTimeRes  = %-5i\n",
                 cfg.maxNSaps, cfg.maxMsgDrn, cfg.sapTimeRes );
        printf( "maxScli    = %-5u  maxXUDTs    = %-5u  nsapTimeRes = %-5i\n",
                cfg.maxScli, cfg.maxXUDTs, cfg.nsapTimeRes );
        printf( "maxAddrs   = %-5u  maxXUDTref  = %-5u  xrefTimeRes = %-5i\n",
                cfg.maxAddrs, cfg.maxXUDTref, cfg.xrefTimeRes );
        printf( "maxRtes    = %-5u  status      = %-5u  asmbTimeRes = %-5i\n",
                cfg.maxRtes, cfg.status, cfg.asmbTimeRes );
        printf( "DefRouting = %-5u  sogThresh   = %-5i\n",
                cfg.DefRouting, cfg.sogThresh );
        printf( "ConnDrop   = %-5u  scliTimeRes = %-5i\n",
                cfg.ConnDrop, cfg.scliTimeRes );
        PAUSE;
        PRT_TIMERCFG( "scliTimer",      &cfg.scliTimer );
        PRT_TIMERCFG( "sstTimer",       &cfg.sstTimer );
        PRT_TIMERCFG( "nsapTimer",      &cfg.nsapTimer );
        PRT_TIMERCFG( "srtTimer",       &cfg.srtTimer );
        PRT_TIMERCFG( "ignoreTimer",    &cfg.ignoreTimer );
        PRT_TIMERCFG( "coordTimer",     &cfg.coordTimer );
        PRT_TIMERCFG( "xrefFrzTimer",   &cfg.xrefFrzTimer );
        PRT_TIMERCFG( "defAsmbTmr",     &cfg.defAsmbTmr );
        printf( "\n" );
        printf( "maxConn    = %-5u  queueThresh = %-5i  frzTimeRes = %i\n",
                cfg.maxConn, cfg.queueThresh, cfg.frzTimeRes );
        printf( "connThresh = %-5i  conTimeRes  = %-5i\n",
                cfg.connThresh, cfg.conTimeRes );
        PAUSE;
        PRT_TIMERCFG( "freezeTimer",    &cfg.freezeTimer );
        PRT_TIMERCFG( "connTimer",      &cfg.connTimer );
        PRT_TIMERCFG( "txInactTimer",   &cfg.txInactTimer );
        PRT_TIMERCFG( "rxInactTimer",   &cfg.rxInactTimer );
        PRT_TIMERCFG( "relTimer",       &cfg.relTimer );
        PRT_TIMERCFG( "repRelTimer",    &cfg.repRelTimer );
        PRT_TIMERCFG( "intvalTimer",    &cfg.intvalTimer );
        PRT_TIMERCFG( "guardTimer",     &cfg.guardTimer );
        PRT_TIMERCFG( "resetTimer",     &cfg.resetTimer );
        PRT_TIMERCFG( "aicTimer",       &cfg.aicTimer );
        PRT_TIMERCFG( "aicRespTimer",   &cfg.aicRespTimer );
        PRT_TIMERCFG( "conRespTimer",   &cfg.conRespTimer );
        printf( "\n" );
        printf( "alarmLevel = %-5u  traceFlags = %-5u\n",
                cfg.alarmLevel, cfg.traceFlags );
        printf( "MemThresh1 = %-5u  MemThresh2 = %-5u MemThresh3 = %-5u\n",
                cfg.memThresh1, cfg.memThresh2, cfg.memThresh3 );
        break;
    case SCCP_TYPE_STATS :
        printf( "Board %d General Statistics:\n", NBoard );
        printf( "Redundancy State: %s %s\n",
                getString( LupTblRunState, stats.spRunState ),
                ( stats.spRunState == SCCP_STANDALONE ? "" : getString( LupTblMateState, stats.spMateState ) ) );
        printf( "MemCongLevel = %-10ld  InCongDisc  = %-10ld\n",
                stats.memCongLevel, stats.inCongDisc );
        printf( "rfNTASN      = %-10ld  rfUnknown   = %-10ld  msgHand = %-10ld\n",
                stats.rfNTASN, stats.rfUnknown, stats.msgHand );
        printf( "rfNTSA       = %-10ld  uDataTx     = %-10ld  msgLoc  = %-10ld\n",
                stats.rfNTSA, stats.uDataTx, stats.msgLoc );
        printf( "rfNetFail    = %-10ld  uDataSrvTx  = %-10ld  gttReq  = %-10ld\n",
                stats.rfNetFail, stats.uDataSrvTx, stats.gttReq );
        printf( "rfNetCong    = %-10ld  uDataRx     = %-10ld  msgTxC0 = %-10ld\n",
                stats.rfNetCong, stats.uDataRx, stats.msgTxC0 );
        printf( "rfSsnFail    = %-10ld  uDataSrvRx  = %-10ld  msgTxC1 = %-10ld\n",
                stats.rfSsnFail, stats.uDataSrvRx, stats.msgTxC1 );
        printf( "rfSsnCong    = %-10ld  xuDataTx    = %-10ld  msgRxC0 = %-10ld\n",
                stats.rfSsnCong, stats.xuDataTx, stats.msgRxC0 );
        printf( "rfUnequip    = %-10ld  xuDataSrvTx = %-10ld  msgRxC1 = %-10ld\n",
                stats.rfUnequip, stats.xuDataSrvTx, stats.msgRxC1 );
        printf( "rfHopViolate = %-10ld  xuDataRx    = %-10ld\n",
                stats.rfHopViolate, stats.xuDataRx );
        printf( "synError     = %-10ld  xuDataSrvRx = %-10ld\n",
                stats.synError, stats.xuDataSrvRx );
        break;
    }

    return;
} /* End of getReq */

/*  Help command handler
 *
 *  Displays help string based on command number in 1st parameter
 */
static void helpReq( DWORD *parms )
{
    printf( "%s", HelpTbl[*parms] );
}

/* Quit command handler
 *
 * Sets a flag to indicate to main loop to exit
 */
static void quit( DWORD *parms )
{
    NOT_USED( parms );
    BDone = TRUE;
}

/*
 * NSAP command handler
 *
 * First parameter indicates SAP number
 * Second parameter indicates type of information desired (CONFIG or STATS)
 * Third paramter is optional RESET flag
 */
static void nSapReq( DWORD *parms )
{
    U16             sapId = (U16) parms[0];
    S16             action = (S16) parms[1];
    U8              bReset = (U8) parms[2];
    SccpNSapCfg     cfg = {0};
    U32             i, j;
    char            *funcName = "Unknown";
    SCCP_STATUS     rtnCode = 0;
    SccpNSapStats   stats = {0};

    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        funcName = "SccpGetNSapCfg()";
        printf( "Calling SccpGetNSapCfg( %u, &cfg, %u )\n", NBoard, sapId );
        #if defined( NO_TX_BOARD )
            memset( &cfg, 0, sizeof( cfg ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetNSapCfg( NBoard, &cfg, sapId );
        #endif
        break;
    case SCCP_TYPE_STATS :
        funcName = "SccpGetNSapStats()";
        printf( "Calling SccpGetNSapStats( %u, &stats, %u, %u )\n", NBoard, sapId, bReset );
        #if defined( NO_TX_BOARD )
            memset( &stats, 0, sizeof( stats ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetNSapStats( NBoard, &stats, sapId, bReset );
        #endif
        break;
    }
    switch ( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT:
        fprintf( stderr, MsgTimeout );
        return;
    case SCCP_NOGENCFG :
        fprintf( stderr, MsgGenCfg );
        return;
    case SCCP_NOTFOUND :
        fprintf( stderr, MsgNotFound, sapId );
        return;
    default :
        fprintf( stderr, MsgMgmtApi, funcName, rtnCode );
        return;
    }

    /* Request successful, display results */
    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        printf( "Board %d Network SAP %d Configuration:\n", NBoard, sapId );
        printf( "swType    = %d\n", cfg.swType );
        printf( "dpcLen    = %u\n", cfg.dpcLen );
        printf( "dpc       = " );
        PRT_PC( cfg.dpc );
        printf( "maxMsgLen = %d\n", cfg.maxMsgLen );
        printf( "txQThr    = %d\n", cfg.txQThr );
        printf( "useMsk    = %u\n", cfg.useMsk );
        for ( j = 0; j < cfg.useMsk; j++ ) {
            U32 blen = cfg.addrMsk[j].length;
            size_t len;
            U8     *p = cfg.addrMsk[j].strg;
            char   buf[CFGT_MAXSTR];

            SccpBcdMaskToAscii(p,blen,buf,CFGT_MAXSTR);
            p = (U8 *)buf;
            len = strlen(buf);
            printf( "addrMsk.length = %u\n", len );
            printf( "addrMsk.strg   = " );
            for ( i = 0; i < len; i++ ) {
                printf( "%c", *p++ );
            }
            printf( "\n" );
        }
        printf( "mem       = %s", MsgNotUsed );
        printf( "selector  = %u\n", cfg.selector );
        printf( "dstProcId = %s", MsgNotUsed );
        printf( "priority  = %s", MsgNotUsed );
        printf( "route     = %s", MsgNotUsed );
        printf( "dstEnt    = %u\n", cfg.dstEnt );
        printf( "dstInst   = %u\n", cfg.dstInst );
        printf( "spId      = %d\n", cfg.spId );
        printf( "hopCnt    = %u\n", cfg.hopCnt );
        printf( "ssf       = %u\n", cfg.ssf );
        printf( "niInd     = %u (%s)\n", cfg.niInd, getString( LupTblNiInd, cfg.niInd ) );
        break;
    case SCCP_TYPE_STATS :
        printf( "Board %d Network SAP %d Statistics:\n", NBoard, sapId );
        printf( "ssAllTx  = %-10ld   ssAllRx  = %-10ld\n", stats.ssAllTx,  stats.ssAllRx  );
        printf( "ssOutGTx = %-10ld   ssOutGRx = %-10ld\n", stats.ssOutGTx, stats.ssOutGRx );
        printf( "ssOutRTx = %-10ld   ssOutRRx = %-10ld\n", stats.ssOutRTx, stats.ssOutRRx );
        printf( "ssProhTx = %-10ld   ssProhRx = %-10ld\n", stats.ssProhTx, stats.ssProhRx );
        printf( "ssStatTx = %-10ld   ssStatRx = %-10ld\n", stats.ssStatTx, stats.ssStatRx );
        printf( "connActv = %-10ld\n", stats.connActive );
        break;
    }

    return;
} /* End of NSapReq */

/*
 * ROUTE command handler
 *
 * First parameter indicates point code
 * Second parameter indicates type of information desired (CONFIG or STATUS)
 */
static void routeReq( DWORD *parms )
{
    U32             dpc = (U32) parms[0];
    S16             action = (S16) parms[1];
    SccpRouteCfg    cfg;
    SCCP_STATUS     rtnCode;
    short           i, j;
    short           rteOnline=0;

    /* both ROUTE CONFIG and ROUTE STATUS get their info from SccpGetRouteCfg() */
    rtnCode = SccpGetRteCfg( NBoard, &cfg, dpc );

    switch ( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT:
        fprintf( stderr, MsgTimeout );
        return;
    case SCCP_NOGENCFG :
        fprintf( stderr, MsgGenCfg );
        return;
    case SCCP_NOTFOUND :
        fprintf( stderr, MsgPCNotFound, CfgFmtPc( dpc, pcLen, pcDispFmt )  );
        return;
    default :
        fprintf( stderr, MsgMgmtApi, "SccpGetRteCfg", rtnCode );
        return;
    }

    /* Request successful, display results */
    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        printf( "\nBoard %d Route %s Configuration:\n", NBoard,
                CfgFmtPc( cfg.dpc, pcLen, pcDispFmt ) );
        printf( "Switch Type = %s\n",
                getString( LupTblSwType, (unsigned long) cfg.swType ) );
        printf( "Backup PC   = %s\n",
                (cfg.bkupPcInd) ? CfgFmtPc( cfg.bkupPc, pcLen, pcDispFmt ) : "None" );
        printf( "%s%s\n", (cfg.rteAltOpc[0].status & SCCP_ADJACENT) ? "Adjacent  " : "",
                (cfg.rteAltOpc[0].status & SCCP_TRANS) ? "Translator" : "" );
        for( i = 0; i < cfg.numAltOpc; i++)
                printf("ALT_OPC     = %s\n", CfgFmtPc( cfg.rteAltOpc[i].rteOpc, pcLen, pcDispFmt ));
        for ( i = 0; i < cfg.numSsns; i++ )
        {
            printf( "Subsystem %-3d\n", cfg.ssnList[i].ssn );
            printf( "   Backup PC     = %s\n",
                    (cfg.ssnList[i].bkupPcInd)
                        ? CfgFmtPc( cfg.ssnList[i].bkupPc, pcLen, pcDispFmt )
                        : "None" );
            printf( "   Concerned PCs = %s",
                    (cfg.ssnList[i].numConPc > 0)
                        ? CfgFmtPc( cfg.ssnList[i].conPcList[0], pcLen, pcDispFmt )
                        : "None" );
            for( j = 1; j < cfg.ssnList[i].numConPc; j++ )
                printf( ", %s", CfgFmtPc( cfg.ssnList[i].conPcList[j], pcLen, pcDispFmt ) );
            printf( "\n" );
        }
        printf( "\n" );
        break;
    case SCCP_TYPE_STATUS :
        if(cfg.numAltOpc == 0) {
            printf( "\nBoard %d Route %s Status: %sAccessable  %s\n", NBoard,
                    CfgFmtPc( cfg.dpc, pcLen, pcDispFmt ),
                    (cfg.rteAltOpc[0].status & SCCP_ONLINE) ? "" : "Not ",
                    (cfg.rteAltOpc[0].status & SCCP_CONG) ? "Congested" : "" );
            if(cfg.rteAltOpc[0].status & SCCP_ONLINE)
                    rteOnline = 1;

        }
        else {
            printf( "\nBoard %d Route %s Status: \n", NBoard,
                CfgFmtPc( cfg.dpc, pcLen, pcDispFmt ));

            for( i = 0; i < cfg.numAltOpc; i++) {
                printf("OPC: %s     Status: %sAccessible    %s\n",
                   CfgFmtPc( cfg.rteAltOpc[i].rteOpc, pcLen, pcDispFmt ),
                   (cfg.rteAltOpc[i].status & SCCP_ONLINE) ? "" : "Not ",
                   (cfg.rteAltOpc[i].status & SCCP_CONG) ? "Congested" : "" );
                if(cfg.rteAltOpc[i].status & SCCP_ONLINE)
                        rteOnline = 1;
            }
        }

        if( rteOnline )
        {
            for ( i = 0; i < cfg.numSsns; i++ )
            {
                printf( "Subsystem %-3d  %sAccessible\n",
                        cfg.ssnList[i].ssn,
                        (cfg.ssnList[i].status & SCCP_ACC) ? "    " : "Not " );
            }
        }
        break;
    }

    return;
} /* End of routeReq */

/*
 * USAP command handler
 *
 * First parameter indicates type of information desired (CONIFG, STATS, STATUS)
 * Second paramter is optional RESET flag
 */
static void uSapReq( DWORD *parms )
{
    U16             sapId = (U16) parms[0];
    S16             action = (S16) parms[1];
    U8              bReset = (U8) parms[2];
    SccpUSapCfg     cfg = {0};
    char            *funcName = "Unknown";
    size_t          i, j;
    U16             num, numOpc;
    SCCP_STATUS     rtnCode = 0;
    SccpUSapStats   stats = {0};
    SccpUSapStatus  status = {0};

    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        funcName = "SccpUSapCfg()";
        printf( "Calling SccpGetUSapCfg( %u, &cfg, %u )\n", NBoard, sapId );
        #if defined( NO_TX_BOARD )
            memset( &cfg, 0, sizeof( cfg ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetUSapCfg( NBoard, &cfg, sapId );
        #endif
        break;
    case SCCP_TYPE_STATS :
        funcName = "SccpUSapStats()";
        printf( "Calling SccpGetUSapStats( %u, &stats, %u, %u )\n", NBoard, sapId, bReset );
        #if defined( NO_TX_BOARD )
            memset( &stats, 0, sizeof( stats ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetUSapStats( NBoard, &stats, sapId, bReset );
        #endif
        break;
    case SCCP_TYPE_STATUS :
        funcName = "SccpUSapStatus()";
        printf( "Calling SccpGetUSapStatus( %u, &status, %u )\n", NBoard, sapId );
        #if defined( NO_TX_BOARD )
            memset( &status, 0, sizeof( status ) );
            rtnCode = SCCP_SUCCESS;
        #else
            rtnCode = SccpGetUSapStatus( NBoard, &status, sapId );
        #endif
        break;
    }

    switch ( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT:
        fprintf( stderr, MsgTimeout );
        return;
    case SCCP_NOGENCFG :
        fprintf( stderr, MsgGenCfg );
        return;
    case SCCP_NOTFOUND :
        fprintf( stderr, MsgNotFound, sapId );
        return;
    default :
        fprintf( stderr, MsgMgmtApi, funcName, rtnCode );
        return;
    }

    /* Request successful, display results */
    switch ( action ) {
    case SCCP_TYPE_CONFIG :
        printf( "Board %d User SAP %d Configuration:\n", NBoard, sapId );
        printf( "swType         = %i\n", cfg.swType );
        printf( "selector       = %u\n", cfg.selector );
        printf( "bkupPcInd      = %u\n", cfg.bkupPcInd );
        if ( cfg.bkupPcInd ) {
            PRT_PC( cfg.bkupPc );
        }

        numOpc = cfg.numAltOpc;
        if(numOpc == 0) {
            /* Print numConPc and each element in conPcList array. */
            num = cfg.usapAltOpc[0].numConPc;
            printf( "numConPc       = %u\n", num );

            for ( i = 0; i < num; i++ ) {
                printf( "conPcList[%u]  = ", i );
                PRT_PC( cfg.usapAltOpc[0].conPcList[i] );
            }
        }
        else {                          /* Multiple OPCs specified */
            printf( "numOpc         = %u\n", numOpc );
            for ( j = 0; j < numOpc; j++) {
                printf( "OPC            = ");
                PRT_PC( cfg.usapAltOpc[j].usapOpc );
                /* Print numConPc and each element in conPcList array. */
                num = cfg.usapAltOpc[j].numConPc;
                printf( "numConPc       = %u\n", num );
                for ( i = 0; i < num; i++ ) {
                    printf( "conPcList[%u]    = ", i );
                    PRT_PC( cfg.usapAltOpc[j].conPcList[i] );
                }
            }
        }

        printf( "useMsk         = %u\n", cfg.useMsk );
        for ( i = 0; i < cfg.useMsk; i++ ) {
            U32    blen = cfg.addrMsk[i].length;
            size_t len;
            U8     *p = cfg.addrMsk[i].strg;
            char   buf[CFGT_MAXSTR];

            blen *= 2;
            SccpBcdMaskToAscii(p,blen,buf,CFGT_MAXSTR);
            p = (U8 *)buf;
            len = strlen(buf);
            printf( "addrMsk.length = %u\n", len );
            printf( "addrMsk.strg   = ");
            for ( j = 0; j < len; j++ ) {
                printf( "%c", (char) *p++ );
            }
            printf( "\n" );
        }
        printf( "hopCnt         = %u\n", cfg.hopCnt );
        printf( "qThresh1 = %u   qThresh2 = %u   qThresh3 = %u\n", cfg.qThresh1, cfg.qThresh2, cfg.qThresh3);
        printf( "aicEnabled     = %s\n", getString( LupTblEnDis, cfg.aicEnabled ) );
        break;
    case SCCP_TYPE_STATS :
        printf( "Board %d User SAP %d Statistics:\n", NBoard, sapId );
        printf( "ssOOSReqGr = %-10ld  ssOOSreqDn = %-10ld  msgTxBSS = %-10ld\n",
                stats.ssOOSReqGr, stats.ssOOSReqDn, stats.msgTxBSS );
        printf( "inCongDisc = %-10ld  inCongRtnd = %-10ld\n",
                stats.inCongDisc, stats.inCongRtnd );
        printf( "outCongDisc = %-10ld  outCongRtnd = %-10ld\n",
                stats.outCongDisc, stats.outCongRtnd );
        break;
    case SCCP_TYPE_STATUS :
        printf( "Board %d User SAP %d Status:\n", NBoard, sapId );
        printf( "USAP status = %-5ld\n", status.sapStatus );
        for(i = 0; i<status.numAltOpc; i++)
            printf( "OPC = 0x%lx  status = %-5ld\n", status.opcInfo[i].opc, status.opcInfo[i].opcStatus );
        printf( "Queue Cong Level = %-5ld\n", status.inCongLevel );
        break;
    }

    return;
} /* End of uSapReq */

/*
 * Trace Control command handler
 *
 * Enables/Disables tracing.  First paramter is type of trace; second parameter
 * is the on/off switch.
 */
static void traceCtrl( DWORD *parms )
{
    U8          bReset = (U8) parms[0];
    /*U8          mask   = (U8) parms[0];*/
    SCCP_STATUS rtnCode;

    printf( "Calling SccpTraceControl( %u, %u, %u )\n", NBoard, bReset, SCCP_TRACE_DATA );

/* TRP - remove required TRACE DATA ON with just TRACE ON command */
#if defined( NO_TX_BOARD )
    rtnCode = SCCP_SUCCESS;
#else
    rtnCode = SccpTraceControl( NBoard, bReset, SCCP_TRACE_DATA );
#endif

    switch( rtnCode ) {
    case SCCP_SUCCESS :
        break;
    case SCCP_TIMEOUT :
        fprintf( stderr, MsgTimeout );
        break;
    default:
        fprintf( stderr, MsgMgmtApi, "SccpTraceControl()", rtnCode );
        break;
    }

    return;
}

/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */

/* getVal
 *
 * Serches look up table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 *
 * Returns:
 *   0      No match
 *   1      Match found
 */
static short getVal( LookUp *tbl, char *str, U32 *val )
{
    /* While not end of table */
    while ( tbl->name != NULL ) {
        if ( strcmp( tbl->name, str ) == 0 ) {
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
    } /* end while */

	/* falling through loop means search failed */
	return( 0 );
} /* end getVal() */

/* getString
 *
 * Serches look up table for a value match and generates the resulting
 * string if found; otherwise returns a pointer to a null-terminated
 * printable string indicating the numeric value.
 *
 * Returns:
 *   0      No match
 *   1      Match found
 */
static char *getString( LookUp *tbl, DWORD val )
{
    static char numVal[20];

    /* While not end of table */
    while ( tbl->name != NULL )
    {
        if ( tbl->value == val )
        {
			/* match found */
            return( tbl->name );
		}
		tbl++;
    } /* end while */

	/* falling through loop means search failed */
    sprintf( numVal, "%ld", val );
    return( numVal );
} /* end getVal() */


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of unsigned longs, and invokes the appropriate command handler.
 */
static void processCmd( char *tokens[], short cnt )
{
    short   i, j;
    U32     params[MAX_PARMS];  /* numeric parameters array */
    LookUp  *lp;
    short   retval;
    int     scan_param;

	/* first search command table for command match */
    for ( i = 0; CmdTable[i] != 0; i++ ) {
        if ( strcmp( tokens[0], CmdTable[i]->command ) == 0 ) {
			break;		/* command matched */
		}
    } /* end for */

	/* if no match found display error message and return */
    if( CmdTable[i] == 0 ) {
        fprintf( stderr, MsgCommand, tokens[0] );
		return;
	}

	/* convert remaining tokens (i.e., parameters) to short integers,
     * using look up tables  where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
    if ( cnt <= CmdTable[i]->nParmsReq  ) {
        fprintf( stderr, MsgNParms );
        fprintf( stderr, HelpTbl[i+1] );
        return;
    }

    for ( j = 0; j < MAX_PARMS; j++ ) {
		params[j] = 0;					/* first initialize to defaults */
    }

    for( j = 1; j < cnt; j++, lp++ ) {
		lp = CmdTable[i]->parmTbl[j - 1];
        if( lp ) {
            /* look up table for this param ? */
            if ( getVal( lp, tokens[j], &params[j - 1] ) == 1 ) {
				continue;				/* parameter succesfully converted */
			}
		}

        /* no look up table or look up failed - try to convert as a point code */
        if( (retval = CfgScanPc( tokens[j], &params[j - 1], pcDispFmt )) == 0 )
        {
            continue;                   /* conversion successful */
        }
        else if( retval == -2 )
        {
            /* point code format, but values out of range */
            fprintf( stderr, MsgPCrange, tokens[j] );
			return;
		}

        /* no look up table and not a point code - try to convert as a number */
        if( sscanf( tokens[j], "%i", &scan_param ) != 1 ) {
			/* conversion failed */
            fprintf( stderr, MsgParm, tokens[j] );
			return;
		}
        params[j - 1] = (DWORD)scan_param;
    } /* end for */

	/* call command handler function */

    if ( CmdTable[i]->func != 0 ) {
        CmdTable[i]->func( (DWORD *)params );
	}

    return;
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
static short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
    for ( cnt = 0; cnt <= MAX_PARMS; cnt++ ) {
		tokens[cnt] = 0;
    }

	/* begin scanning for tokens */
	cnt = 0;
    while ( *p != 0 && cnt <= MAX_PARMS ) {
		/* skip leading space or tabs */
        while ( *p == ' ' || *p == '\t' ) {
			p++;
        }

		/* beginning of token (or end of line) */
        if ( *p != 0 ) {
			/* scan for end of token, converting to upper case as we go */
			endp = p;
            while ( *endp != ' ' && *endp != '\t' && *endp != 0 ) {
                if ( islower( *endp ) ) {
                    *endp = (char) toupper( *endp );
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
            if( *endp == 0 ) {
				p = endp;	/* end of input - terminate scan */
            } else {
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
        }  /* End if *p != 0 */
    } /* End while *p != 0 .... */

	return( cnt );
}

/* --------------------------------------------------------------------------
 * main
 *   The main SccpMgr function scans the command line arguments for options,
 *   initializes the SCCP Management API library, and executes the command
 *   processing loop until a 'quit' command is received.
 *
 * Parameters:
 *
 * Returns:
 *   0      Successfull
 *   1      Error
 * -------------------------------------------------------------------------- */
int main( int argc, char *argv[] )
{
    char        buffer[256];
    short       cnt;
    SCCP_STATUS rtnCode;
    char        *tokens[MAX_PARMS +1];
    U16         tmpU16;

    printf( MsgBanner, SCCPMGR_VERSION, __DATE__ );

    /* Parse the command line for optional board number. */
    for ( cnt = 1; cnt < argc; cnt++ ) {
        int   option;
        char  *p = argv[cnt];

        if (*p++ != '-') {
            RETURN_USAGE;
        }
        option = tolower( *p++ );

        /* Move p to the option value, which is either immediately
         * following the option or is next argument.  */
        if ( *p != '\0' ) {
            ;
        } else if ( ++cnt < argc ) {
            p = argv[cnt];
        } else {
            RETURN_USAGE;
        }

        switch ( option ) {
        case 'b' :    /* Set Board number */
            if ( sscanf( p, "%hi", &tmpU16 ) != 1 ) {
                RETURN_USAGE;
            } else if ( ( tmpU16 < 1 ) || ( tmpU16 > 8 ) ) {
                RETURN_BOARD;
            }
            NBoard = (U8)tmpU16;
            break;
        case 'p' :    /* Set  point code format */
            if( (strcmp( p, "i" ) == 0) || (strcmp( p, "itu" ) == 0) ||
                (strcmp( p, "I" ) == 0) || (strcmp( p, "ITU" ) == 0) )
            {
                pcDispFmt = SS7_PCDISP_INTL;
                pcLen = PCLEN_ITU;
            }
            else if( (strcmp( p, "a" ) == 0) || (strcmp( p, "ansi" ) == 0) ||
                     (strcmp( p, "A" ) == 0) || (strcmp( p, "ANSI" ) == 0) )
            {
                pcDispFmt = SS7_PCDISP_DFLT;
                pcLen = PCLEN_ANSI;
            }
            else
            {
                RETURN_USAGE;
            }
            break;
        default:
            RETURN_USAGE;
        } /* End of switch */
    }   /* End of for */

    /* Initialize the SCCP Management API */
    printf( "Calling SccpMgmtInit( %u, SCCPMGR_ENT, 0 )\n", NBoard );
#if defined( NO_TX_BOARD )
    rtnCode = SCCP_SUCCESS;
#else
    rtnCode = SccpMgmtInit( NBoard, SCCPMGR_ENT, 0 );
#endif
    if ( rtnCode != SCCP_SUCCESS) {
        RETURN_MAPI( "SccpMgmtInit()", rtnCode );
    }

    /* Command processing loop */
    printf( MsgPrompt, NBoard );
    while ( fgets( buffer, 256, stdin ) != NULL ) {
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        cnt = tokenize( buffer, tokens );
        if ( cnt > 0 ) {
            processCmd( tokens, cnt );
        }
        if ( BDone ) {
            break;
        }
        printf( MsgPrompt, NBoard );
    } /* End of while */

    printf( "\n" );

    printf( "Calling SccpMgmtTerm( %u )\n", NBoard );
#if defined( NO_TX_BOARD )
    rtnCode = SCCP_SUCCESS;
#else
    rtnCode = SccpMgmtTerm( NBoard );
#endif
    if ( rtnCode != SCCP_SUCCESS ) {
        RETURN_MAPI( "SccpMgmtTerm()", rtnCode );
    }

    return( 0 );
} /* End of main() */
