/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 2008 - 2012 Dialogic Inc. All Rights Reserved.
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/******************************************************************************
 * NAME:  m3uaapp.c
 *
 * PURPOSE:
 *
 * Test app to send and receive M3UA packets and generate other M3UA API calls.
 * Always displays any packets (or other events) received. User hits <CR>
 * and can get a prompt to enter M3UA primitive command.
 *
 * To run:
 *
 *  m3uaapp  -o <opc> [-s <sio>] [-b <board>] [-n <nsap> ] [-?]
 *
 *      <opc>       originating point code to use for outgoing packets: required
 *
 *      <sio>       Service Information Octet value; default: 0x85 (ISUP)
 *
 *      <board>     Board number to use (defaults to 1)
 *
 *      <nsap>      M3UA network SAP number to bind to
 *
 * User Commands:
 *
 *      SEND <dpc>              # prompts for data to send
 *      FLOW [OFF | ON]
 *      APISTATS [RESET]
 *
*************************************************************************/
#ifdef UNIX
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
/* #include <sys/stream.h> */
#include <ctype.h>
#endif                                  /* UNIX */

#ifdef __OS2__
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#endif                                  /* __OS2__ */

#ifdef WIN32
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <ctype.h>
#endif                                  /* WIN32 */

#include "ss7type.h"
#include "m3uadef.h"
#include "m3uamapi.h"

#ifndef M3UA_SVCID
#define M3UA_SVCID        0x36      /* Service ID */
#endif
#define DFLT_ENT          0x14      /* Default application entity ID  JK was 0x20*/
#define DFLT_INST         0         /* PC is always board zero */
#define DFLT_SAP          0         /* defaults to first SAP in configuration file */
#define MAX_PACKET        272
#define MAX_PARMS         10

/* globals (with defaults) for parameter values */
U32         Dpc         = 0;        /* dest point code - no default */
U32         Opc         = 0;        /* my point code, no default */
U8          Sio         = 5;        /* SIO value to send (default national ISUP) */
U8          MyEnt       = DFLT_ENT; /* application entity ID, can be overridden to allow
                                     * multiple copies to run simultaneously */
S16         MySap       = 0;        /* M3UA NSAP to bind to (default 0) */
int         Board       = 1;        /* M3UA's Board Number */

/* other globals */
U8              Buf[MAX_PACKET];    /* packet buffer */
U32             PktXmitCnt = 0;     /* packet xmit count */
U32             ByteXmitCnt = 0;    /* byte xmit count */

CTAHD      FstCtaHd  = NULL_CTAHD;
CTAQUEUEHD FstQueue;

/* array of pont code status to printf string status*/
char *pcStatus[] = {"UNKNOWN", "PAUSE", "RESUME", "CONG", "USR UNAVAIL", "RESTART", "RESTART END", "CONG END", "PRIMARY", "BACKUP", "STANDALONE", "FLC START", "FLC END"};

#ifdef UNIX
#define _getch getchar
int _kbhit(void); /* Function prototype for UNIX */
#endif

void ErrorAndExit(DWORD  errId,
                  char  *str)
{
   printf( "Error : (%d) error returned in (%s) call.\n", errId, str );
   exit(1);
}

DWORD NMSSTDCALL ErrHdlr( CTAHD  ctahd,
                          DWORD  errorcode,
                          char  *errtxt,
                          char  *func )
{
    /* print out the error and exit */
    char text[40];
    ctaGetText(ctahd, errorcode, text, sizeof(text) );
    printf( "M3UA ErrorHandler: %s on ctahd=%#x, %s\n", errtxt, ctahd, func );
    return errorcode;
}

DWORD InitializeM3UA(void)
{
    CTA_EVENT        Event;
    DWORD            Ret;
    CTA_SERVICE_DESC m3uaOpenSvcLst[]   = {{{"M3UA", "M3UAMGR"}, {0}, {0}, {0}}};
    CTA_SERVICE_NAME m3uaServiceNames[] = {{ "M3UA", "M3UAMGR" }};
    CTA_INIT_PARMS   m3uaInitparms      = { 0 };
    CTA_REV_INFO     info;
    char tmpbuf[80];

    ctaGetVersion(&info, sizeof(CTA_REV_INFO));
    if (info.compatlevel != CTA_COMPATLEVEL)
    {
        printf("CTA compatibility level does not match. re-compile.\n");
        exit(1);
    }

    /* bind this application as a user of the M3UA layer */
    m3uaInitparms.size           = sizeof(CTA_INIT_PARMS);
    m3uaInitparms.traceflags     = CTA_TRACE_ENABLE;
    m3uaInitparms.parmflags      = CTA_PARM_MGMT_SHARED;
    m3uaInitparms.ctacompatlevel = CTA_COMPATLEVEL;
    m3uaInitparms.filename       = NULL;

    /* ctaSetErrorHandler(ErrHdlr, NULL); */

    Ret = ctaInitialize( m3uaServiceNames,
                         sizeof(m3uaServiceNames) / sizeof(m3uaServiceNames[0]),
                        &m3uaInitparms);
    if (Ret != M3UA_SUCCESS)
    {
        /* ctdaemon is not running.
        *  Initialize without tracing.
        */
        m3uaInitparms.traceflags = 0;
        m3uaInitparms.parmflags  = 0;

        Ret = ctaInitialize( m3uaServiceNames,
                             sizeof(m3uaServiceNames) / sizeof(m3uaServiceNames[0]),
                            &m3uaInitparms);
       ctaGetText( NULL_CTAHD, Ret, tmpbuf, sizeof(tmpbuf) );
       if (Ret != SUCCESS)
       {
            ErrorAndExit( Ret, "ctaInitialize" );
       }
    }

    Ret = ctaCreateQueue( NULL, 0, &FstQueue );
    if (Ret != M3UA_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateQueue" );
    }

    /* Open FstCtaHd on FstQueue                   */
    Ret = ctaCreateContext( FstQueue, 0, NULL, &FstCtaHd );
    if (Ret != M3UA_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaCreateContext" );
    }

    //MyEnt = DFLT_ENT + MySap;

    m3uaOpenSvcLst[0].svcargs.args[0] = Board;              /* board number              */
    m3uaOpenSvcLst[0].svcargs.args[1] = Sio;                /* Service Information Octet */
    m3uaOpenSvcLst[0].svcargs.args[2] = MySap;              /* network SAP number        */
    m3uaOpenSvcLst[0].svcargs.args[3] = MyEnt;              /* application entity ID     */
    m3uaOpenSvcLst[0].svcargs.args[4] = DFLT_INST;          /* Inst ID                   */
    m3uaOpenSvcLst[0].svcargs.args[5] = 0;                  /* Service user ID           */
    m3uaOpenSvcLst[0].svcargs.args[6] = 256;                /* Pool size                 */

    Ret = ctaOpenServices( FstCtaHd, m3uaOpenSvcLst, 1 );
    if (Ret != M3UA_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaOpenServices" );
    }

    /* Wait for service open to complete.*/
    Ret = ctaWaitEvent( FstQueue, &Event, 5000 );
    if (Ret != M3UA_SUCCESS)
    {
        ErrorAndExit( Ret, "ctaWaitEvent" );
    }

    if( Event.id == CTAEVN_OPEN_SERVICES_DONE )
    {
        if( Event.value != CTA_REASON_FINISHED )
        {
            printf("ERROR: Reason Value = %d ", Event.value);
            ErrorAndExit( Ret, "Reason value" );
        }
    }
    else
    {
        if( Event.id == CTAEVN_WAIT_TIMEOUT )
        {
            ErrorAndExit( Ret, "ctaWaitEvent: REASON = TIMEOUT " );
        }
        else
        {
            printf("ERROR: Open Services failed  id = %d ", Event.id);
            ErrorAndExit( Ret, "Open Services failed" );
        }
    }

    return(M3UA_SUCCESS);
}

DWORD TermM3UA(void)
{
    CTA_EVENT  Event;
    DWORD      Ret;
    char*      m3uaCloseSvcLst[] = {"M3UA"};

  /*
   *   Close service and destroy context.
   */
   Ret = ctaCloseServices( FstCtaHd, m3uaCloseSvcLst, 1 );
   if ( Ret != M3UA_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaCloseServices" );
   }

   /* Wait for service close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != M3UA_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_CLOSE_SERVICES_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyContext( FstCtaHd );

   if ( Ret != M3UA_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyContext" );
   }

   /* Wait for context close to complete. */
   Ret = ctaWaitEvent( FstQueue, &Event, (unsigned)CTA_WAIT_FOREVER );
   if ( Ret != M3UA_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }
   if (( Event.id != CTAEVN_DESTROY_CONTEXT_DONE )  || ( Event.value != CTA_REASON_FINISHED ))
   {
        ErrorAndExit( Ret, "ctaWaitEvent" );
   }

   Ret = ctaDestroyQueue( FstQueue );
   if ( Ret != M3UA_SUCCESS )
   {
        ErrorAndExit( Ret, "ctaDestroyQueue" );
   }

   return M3UA_SUCCESS;
}

/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
	 ByteCnt    #Bytes in Buf
OUT: <file>      gets multiple lines
*/
void showHex(unsigned char *Buf, unsigned short ByteCnt)
{
		unsigned short Li, LMax, ToDo, Bi, Ai;
		char Ascii[80];
		unsigned char B;

/* Dump to std out in hex followed by ascii */

for (ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax)
	{LMax = (ToDo > 16) ? 16 : ToDo;            /* max bytes this line */
	 printf("%04X| ", Bi);
	 Ai = 0;
	 for (Li = 0; Li < LMax; Li++)              /* each byte this line */
		 {B = *Buf++;                           /* next byte to show */
		  printf("%02X ", B);            		/* as hex */
		  if (!isprint(B))                      /* printable ascii? */
			 B = '.';                           /* no, use this */
		  Ascii[Ai++] = B;                      /* into ascii portiob */
		 }	/* for Li */
	 for (Li = LMax; Li < 16; Li++)             /* pad short ln if necessary */
		 printf("   ");                 		/* so asci lines up */
	 Ascii[Ai] = 0;                             /* terminate the ascii */
	 printf("    ");
	 printf("%s\n", Ascii);                     /* show the ascii */
	 Bi += LMax;
	}	/* for ToDo */

}	/* ShowHex */


/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
    for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
    {
		tokens[cnt] = 0;
    }

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}


/* cmdPrompt
 *
 * Prompts user to enter command, collects parameters, and issues appropriate
 * M3UA API command
 */
void cmdPrompt(void)
{
    U32     a1, a2, a3;                 /* individual point code components */
    U8      buffer[512];
    U8     *tokens[MAX_PARMS +1];
    S16     len;
    short   cnt;
    DWORD   status;
    S16     action;
    Bool    bReset = FALSE;
    M3UAAPISTATS ApiStats;
    unsigned int    scan_a1, scan_a2, scan_a3, scan_dpc, scan_len;

    /* prompt for a command */
    printf( "Command?>" );
    fgets((char*)buffer, 256, stdin);
    while ( (strlen((const char *)buffer) > 0) && (buffer[strlen((const char *)buffer) - 1] < ' ') )
        buffer[strlen((const char *)buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
    printf( "\n" );
    cnt = tokenize((char*)buffer, (char**)tokens );
    if( cnt == 0 )
        return;

    if( (strcmp( (const char *)tokens[0], "SEND" ) == 0) || (strcmp( (const char *)tokens[0], "S" ) == 0) )
    {
        if( cnt < 2 )
        {
            printf( "Point code expected\n" );
            return;
        }
        /* scan next token for valid DPC */
        if( (sscanf( (char*)tokens[1], "%u.%u.%u", &scan_a1, &scan_a2, &scan_a3 )) == 3 )
        {
            a1 = (U32)scan_a1;
            a2 = (U32)scan_a2;
            a3 = (U32)scan_a3;
            Dpc = ((a1 & 0xFF) << 16) + ((a2 & 0xFF) << 8) + (a3 & 0xFF);
        }
        else if( (sscanf( (char*)tokens[1], "%u", &scan_dpc )) == 1 )
        {
            Dpc = (U32)scan_dpc;
        }
        else
        {
            printf( "Parameter error [%s] - point code expected\n", tokens[1]);
            return;
        }

        /* prompt for message data */
        printf( "Message Data?>" );
        fgets((char*)buffer, 512, stdin);
        while ( (strlen((const char *)buffer) > 0) && (buffer[strlen((const char *)buffer) - 1] < ' ') )
            buffer[strlen((const char *)buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        len = (S16)strlen( (const char *)buffer );
        if (len > 268)
            len = 268;
        printf("%s len = %d\n", buffer, len );
        if( len > 0 )
        {
            /* send the data, including terminating null */
            status = M3UASendData( FstCtaHd, MySap, Opc, Dpc, (U8)0, (U8)1, (U8*)buffer, (S16)len, Sio );
            if( status != M3UA_SUCCESS )
            {
                printf( "M3uaSendData failed, status = %d\n", status );
            }
        }
    }
    else if( (strcmp( (const char *)tokens[0], "SENDLEN" ) == 0))
    {
        if( cnt < 2 )
        {
            printf( "Point code and length expected\n" );
            return;
        }
        /* scan next token for valid DPC */
        if( (sscanf( (char*)tokens[1], "%u.%u.%u", &scan_a1, &scan_a2, &scan_a3 )) == 3 )
        {
            a1 = (U32)scan_a1;
            a2 = (U32)scan_a2;
            a3 = (U32)scan_a3;
            Dpc = ((a1 & 0xFF) << 16) + ((a2 & 0xFF) << 8) + (a3 & 0xFF);
        }
        else if( (sscanf( (char*)tokens[1], "%u", &scan_dpc )) == 1 )
        {
            Dpc = (U32)scan_dpc;
        }
        else
        {
            printf( "Parameter error [%s] - point code expected\n", tokens[1]);
            return;
        }

        if (cnt < 3)
        {
            printf( "Length expected\n" );
            return;
        }

        if( (sscanf( (char*)tokens[2], "%u", &scan_len )) != 1 )
        {
            printf( "Parameter error [%s] - numeric length expected\n", tokens[2]);
            return;
        }
        len = (S16)scan_len;
        if (len < 1 || len > 268)
        {
            printf ("Length must be from 1 to 268 bytes\n");
            return;
        }

        strcpy ((char *)buffer, "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"
                "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890"
                "12345678901234567890123456789012345678901234567890123456789012345678");
        buffer[len] = 0;
        printf("%s len = %d\n", buffer, len );

        /* send the data, including terminating null */
        status = M3UASendData( FstCtaHd, MySap, Opc, Dpc, (U8)0, (U8)1, (U8*)buffer, len, Sio );
        if( status != M3UA_SUCCESS )
        {
            printf( "M3uaSendData failed, status = %d\n", status );
        }
    }
    else if( (strcmp( (const char *)tokens[0], "FLOW" ) == 0) || (strcmp( (const char *)tokens[0], "F" ) == 0) )
    {
        /* scan next token for valid DPC */
        if( cnt < 2 )
        {
            printf( "FLOW OFF or ON expected\n" );
            return;
        }
        if( strncmp( (const char *)tokens[1], "ON", 2 ) == 0 )
            action = 1;
        else if( strncmp( (const char *)tokens[1], "OFF", 3 ) == 0 )
            action = 0;
        else
        {
            printf( "FLOW OFF or ON expected\n" );
            return;
        }
        status = M3UAFlow( FstCtaHd, MySap, action );
        if( status != M3UA_SUCCESS )
        {
            printf( "M3uaFlow failed, status = %d\n", status );
        }
    }
    else if( (strcmp( (const char *)tokens[0], "APISTATS" ) == 0) || (strcmp( (const char *)tokens[0], "A" ) == 0) )
    {
        if (cnt >= 2)
        {
            /* Check for RESET */
            if (strncmp((const char *)tokens[1], "RESET", 5) == 0)
            {
                bReset = TRUE;
            }
        }
        status = M3UAGetApiStats( FstCtaHd, &ApiStats, bReset);
        if ( status != M3UA_SUCCESS )
        {
            ErrorAndExit( status, "M3UAGetApiStats" );
        }
        printf ("API STATS\n");
        printf ("Curr: %d, Peak: %d, RxSucc: %d, RxFail: %d\n",
            ApiStats.qCount, ApiStats.qPeak, ApiStats.rxSuccess, ApiStats.rxFailed);
        printf ("Pend: %d, Peak: %d, TxSucc: %d, TxFail: %d\n",
            ApiStats.txPending, ApiStats.txPendPeak, ApiStats.txSuccess, ApiStats.txFailed);
        printf ("LastErr: %d, currCongLevel: %d\n\n",
            ApiStats.txLastErr, ApiStats.apiQCongLvl);
    }
    else if ( (strcmp( (const char *)tokens[0], "PCSTAT" ) == 0) || (strcmp( (const char *)tokens[0], "PS" ) == 0) )
    {
        if (cnt < 2 )
        {
            printf("PCSTAT Command requires Point Code\n");
            return;
        }
        if( (sscanf( (char*)tokens[1], "%u.%u.%u", &scan_a1, &scan_a2, &scan_a3 )) == 3 )
        {
            a1 = (U32)scan_a1;
            a2 = (U32)scan_a2;
            a3 = (U32)scan_a3;
            Dpc = ((a1 & 0xFF) << 16) + ((a2 & 0xFF) << 8) + (a3 & 0xFF);
        }
        else if( (sscanf( (char*)tokens[1], "%u", &scan_dpc )) == 1 )
        {
            Dpc = (U32)scan_dpc;
        }
        else
        {
            printf( "Parameter error [%s] - point code expected\n", tokens[1]);
            return;
        }
        status = M3UAPcStatus( FstCtaHd, MySap, Dpc );
        if ( status != M3UA_SUCCESS )
        {
            printf("Point Code Status Request Failed: %d \n", status);
            return;
        }
    }
    else
    {
        printf( "SEND, SENDLEN, FLOW, PCSTAT or APISTATS command expected\n" );
    }
}

/* m3uaHandler
 *
 * M3UA event handler routine, to be called by "tx_core" whenever an asynchronous
 * event message from M3UA is pending.
 */
DWORD m3uaHandler(void)
{
    DWORD   status;
    U16     len;
    union
    {
        U8          msgCode;    /* overlays "code" field of DATA_IND/STAT_IND */
        DATA_IND    dataind;    /* incoming data event */
        STAT_IND    statind;    /* incoming status event */
        ASSOC_IND   assocind;   /* Association state change indication */
        BIND_CFM    bindcfm;    /* Bind confirmation */
        STATUS_CFM  staCfm;     /* Status Confrm */
    } msg;

    len = sizeof( msg );
    status = M3UARetrieveMessage( FstCtaHd, (void *) &msg, &len );
   /* showHex((unsigned char*) &msg, len); */
    if( status == M3UA_NO_MSG )
    {
        /* "NO_MSG" unexpected, but not fatal */
        printf( "Warning: M3uaRetrieveMessage returned NO_MSG\n" );
        return status;
    }

    if( status != M3UA_SUCCESS )
    {
        printf( "M3uaRetrieveMessage failed, status %d\n", status );
        return status;  /* fatal */
    }

    /* got an incoming message, decode it */
    switch( msg.msgCode )
    {
        case M3UA_BIND_CFM:
            printf ("Bind Confirm received.  spId = %d, suId = %d\n",
                msg.bindcfm.spId, msg.bindcfm.suId);
            break;

        case M3UA_DATA_IND:
            /* Subtract off size of header fields to get actual data length */

            /* Note that the header is packed, spares in the structure are not included.  Also the message "code" field
             * in the DATA_IND structure has already been stripped (and placed into the msgCode field) and is not included
             * in the length returned from M3UARetrieveMsg().  Therefore it is not included in the DATA_IND_HDR_SIZE define
             * either. */
            len -= DATA_IND_HDR_SIZE;           /* Changed from -= (sizeof(DATA_IND) - MAXDATA) due to lack of spare packing */
            printf( "Packet received from 0x%x, length %d\n", msg.dataind.opc, len );
            showHex( msg.dataind.data, (unsigned short) len );
            printf( "\n" );
            break;

        case M3UA_STAT_CFM:
            printf("DPC          0x%X\n", msg.staCfm.dpc);
            printf("Status       %s\n", (msg.staCfm.status < 13 ? pcStatus[msg.staCfm.status] : "N/A"));
            printf("CongLevel    %d\n", msg.staCfm.congLevel);
        break;

        case M3UA_STAT_IND:
            /* print out status indication message */
            switch( msg.statind.status )
            {
                case StatPaused:
                    printf( "M3UA Pause: DPC=0x%x, OPC=0x%x\n", msg.statind.pc, msg.statind.opc );
                    break;

                case StatResumed:
                    printf( "M3UA Resume: DPC=0x%x, OPC=0x%x\n", msg.statind.pc, msg.statind.opc );
                    break;

                case StatCongested:
                    printf( "M3UA Destination Congested 0x%x, Priority %d\n",
                            msg.statind.pc, msg.statind.priority );
                    break;

                case StatUsrUnavail:
                    printf( "M3UA Remote User Unavailable: DPC=0x%x, OPC=0x%x\n", msg.statind.pc, msg.statind.opc );
                    break;

                case StatRestart:
                    printf( "M3UA Restart Begins\n" );
                    break;

                case StatRestartEnds:
                    printf( "M3UA Restart Ends\n" );
                    break;

                case StatCongestionEnds:
                    printf( "M3UA Destination Congestion Ended 0x%x, Priority %d\n",
                            msg.statind.pc, msg.statind.priority );
                    break;

                case StatPrimary:
                    printf( "M3UA Now Primary\n" );
                    break;

                case StatBackup:
                    printf( "M3UA Now Backup\n" );
                    break;

                case StatStandAlone:
                    printf( "M3UA Now Stand-Alone\n" );
                    break;

                case StatAssociation:                    
                    printf( "M3UA Association %d: state %s \n", msg.assocind.associd, 
                    (msg.assocind.assoc_state == M3UA_ASSOC_DOWN) ? "DOWN" : "ACTIVE");
                    break;

                default:
                    printf( "Unknown MTP Status indication received, status %d, DPC=0x%x, OPC=0x%x \n",
                            msg.statind.status, msg.statind.pc, msg.statind.opc );
                    break;
            }                   /* switch( msg.statind.status ) */
            break;

        default:
            printf( "Unknown message received, msgcode %d\n",  msg.msgCode );
            break;
    }                           /* switch( msg.msgcode ) */

    return (M3UA_SUCCESS);
}


/*
 * Main routine
 *
 * Parse command line arguments, start main processing loop
 */
int main( int argc, char *argv[] )
{
    int        i, j, k;
    int        sio;
    DWORD      Ret;
    CTA_EVENT  Event;
    short      n;
    char      *ap;
    int        c;
    short      bOpc   = 0;   /* Has required OPC been specified */
    U8         inloop = 1;

	/* process arguments */
    for (n = 1; n < argc; n++)
    {
		ap = argv[n];
        if (*ap == '-')
        {
            switch (*++ap)
            {
            case 'o' :     /* OPC */
				n++;
				i = j = k = -1;
				sscanf(argv[n], "%d.%d.%d", &i, &j, &k);
                if (i == -1 || j == -1 || k == -1)
                {
                    printf( "Invalid originating point code\n");
					exit( 1 );
				}
                Opc = (((U32) i << 16) + ((U32) j << 8) + (U32) k);
                bOpc = 1;           /* They specified required OPC */
				break;

			case 's' :     /* SIO */
				n++;
				sscanf(argv[n], "%i", &sio);
                if ( sio < 0 || sio > 255 )
                {
                    printf("SIO must be in range 0 - 0xFF\n");
					exit( 1 );
				}
				Sio = (unsigned char) sio;
				break;

            case 'b' :     /* board number */
				n++;
                Board = atoi(argv[n]);
                if( Board < 1 || Board > 8 )
                {
                    printf("Board must be in range 1 - 8\n");
					exit(1);
				}
				break;

            case 'n' :     /* NSAP number */
				n++;
                MySap = (S16)atoi(argv[n]);
                if( MySap < 0 || MySap > 15 )
                {
                    printf("NSAP must be in range 0-15\n");
					exit(1);
				}
                break;

            case '?' :      /* Help */
                printf ("m3uaapp  -o <opc> [-s <sio>] [-b <board>] [-n <nsap> ] -?\n");
                printf ("    <opc>       originating point code to use for outgoing packets: required\n");
                printf ("    <sio>       Service Information Octet value; default: 0x85 (ISUP)\n");
                printf ("    <board>     Board number to use (defaults to 1)\n");
                printf ("    <nsap>      MTP network SAP number to bind to\n");
                break;

			default:
				printf("Unrecogized Parameter: %s\n", argv[n]);
				exit(1);

			}	/* end switch */
        }
        else
        {
			printf("Unrecogized Parameter: %s\n", argv[n]);
			exit(1);
		}
	}	/* end for loop */


    if (!bOpc)
    {
        printf ("Required parameter OPC not specified\n");
        exit (1);
    }

    /* initialize API */
    if ( (Ret = InitializeM3UA()) != M3UA_SUCCESS )
    {
        fprintf(stderr, "Error %d initializing API\n", Ret);
        exit(1);
    }

    while ( inloop )
    {
        /* cta wait event */
        Ret = ctaWaitEvent( FstQueue, &Event, 1000 );
        if ( Ret != M3UA_SUCCESS )
        {
            ErrorAndExit( Ret, "ctaWaitEvent" );
        }

        switch ( Event.id )
        {
            case M3UAEVN_DATA:
                /* an M3UA event has occured, call M3UA Retrieve Message from m3uaHandler() to get the event */
                m3uaHandler();
                break;

            case CTAEVN_WAIT_TIMEOUT:
                /* 1 seconds passed, check for keyboard input */
                break;
        }

        /* make sure a key was hit */
        if (_kbhit())
        {
           c = _getch();
           if (c == 'q' || c == 'Q')
           {
              inloop = 0;
              break;
           }
           else
           {
              cmdPrompt();
           }
        }
    }
    /* when user has selected "quit" */
    TermM3UA();

} /* end of main */

#ifdef UNIX
/******************************************************************************
 * Function:   _kbhit
 * Returns:
 * Notes:      Implements WinNT-like _kbhit
 */
int _kbhit(void)
{
   int RetCode = 0;
   struct pollfd kbdfd;

   kbdfd.fd = 0; /*input*/
   kbdfd.events = POLLIN;
   kbdfd.revents = 0;

   RetCode = poll(&kbdfd,1,100);
   if (RetCode <= 0) {
      RetCode = 0; /*timeout*/
   } else {
      RetCode = 1;
   }

   return (RetCode);
}
#endif

