/*****************************************************************************
 * m3uamgr.c - M3UA management test application
 *
 * Sample M3UA management application; sends commands to task on TX series
 * processor and waits for response (status and optional data, such as entity
 * status or statistics).
 *
 * To run:
 *
 *  m3uamgr  [-b <board>]
 *
 *****************************************************************************/

/*****************************************************************************
 **
 **                         PROPRIETARY NOTICE
 **
 **  Copyright (C) 2008 - 2012 Dialogic Inc. All Rights Reserved.
 **
 **
 **  The following is licensed program material and is considered
 **  proprietary and confidential.  No part of this program may be
 **  reproduced or used in any form or by any means without the
 **  express written permission of Dialogic Corporation.
 **
 *****************************************************************************/

#include <stdio.h>
#include <ctype.h>
#ifdef UNIX
#define getch	getchar
#else
#include <conio.h>          /* For getch() call */
#endif
#include <string.h>
#include <stdlib.h>
#include "ss7type.h"
#include "sigerr.h"
#include "cfgscan.h"                    /* for point code display and scanning utilities */
#include "m3uamapi.h"
#include "m3uaver.h"
#include "osdep.h"

/* Globals */
Bool forever = TRUE;
short cp = 1;
Bool bDebug = FALSE;

/* These may move to an .h file */
/**************************************************/
#define SYS_STA         1
#define M3UA_STA        2
#define NSAP_STA        3
#define ADDR_TRANS_STA  4
#define DPC_STA         5
#define NWK_STA         6
#define RTE_STA         7
#define PS_STA          8
#define PSP_STA         9
#define SCTSAP_STA      10
#define RK_STA          11  /* Dynamic Routing Key Status */
#define PSPRK_STA       12
#define M3UA_STS        13
#define NSAP_STS        14
#define SCTSAP_STS      15
#define PSP_STS         16
#define MAX_PARMS       10

#define CMD_QUIT        0
#define CMD_HELP        100
#define CMD_STATS       101
#define CMD_STATUS      102
#define CMD_BOARD       103
#define CMD_ALARM       104
#define CMD_DEBUG       105
#define CMD_TRACE       106
#define CMD_SCTSAP      107
#define CMD_ASSOC       108
#define CMD_ASP         109
#define CMD_RK          110
#define CMD_TOS         111

#define UNKNOWN         255

/*---------------------------------------------------------------------------*
 * cmdTbl - command table entry
 *---------------------------------------------------------------------------*/
struct cmdTbl
{
    char   *name;
    U16     val;
};

/*---------------------------------------------------------------------------*
 * cmdList - main command list
 *---------------------------------------------------------------------------*/
struct cmdTbl cmdList[] =
{
    {   "?",        CMD_HELP    },
    {   "STATS",    CMD_STATS   },
    {   "STATUS",   CMD_STATUS  },
    {   "BOARD",    CMD_BOARD   },
    {   "ALARM",    CMD_ALARM   },
    {   "DEBUG",    CMD_DEBUG   },
    {   "TRACE",    CMD_TRACE   },
    {   "SCTSAP",   CMD_SCTSAP  },
    {   "ASSOC",    CMD_ASSOC   },
    {   "ASP",      CMD_ASP     },
    {   "RK",       CMD_RK      },
    {   "TOS",      CMD_TOS     },
    {   "Q",        CMD_QUIT    },
    {   "UNKNOWN",  UNKNOWN     }
};

/*---------------------------------------------------------------------------*
 * staList - status command types
 *---------------------------------------------------------------------------*/
struct cmdTbl staList[] =
{
    {   "SYS",      SYS_STA         },
    {   "M3UA",     M3UA_STA        },
    {   "NSAP",     NSAP_STA        },
    {   "ADDR",     ADDR_TRANS_STA  },
    {   "DPC",      DPC_STA         },
    {   "NWK",      NWK_STA         },
    {   "ROUTE",    RTE_STA         },
    {   "PS",       PS_STA          },
    {   "PSP",      PSP_STA         },
    {   "SCTSAP",   SCTSAP_STA      },
    {   "RK",       RK_STA          },
    {   "PSPRK",    PSPRK_STA       },
    {   "UNKNOWN",  UNKNOWN         }
};

/*---------------------------------------------------------------------------*
 * stsList - statistics command types
 *---------------------------------------------------------------------------*/
struct cmdTbl stsList[] =
{
    {   "M3UA",     M3UA_STS    },
    {   "NSAP",     NSAP_STS    },
    {   "SCTSAP",   SCTSAP_STS  },
    {   "PSP",      PSP_STS     },
    {   "UNKNOWN",  UNKNOWN     }
};

/*---------------------------------------------------------------------------*
 * sctsapCtrlList - SAP control operations
 *---------------------------------------------------------------------------*/
struct cmdTbl sctsapCtrlList[] =
{
    {   "ENA",      M3UA_CTRL_SCTSAP_ENA    },
    {   "DIS",      M3UA_CTRL_SCTSAP_DIS    },
    {   "UNKNOWN",  UNKNOWN                 }
};

/*---------------------------------------------------------------------------*
 * assocCtrlList - Association control operations
 *---------------------------------------------------------------------------*/
struct cmdTbl assocCtrlList[] =
{
    {   "EST",      M3UA_CTRL_ASSOC_EST     },
    {   "TERM",     M3UA_CTRL_ASSOC_TERM    },
    {   "INH",      M3UA_CTRL_ASSOC_INH     },
    {   "UNI",      M3UA_CTRL_ASSOC_UNI     },
    {   "UNKNOWN",  UNKNOWN                 }
};

/*---------------------------------------------------------------------------*
 * aspCtrlList - ASP control operations
 *---------------------------------------------------------------------------*/
struct cmdTbl aspCtrlList[] =
{
    {   "UP",       M3UA_CTRL_ASP_UP    },
    {   "DN",       M3UA_CTRL_ASP_DN    },
    {   "AC",       M3UA_CTRL_ASP_AC    },
    {   "IA",       M3UA_CTRL_ASP_IA    },
    {   "UNKNOWN",  UNKNOWN             }
};

/*---------------------------------------------------------------------------*
 * rkCtrlList
 *---------------------------------------------------------------------------*/
struct cmdTbl rkCtrlList[] =
{
    {   "REG",      M3UA_CTRL_RK_REG    },
    {   "DEREG",    M3UA_CTRL_RK_DEREG  },
    {   "UNKNOWN",  UNKNOWN             }
};

/*---------------------------------------------------------------------------*
 * debugCtrlList
 *---------------------------------------------------------------------------*/
struct cmdTbl debugCtrlList[] =
{
    {   "ENA",      M3UA_CTRL_DEBUG_ON  },
    {   "DIS",      M3UA_CTRL_DEBUG_OFF },
    {   "UNKNOWN",  UNKNOWN             }
};

/*---------------------------------------------------------------------------*
 * traceCtrlList
 *---------------------------------------------------------------------------*/
struct cmdTbl traceCtrlList[] =
{
    {   "ENA",      M3UA_CTRL_TRACE_ON  },
    {   "DIS",      M3UA_CTRL_TRACE_OFF },
    {   "UNKNOWN",  UNKNOWN             }
};

/*---------------------------------------------------------------------------*
 * err2Str
 *---------------------------------------------------------------------------*/
struct cmdTbl err2Str[] =
{
    {   "Invalid Parameter",    M3UA_PARAM      },
    {   "Invalid Board",        M3UA_BOARD      },
    {   "Driver Error",         M3UA_DRIVER     },
    {   "Uninitialized Board",  M3UA_HANDLE     },
    {   "Request Timeout",      M3UA_TIMEOUT    },
    {   "Invalid Response",     M3UA_RESPONSE   },
    {   "Invalid Argument(s)",  14              },
    {   "Invalid SAP ID ",      12              },
    {   "Invalid PS Id ",       20              },
    {   "Invalid PSP Id ",      258             },
    {   "Invalid Network Id",   259             },
    {   "Invalid DPC/Network",  284             },
    {   "Unknown Error",        UNKNOWN         }
};

/*---------------------------------------------------------------------------*
 * Other Global String Arrays
 *---------------------------------------------------------------------------*/
char *sapStatus[]       = { "UNKNOWN","UNBOUND", "WAIT_BND", "BOUND", "WAIT_OPEN", "READY" };
char *suTypeStr[]       = { "N/A(0)", "N/A(1)", "N/A(2)", "SCCP", "TUP", "ISUP", "N/A(6)",
                            "N/A(7)", "N/A(8)", "N/A(9)", "N/A(10)", "N/A(11)", "N/A(12)", "BICC" };
char *dpcSta[]          = { "UNKNOWN", "AVAILABLE","CONGESTED", "UNAVAILABLE", "RESTRICTED" };
char *psSta[]           = { "UNKNOWN", "DOWN", "INACTIVE", "ACTIVE", "PENDING" };
char *aspSta[]          = { "UNSUP", "DOWN","INACTV", "ACTV" };
char *assocSta[]        = { "UNKNWN", "DOWN", "ACTIVE", "CONGSTRT", "CONGDROP" };
char *pspType[]         = { "LOCAL", "ASP", "SGP", "IPSP" };
char *nwkSsf[]          = { "INTL", "SPARE", "NAT", "RSVRD" };
char *nwkDpcLen[]       = { "DPC14", "DPC24", "DFLT", "DPC16" };
char *nwkSwtch[]        = { "N/A", "ANSI", "ITU","N/A", "CHINA", "BICI", "ANS96", "N/A", "N/A", "N/A", "TTC", "NTT" };
char *highAvailState[]  = { "STANDALONE", "STARTING", "PRIMARY", "BACKUP" };

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*
 * LOCAL FUNCTION PROTOTYPES
 *+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void    quit                ( void );
char   *getErrStr           ( S16 err );
void    HelpCommand         ( char *args[] );
void    setCP               ( char *args[] );
void    statusReq           ( char *args[] );
void    statsReq            ( char *args[] );
void    doGenStats          ( char *args[] );
void    doNSapStats         ( char *args[] );
void    doSctSapStats       ( char *args[] );
void    doPspStats          ( char *args[] );
void    doGenStatus         ( char *args[] );
void    doNSapStatus        ( char *args[] );
void    doAddrTransStatus   ( char *args[] );
void    doDpcStatus         ( char *args[] );
void    doRouteStatus       ( char *args[] );
void    doPsStatus          ( char *args[] );
void    doPspStatus         ( char *args[] );
void    doSctSapStatus      ( char *args[] );
void    doRouteKeyStatus    ( char *args[] );
void    doPspRouteStatus    ( char *args[] );

/*****************************************************************************
 * HelpCommand
 *****************************************************************************/
void HelpCommand( char *args[] )
{
    UNREFERENCED_PARAMETER(args);
    /* cols:          1         2         3         4         5         6         7         8
    /*       12345678901234567890123456789012345678901234567890123456789012345678901234567890 */
    printf( "\nStatus commands\n" );
    printf( "---------------\n" );
    printf( "STATUS M3UA                    - M3UA General Status\n" );
    printf( "STATUS NSAP <sapId>            - NSAP Status for <sapid>\n" );
    printf( "STATUS ADDR                    - Address Translation Table Status\n" );
    printf( "STATUS DPC <nwkId> <dpc>       - DPC Status\n" );
    printf( "STATUS NWK <nwkId>             - Network Configuration Status\n" );
    printf( "STATUS PS <psId>               - Peer Server Status\n" );
    printf( "STATUS PSP <pspId>             - Peer Server Process Status\n" );
    printf( "STATUS SCTSAP <sapId>          - SCTSAP Status\n" );
    printf( "STATUS RK                      - Dynamic Routing Key Status\n" );
    printf( "STATUS PSPRK <pspId>           - Routing Key Request sent to PSP\n" );

    printf( "\nStatistics commands\n" );
    printf( "-------------------\n" );
    printf( "STATS  M3UA [RESET]            - Display General Statistics [optionally reset]\n" );
    printf( "STATS  NSAP <sapId> [RESET]    - Display NSAP Statistics [optionally reset]\n" );
    printf( "STATS  SCTSAP <sapId> [RESET]  - Display SCTSAP Statistics [optionally reset]\n" );
    printf( "STATS  PSP <pspId> [RESET]     - Display PSP Statistics [optionally reset]\n" );

    printf( "\nControl commands\n" );
    printf( "----------------\n" );
    printf( "SCTSAP ENA|DIS                 - Enable or Disable the Service Access Point\n" );
    printf( "ASSOC  EST|TERM|INH|UNI <pspId>- Perform an action on an association\n" );
    printf( "ASP    UP|DN|AC|IA <pspId>     - Perform an action on an ASP\n" );
    printf( "RK     REG|DEREG               - Register or Deregister Dynamic Routing Keys\n" );
    printf( "TOS    <pspId> <tosVal>        - Set Type of Service for an association\n" );
    printf( "DEBUG  ENA|DIS                 - Enable or Disable Debug Logging\n" );
    printf( "TRACE  ENA|DIS                 - Enable or Disable Data Tracing\n" );
    printf( "BOARD  <boardNum>              - Switch current board\n" );
    printf( "Q                              - Quit M3UA manager\n" );
    printf( "?                              - Show Help\n" );
}

/*****************************************************************************
 * u32ToIp - Convert IP to dot format
 *****************************************************************************/
void u32ToIp( U32 addr, char *ipStr )
{
    sprintf( ipStr, "%d.%d.%d.%d",
             (addr & 0xFF000000) >> 24, (addr & 0x00FF0000) >> 16,
             (addr & 0x0000FF00) >> 8, (addr & 0x000000FF) );
}

/*****************************************************************************
 * getErrStr - Convert Error Code to String
 *****************************************************************************/
char *getErrStr( S16 err )
{
    short   i;

	for (i = 0; err2Str[i].val != UNKNOWN; i++)
	{
		if (err2Str[i].val == err)
        {
            return( err2Str[i].name );
        }
	}

    return( "Unknown Error" );
}

/*****************************************************************************
 * setCP
 *****************************************************************************/
void setCP( char *args[] )
{
    short   newCp;

    if (args[0] != NULL)
    {
        newCp = (short) atoi( args[0] );
    }
    else
    {
        printf( "Invalid Board number\n" );

        return;
    }

    if ( newCp != cp )
    {
        if ( (newCp > 8) || (newCp < 1) )
        {
            printf( "Board %d out of Range. Must be in range 1..8\n", newCp );

            return;
        }

        if ( M3uaMgmtInit( (U8)newCp, ENTIT, 0 ) == M3UA_SUCCESS )  /* Use M3UA entity ID for MGR since
                                                                     * cfg & task using ENTSN (mtp3)*/
        {
            M3uaMgmtTerm( (U8)cp );
            cp = newCp;
        }
        else
        {
            printf( "Board initialization failed\n" );
        }
    }
}

/*****************************************************************************
 * quit
 *****************************************************************************/
void quit( void )
{
	forever = FALSE;
}

/*****************************************************************************
 * statsReq - Handle all statistics requests (calling relevant sub-function)
 *****************************************************************************/
void statsReq( char * args[] )
{
    short   i;
    Bool    found = FALSE;

    /* first search Ent table for match */
    found = FALSE;
    if ( args[0] == NULL )
    {
        printf( "Invalid Stats Request.\n" );
        printf( "Usage: STATS <type>\n" );
        printf( "  where <type> = M3UA | NSAP | SCTSAP | PSP\n" );

        return;
    }

    for (i = 0; stsList[i].val != UNKNOWN; i++)
    {
        if (strcmp( args[0], stsList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }

    if ( !found )
    {
        printf( "Error: Unknown STATS Command (%s)\n", args[0] );
    }

    switch (stsList[i].val)
    {
        case M3UA_STS:
            doGenStats( (char **)&args[1] );
            break;

        case NSAP_STS:
            doNSapStats( (char **)&args[1] );
            break;

        case SCTSAP_STS :
            doSctSapStats( (char **)&args[1] );
            break;

        case PSP_STS:
            doPspStats( (char **)&args[1] );
            break;

        default:
            printf( "Error: Unknown STATS Command!\n" );
    }
}

/*****************************************************************************
 * doGenStats - perform M3UA/General Statistics request
 *****************************************************************************/
void doGenStats( char *args[] )
{
    M3UAGenSts  sts;
    STATUS      status;
    Bool        isReset = FALSE;

    memset( &sts,0, sizeof(M3UAGenSts) );
    if (args[0] != NULL)
    {
        if ( strcmp(args[0], "RESET") == 0 )
        {
            isReset = TRUE;
        }
    }
    status = M3uaGenStatistics( (U8)cp, &sts, isReset );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA General Statistics Request Failed: %s \n", getErrStr( status ) );

        return;
    }

    printf( "\n=========M3UA General Statistics since %02d-%02d-%02d %02d:%02d:%02d=========\n",
            sts.dt.month, sts.dt.day, sts.dt.year + 1900, sts.dt.hour, sts.dt.min, sts.dt.sec );

    printf( "\tMTP3 Statistics:\n" );
    printf( "\tCounter       %-12s    %-12s\n", "Tx", "Rx" );
    printf( "\t=====================================================\n" );
    printf( "\tData          %-12u    %-12u\n", sts.txMtp3Sts.data, sts.rxMtp3Sts.data );
    printf( "\tPause         %-12u    %-12u\n", sts.txMtp3Sts.pause, sts.rxMtp3Sts.pause );
    printf( "\tResume        %-12u    %-12u\n", sts.txMtp3Sts.resume, sts.rxMtp3Sts.resume );
    printf( "\tCong          %-12u    %-12u\n", sts.txMtp3Sts.cong, sts.rxMtp3Sts.cong );
    printf( "\tRestrict      %-12u    %-12u\n", sts.txMtp3Sts.drst, sts.rxMtp3Sts.drst );
    printf( "\tResetBegin    %-12u    %-12u\n", sts.txMtp3Sts.rstBeg, sts.rxMtp3Sts.rstBeg );
    printf( "\tResetEnd      %-12u    %-12u\n", sts.txMtp3Sts.rstEnd, sts.rxMtp3Sts.rstEnd );
    printf( "\tUPUs          %-12u    %-12u\n", sts.txMtp3Sts.upu, sts.rxMtp3Sts.upu );

    printf( "\n\tM3UA Statistics:\n" );
    printf( "\tCounter       %-12s    %-12s\n", "Tx", "Rx" );
    printf( "\t=====================================================\n" );
    printf( "\tData          %-12u    %-12u\n", sts.txM3uaSts.data, sts.rxM3uaSts.data );
    printf( "\tDUNA          %-12u    %-12u\n", sts.txM3uaSts.duna, sts.rxM3uaSts.duna );
    printf( "\tDAVA          %-12u    %-12u\n", sts.txM3uaSts.dava, sts.rxM3uaSts.dava );
    printf( "\tDAUD          %-12u    %-12u\n", sts.txM3uaSts.daud, sts.rxM3uaSts.daud );
    printf( "\tSCON          %-12u    %-12u\n", sts.txM3uaSts.scon, sts.rxM3uaSts.scon );
    printf( "\tDUPU          %-12u    %-12u\n", sts.txM3uaSts.dupu, sts.rxM3uaSts.dupu );
    printf( "\tDRST          %-12u    %-12u\n", sts.txM3uaSts.drst, sts.rxM3uaSts.drst );
    printf( "\tREGREQ        %-12u    %-12u\n", sts.txM3uaSts.regReq, sts.rxM3uaSts.regReq );
    printf( "\tDEREGREQ      %-12u    %-12u\n", sts.txM3uaSts.deRegReq, sts.rxM3uaSts.deRegReq );
    printf( "\tREGRSP        %-12u    %-12u\n", sts.txM3uaSts.regRsp, sts.rxM3uaSts.regRsp );
    printf( "\tDEREGRSP      %-12u    %-12u\n", sts.txM3uaSts.deRegRsp, sts.rxM3uaSts.deRegRsp );
    printf( "\tASPUP         %-12u    %-12u\n", sts.txM3uaSts.aspUp, sts.rxM3uaSts.aspUp );
    printf( "\tASPUPACK      %-12u    %-12u\n", sts.txM3uaSts.aspUpAck, sts.rxM3uaSts.aspUpAck );
    printf( "\tASPDN         %-12u    %-12u\n", sts.txM3uaSts.aspDn, sts.rxM3uaSts.aspDn );
    printf( "\tASPDNACK      %-12u    %-12u\n", sts.txM3uaSts.aspDnAck, sts.rxM3uaSts.aspDnAck );
    printf( "\tASPAC         %-12u    %-12u\n", sts.txM3uaSts.aspAc, sts.rxM3uaSts.aspAc );
    printf( "\tASPACACK      %-12u    %-12u\n", sts.txM3uaSts.aspAcAck, sts.rxM3uaSts.aspAcAck );
    printf( "\tASPIA         %-12u    %-12u\n", sts.txM3uaSts.aspIa, sts.rxM3uaSts.aspIa );
    printf( "\tASPIAACK      %-12u    %-12u\n", sts.txM3uaSts.aspIaAck, sts.rxM3uaSts.aspIaAck );
    printf( "\tHBEAT         %-12u    %-12u\n", sts.txM3uaSts.hBeat, sts.rxM3uaSts.hBeat );
    printf( "\tHBEATACK      %-12u    %-12u\n", sts.txM3uaSts.hBeatAck, sts.rxM3uaSts.hBeatAck );
    printf( "\tERROR         %-12u    %-12u\n", sts.txM3uaSts.err, sts.rxM3uaSts.err );
    printf( "\tNOTIFY        %-12u    %-12u\n", sts.txM3uaSts.notify, sts.rxM3uaSts.notify );

    printf( "\n\tDATA Statistics:\n" );
    printf( "\tTx/Rx           %-12s   %-12s\n", "PDUs", "Bytes" );
    printf( "\t=====================================================\n" );
    printf( "\tLowerIntf Tx    %-12u    %-12u\n", sts.liTxDataSts.nPdus, sts.liTxDataSts.pduBytes );
    printf( "\tLowerIntf Rx    %-12u    %-12u\n", sts.liRxDataSts.nPdus, sts.liRxDataSts.pduBytes );
    printf( "\tUpperIntf Tx    %-12u    %-12u\n", sts.uiTxDataSts.nPdus, sts.uiTxDataSts.pduBytes );
    printf( "\tUpperIntf Rx    %-12u    %-12u\n", sts.uiRxDataSts.nPdus, sts.uiRxDataSts.pduBytes );

    printf( "\n\tError Statistics:\n" );
    printf( "\tError           %-12s    %-12s\n", "Upward", "Downward" );
    printf( "\t=====================================================\n" );
    printf( "\tNoRoute         %-12u    %-12u\n", sts.upDataErrSts.dropNoRoute, sts.downDataErrSts.dropNoRoute );
    printf( "\tPC-Unavail      %-12u    %-12u\n", sts.upDataErrSts.dropPcUnavail, sts.downDataErrSts.dropPcUnavail );
    printf( "\tPC-Cong         %-12u    %-12u\n", sts.upDataErrSts.dropPcCong, sts.downDataErrSts.dropPcCong );
    printf( "\tPSP-Unavail     %-12u    %-12u\n", sts.upDataErrSts.dropNoPspAvail, sts.downDataErrSts.dropNoPspAvail );
    printf( "\tNSAP-Unavail    %-12u    %-12u\n", sts.upDataErrSts.dropNoNSapAvail, sts.downDataErrSts.dropNoNSapAvail );
    printf( "\tLoadSH-Fail     %-12u    %-12u\n", sts.upDataErrSts.dropLoadShFail, sts.downDataErrSts.dropLoadShFail );
    printf( "\tM3uaMsgFail     %-12u    %-12u\n", sts.upDataErrSts.dropMmhFail, sts.downDataErrSts.dropMmhFail );
    printf( "\tQue-Cong        %-12u    %-12u\n", sts.upDataErrSts.dataQCong, sts.downDataErrSts.dataQCong );
    printf( "\tAS-Pending      %-12u    %-12u\n", sts.upDataErrSts.dataQAsPend, sts.downDataErrSts.dataQAsPend );
}

/*****************************************************************************
 * doNSapStats - perform NSAP Statistics request
 *****************************************************************************/
void doNSapStats( char *args[] )
{
    M3UANSapSts sts;
    STATUS      status;
    S16         spId;
    Bool        isReset = FALSE;

    memset( &sts,0, sizeof(M3UANSapSts) );
    if (args[0] != NULL)
    {
        spId = (S16)(atoi(args[0]));
    }
    else
    {
        printf( "NSAP Statistics Request: Invalid spId \n" );

        return;
    }

    if ( NULL != args[1] )
    {
        if ( strcmp(args[1], "RESET") == 0 )
        {
            isReset = TRUE;
        }
    }
    status = M3uaNSapStatistics( (U8)cp, &sts, isReset, spId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA NSAP Statistics Request Failed: %s \n", getErrStr( status ) );

        return;
    }

    printf( "\n=========M3UA NSAP %d Statistics since %02d-%02d-%02d %02d:%02d:%02d=========\n",sts.spId,
            sts.dt.month, sts.dt.day, sts.dt.year + 1900, sts.dt.hour, sts.dt.min, sts.dt.sec );
    printf( "\tMTP3 Statistics:\n" );
    printf( "\tCounter       %-12s    %-12s\n", "Tx", "Rx" );
    printf( "\t=====================================================\n" );
    printf( "\tData          %-12u    %-12u\n", sts.txMtp3Sts.data, sts.rxMtp3Sts.data );
    printf( "\tPause         %-12u    %-12u\n", sts.txMtp3Sts.pause, sts.rxMtp3Sts.pause );
    printf( "\tResume        %-12u    %-12u\n", sts.txMtp3Sts.resume, sts.rxMtp3Sts.resume );
    printf( "\tCong          %-12u    %-12u\n", sts.txMtp3Sts.cong, sts.rxMtp3Sts.cong );
    printf( "\tRestrict      %-12u    %-12u\n", sts.txMtp3Sts.drst, sts.rxMtp3Sts.drst );
    printf( "\tResetBegin    %-12u    %-12u\n", sts.txMtp3Sts.rstBeg, sts.rxMtp3Sts.rstBeg );
    printf( "\tResetEnd      %-12u    %-12u\n", sts.txMtp3Sts.rstEnd, sts.rxMtp3Sts.rstEnd );
    printf( "\tUPUs          %-12u    %-12u\n", sts.txMtp3Sts.upu, sts.rxMtp3Sts.upu );

    printf( "\n\tDATA Statistics:\n" );
    printf( "\tTx/Rx       %-12s   %-12s\n", "PDUs", "Bytes" );
    printf( "\t=====================================================\n" );
    printf( "\tTx          %-12u   %-12u\n", sts.txDataSts.nPdus, sts.txDataSts.pduBytes );
    printf( "\tRx          %-12u   %-12u\n", sts.rxDataSts.nPdus, sts.rxDataSts.pduBytes );

    printf( "\n\tError Statistics:\n" );
    printf( "\tError        %-8s\n","Msg Dropped" );
    printf( "\t=====================================================\n" );
    printf( "\tNoRoute      %6u\n", sts.dataErrSts.dropNoRoute );
    printf( "\tPC-Unavail   %6u\n", sts.dataErrSts.dropPcUnavail );
    printf( "\tPC-Cong      %6u\n", sts.dataErrSts.dropPcCong );
    printf( "\tPSP-Unavail  %6u\n", sts.dataErrSts.dropNoPspAvail );
    printf( "\tNSAP-Unavail %6u\n", sts.dataErrSts.dropNoNSapAvail );
    printf( "\tLoadSH-Fail  %6u\n", sts.dataErrSts.dropLoadShFail );
    printf( "\tM3uaMsgFail  %6u\n", sts.dataErrSts.dropMmhFail );
    printf( "\tQue-Cong     %6u\n", sts.dataErrSts.dataQCong );
    printf( "\tAS-Pending   %6u\n", sts.dataErrSts.dataQAsPend );
}

/*****************************************************************************
 * doSctSapStats - perform SCT SAP Statistics request
 *****************************************************************************/
void doSctSapStats( char *args[] )
{
    M3UASctSapSts   sts;
    STATUS          status;
    S16             suId;
    Bool            isReset = FALSE;

    memset( &sts,0,sizeof(M3UASctSapSts) );
    if (args[0] != NULL)
    {
        suId = (S16)(atoi( args[0] ));
    }
    else
    {
        printf( "SCT SAP Statistics Request: Invalid spId \n" );

        return;
    }

    if ( NULL != args[1] )
    {
        if ( strcmp(args[1], "RESET") == 0 )
        {
            isReset = TRUE;
        }
    }
    status = M3uaSctSapStatistics( (U8)cp, &sts, isReset, suId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA SCTSAP Statistics Request Failed: %s \n", getErrStr( status ) );

        return;
    }

    printf( "\n=========M3UA SCTSAP %d Statistics since %02d-%02d-%02d %02d:%02d:%02d=========\n",sts.suId,
            sts.dt.month, sts.dt.day, sts.dt.year + 1900, sts.dt.hour, sts.dt.min, sts.dt.sec );
    printf( "\n\tDATA Statistics:\n" );
    printf( "\tTx/Rx       %-12s   %-12s\n", "PDUs", "Bytes" );
    printf( "\t=====================================================\n" );
    printf( "\tTx          %-12u   %-12u\n", sts.txDataSts.nPdus, sts.txDataSts.pduBytes );
    printf( "\tRx          %-12u   %-12u\n", sts.rxDataSts.nPdus, sts.rxDataSts.pduBytes );
}

/*****************************************************************************
 * doPspStats - perform PSP Statistics request
 *****************************************************************************/
void doPspStats( char *args[] )
{
    M3UAPspSts  sts;
    STATUS      status;
    U16         pspId;
    Bool        isReset = FALSE;
    U8          i;

    memset( &sts,0,sizeof(M3UAPspSts) );
    if (args[0] != NULL)
    {
        pspId = (U16)(atoi( args[0] ));
    }
    else
    {
        printf( "SCT SAP Statistics Request: Invalid spId \n" );

        return;
    }

    if ( NULL != args[1] )
    {
        if ( strcmp(args[1], "RESET") == 0 )
        {
            isReset = TRUE;
        }
    }
    status = M3uaPspStatistics( (U8)cp, &sts, isReset, pspId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA PSP Statistics Request Failed: %s \n", getErrStr( status ) );

        return;
    }

    printf( "\n=======M3UA PSP %d Statistics since %02d-%02d-%02d %02d:%02d:%02d=======\n",sts.pspId,
            sts.dt.month, sts.dt.day, sts.dt.year + 1900, sts.dt.hour, sts.dt.min, sts.dt.sec );
    for ( i = 0; i < M3UA_MAX_SEP ; i++ )
    {
        printf( "\n\tM3UA Statistics for Association %d:\n", i );
        printf( "\tCounter     %-12s   %-12s\n","Tx", "Rx" );
        printf( "\t=====================================================\n" );
        printf( "\tData        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.data, sts.assocSts[i].rxM3uaSts.data );
        printf( "\tDUNA        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.duna, sts.assocSts[i].rxM3uaSts.duna );
        printf( "\tDAVA        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.dava, sts.assocSts[i].rxM3uaSts.dava );
        printf( "\tDAUD        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.daud, sts.assocSts[i].rxM3uaSts.daud );
        printf( "\tSCON        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.scon, sts.assocSts[i].rxM3uaSts.scon );
        printf( "\tDUPU        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.dupu, sts.assocSts[i].rxM3uaSts.dupu );
        printf( "\tDRST        %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.drst, sts.assocSts[i].rxM3uaSts.drst );
        printf( "\tREGREQ      %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.regReq, sts.assocSts[i].rxM3uaSts.regReq );
        printf( "\tDEREGREQ    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.deRegReq, sts.assocSts[i].rxM3uaSts.deRegReq );
        printf( "\tREGRSP      %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.regRsp, sts.assocSts[i].rxM3uaSts.regRsp );
        printf( "\tDEREGRSP    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.deRegRsp, sts.assocSts[i].rxM3uaSts.deRegRsp );
        printf( "\tASPUP       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspUp, sts.assocSts[i].rxM3uaSts.aspUp );
        printf( "\tASPUPACK    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspUpAck, sts.assocSts[i].rxM3uaSts.aspUpAck );
        printf( "\tASPDN       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspDn, sts.assocSts[i].rxM3uaSts.aspDn );
        printf( "\tASPDNACK    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspDnAck, sts.assocSts[i].rxM3uaSts.aspDnAck );
        printf( "\tASPAC       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspAc, sts.assocSts[i].rxM3uaSts.aspAc );
        printf( "\tASPACACK    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspAcAck, sts.assocSts[i].rxM3uaSts.aspAcAck );
        printf( "\tASPIA       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspIa, sts.assocSts[i].rxM3uaSts.aspIa );
        printf( "\tASPIAACK    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.aspIaAck, sts.assocSts[i].rxM3uaSts.aspIaAck );
        printf( "\tHBEAT       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.hBeat, sts.assocSts[i].rxM3uaSts.hBeat );
        printf( "\tHBEATACK    %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.hBeatAck, sts.assocSts[i].rxM3uaSts.hBeatAck );
        printf( "\tERROR       %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.err, sts.assocSts[i].rxM3uaSts.err );
        printf( "\tNOTIFY      %-12u   %-12u\n", sts.assocSts[i].txM3uaSts.notify, sts.assocSts[i].rxM3uaSts.notify );

        printf( "\n\tDATA Statistics for Association %d\n", i );
        printf( "\tTx/Rx       %-12s   %-12s\n", "PDUs", "Bytes" );
        printf( "\t=====================================================\n" );
        printf( "\tTx          %-12u   %-12u\n", sts.assocSts[i].txDataSts.nPdus, sts.assocSts[i].txDataSts.pduBytes );
        printf( "\tRx          %-12u   %-12u\n", sts.assocSts[i].rxDataSts.nPdus, sts.assocSts[i].rxDataSts.pduBytes );

        printf( "\n\tError Statistics for Association %d\n", i );
        printf( "\tError        %-8s\n","Msg Dropped" );
        printf( "\t=====================================================\n" );
        printf( "\tNoRoute      %6u\n", sts.assocSts[i].dataErrSts.dropNoRoute );
        printf( "\tPC-Unavail   %6u\n", sts.assocSts[i].dataErrSts.dropPcUnavail );
        printf( "\tPC-Cong      %6u\n", sts.assocSts[i].dataErrSts.dropPcCong );
        printf( "\tPSP-Unavail  %6u\n", sts.assocSts[i].dataErrSts.dropNoPspAvail );
        printf( "\tNSAP-Unavail %6u\n", sts.assocSts[i].dataErrSts.dropNoNSapAvail );
        printf( "\tLoadSH-Fail  %6u\n", sts.assocSts[i].dataErrSts.dropLoadShFail );
        printf( "\tM3uaMsgFail  %6u\n", sts.assocSts[i].dataErrSts.dropMmhFail );
        printf( "\tQue-Cong     %6u\n", sts.assocSts[i].dataErrSts.dataQCong );
        printf( "\tAS-Pending   %6u\n", sts.assocSts[i].dataErrSts.dataQAsPend );
    }
}

/*****************************************************************************
 * doGenStatus - perform M3UA/General status request
 *****************************************************************************/
void doGenStatus( char *args[] )
{
    M3UAGenSta          sta;
    M3UAGenCfg          cfg;
    M3UAGenTimerCfg    *tmr;
    STATUS              status;

    UNREFERENCED_PARAMETER(args);

    /* Init Memory */
    memset( &sta,0,sizeof(M3UAGenSta) );
    memset( &cfg,0,sizeof(M3UAGenCfg) );
    status = M3uaGenStatus( (U8)cp, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA Status Request Failed: %s\n",getErrStr( status ) );

        return;
    }

    printf( "\nM3UA Memory Size        = %u\n", sta.memSize );
    printf( "M3UA Memory Allocated   = %u\n", sta.memAlloc );
    printf( "M3UA HA State           = %s\n", highAvailState[sta.haState] );
    if ( (sta.haState == M3UA_HAST_PRIMARY) || (sta.haState == M3UA_HAST_BACKUP) )
    {
        printf( "M3UA Connected          = %s\n\n", sta.isolated == FALSE ? "TRUE" : "FALSE" );
    }
    else
    {
        printf( "\n" );
    }
    status = M3uaGetGenCfg( (U8)cp, &cfg );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA Get General Configuration Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n====================Current M3UA Configuration====================\n" );
    printf( "NodeType      = %-8s", cfg.nodeType == 2 ? "ASP" : "SGP" );
    printf( "dpcLrnFlag    = %-8s", cfg.dpcLrnFlag == TRUE ? "TRUE" : "FALSE" );
    printf( "maxNmbNSap    = %-8u\n", cfg.maxNmbNSap );

    printf( "maxNmbSctSap  = %-8u", cfg.maxNmbSctSap );
    printf( "maxNmbNwk     = %-8u", cfg.maxNmbNwk );
    printf( "maxNmbRtEnt   = %-8u\n", cfg.maxNmbRtEnt );

    printf( "maxNmbDpcEnt  = %-8u", cfg.maxNmbDpcEnt );
    printf( "maxNmbPs      = %-8u", cfg.maxNmbPs );
#if (defined(ITASP) && defined(OG_RTE_ON_LPS_STA))
    printf( "maxNmbLps     = %-8u\n", cfg.maxNmbLps );
#endif
    printf( "maxNmbPsp     = %-8u\n", cfg.maxNmbPsp );

    printf( "maxNmbMsg     = %-8u", cfg.maxNmbMsg );
    printf( "maxNmbRndRbnLs= %-8u", cfg.maxNmbRndRbnLs );
    printf( "maxNmbSlsLs   = %-8u\n", cfg.maxNmbSlsLs );

    printf( "maxNmbSls     = %-8u", cfg.maxNmbSls );
    printf( "drkmSupp      = %-8s", cfg.drkmSupp == TRUE ? "TRUE" : "FALSE" );
    printf( "drstSupp      = %-8s\n", cfg.drstSupp == TRUE ? "TRUE" : "FALSE" );

    printf( "qSize         = %-8u", cfg.qSize );
    printf( "congLevel1    = %-8u", cfg.congLevel1 );
    printf( "congLevel2    = %-8u\n", cfg.congLevel2 );

    printf( "congLevel3    = %-8u", cfg.congLevel3 );
    printf( "timeRes       = %-8u", cfg.timeRes );
    tmr = &cfg.tmr;
    printf( "\n========================Current M3UA Timers=======================\n" );
    printf( "tmrRestart    = %-8u", tmr->tmrRestart.val );
    printf( "tmrMtp3Sta    = %-8u", tmr->tmrMtp3Sta.val );
    printf( "tmrAsPend     = %-8u\n", tmr->tmrAsPend.val );

    printf( "tmrHeartbeat  = %-8u", tmr->tmrHeartbeat.val );
    printf( "tmrAspUp1     = %-8u", tmr->tmrAspUp1.val );
    printf( "tmrAspUp2     = %-8u\n", tmr->tmrAspUp2.val );

    printf( "nmbAspUp1     = %-8u", tmr->nmbAspUp1 );
    printf( "tmrAspDn      = %-8u", tmr->tmrAspDn.val );
    printf( "tmrAspM       = %-8u\n", tmr->tmrAspM.val );

    printf( "tmrDaud       = %-8u", tmr->tmrDaud.val );
    printf( "tmrDrkm       = %-8u", tmr->tmrDrkm.val );
    printf( "maxNmbRkTry   = %-8u\n", tmr->maxNmbRkTry );

    printf( "tmrDunaSettle = %-8u", tmr->tmrDunaSettle.val );
    printf( "tmrSeqCntrl   = %-8u\n", tmr->tmrSeqCntrl.val );
}

/*****************************************************************************
 * doNSapStatus - perform Address Translation status request
 *****************************************************************************/
void doNSapStatus( char *args[] )
{
    M3UANSapCfg     cfg;
    M3UANSapSta     sta;
    STATUS          status;
    S16             sapId = 0;

    memset( &cfg,0,sizeof(M3UANSapCfg) );
    memset( &sta,0,sizeof(M3UANSapSta) );
    if (args[0] != NULL)
    {
        sapId = (S16)(atoi( args[0] ));
    }
    else
    {
        printf( "STATUS NSAP Error: Invalid sapid \n" );
    }
    /* Get Status */
    status = M3uaNSapStatus( (U8)cp, sapId, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA NSAP Status Request Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n====================NSAP %d Status==========================\n", sapId );
    printf( "Local SAP ID        = %u\n", sta.lclSapId );
    printf( "Remote SAP ID       = %u\n", sta.remSapId );
    printf( "High Level Status   = %s\n", sapStatus[sta.hlSt] );
    /* Get Configuration */
    status = M3uaGetNSapCfg( (U8)cp, &cfg, sapId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA NSAP Config Request Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n====================NSAP %d Configuration====================\n", sapId );
    printf( "SAP ID      = %-8u", cfg.sapId );
    printf( "Network Id  = %-8u", cfg.nwkId );
    printf( "Su Type     = %-8s\n", (cfg.suType < 14) ? suTypeStr[cfg.suType] : "N/A" );
    printf( "Selector    = %-8u", cfg.selector );
    printf( "Mem Pool    = %-8u", cfg.mem.pool );
    printf( "Mem Region  = %-8u\n", cfg.mem.region );
    printf( "Priority    = %-8u", cfg.prior );
    printf( "Route       = %-8u\n", cfg.route );
}

/*****************************************************************************
 * doAddrTransStatus - perform Address Translation status request
 *****************************************************************************/
void doAddrTransStatus( char *args[] )
{
    M3UAAtSta   sta;
    STATUS      status;

    UNREFERENCED_PARAMETER(args);

    memset( &sta, 0, sizeof(M3UAAtSta) );

    status = M3uaAddrTransStatus( (U8)cp, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA Address Translation Request Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n====================Address Translation Status====================\n" );
    printf( "Number of DPCs      = %d\n", sta.nmbDpc );
    printf( "Number of Routes    = %d\n", sta.nmbRout );
}

/*****************************************************************************
 * doDpcStatus - perform DPC status request
 *****************************************************************************/
void doDpcStatus( char *args[] )
{
    M3UADpcSta  sta;
    STATUS      status;
    U32         dpc;
    char        strDpc[12];
    U8          nwId;

    if (args[0] != NULL)
    {
        nwId = (U8)(atoi( args[0] ));
    }
    else
    {
        printf( "DPC Status Error: Invalid Network Id (nwkId)\n" );

        return;
    }

    if ( NULL != args[1] )
    {
        strcpy( strDpc, args[1] );
        CfgScanPc( strDpc, &dpc, SS7_PCDISP_DFLT );
    }
    else
    {
        printf( "DPC Status Error: Invalid DPC\n" );

        return;
    }

    status = M3uaDpcStatus( (U8)cp, dpc, nwId, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA DPC Status Request Failed: %s\n", getErrStr( status) );

        return;
    }

    printf( "\n=======================DPC Status =======================\n" );
    printf( "Network Id      = %d\n", sta.nwkId );
    printf( "       DPC      = %s\n", CfgFmtPc( sta.dpc,DPC24,SS7_PCDISP_DFLT ) );
    printf( "DPC Status      = %s\n", sta.dpcSt < 5 ? dpcSta[sta.dpcSt] : "N/A" );
    printf( "Cong level      = %d\n", sta.congLevel );
}

/*****************************************************************************
 * doRouteStatus - perform Routing Config status request
 *****************************************************************************/
void doRouteStatus( char *args[] )
{
#ifdef TODO_CLEAN
    M3UARteCfg     sta;
    STATUS         status;

    memset( &sta, 0, sizeof(M3UARteCfg) );
    status = M3uaRteStatus( (U8)cp, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "Route Configuration Status Request Failed:  %s\n", getErrStr( status ) );

        return;
    }
#endif

    UNREFERENCED_PARAMETER(args);
}

/*****************************************************************************
 * doNwkStatus - perform network Config status request
 *****************************************************************************/
void doNwkStatus( char *args[] )
{
    M3UANwkCfg     cfg;
    STATUS         status;
    U8             nwkId;
    char           suStr[10];
    U8             i;

    memset( &cfg, 0, sizeof(M3UANwkCfg) );

    if (args[0] != NULL)
    {
        nwkId = (U8)(atoi( args[0] ));
    }
    else
    {
        printf( "Network Config Status Error: Invalid Network Id\n" );

        return;
    }

    status = M3uaGetNwkCfg( (U8)cp, &cfg, nwkId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "Network Config Request failed: %s \n", getErrStr( status ) );

        return;
    }

    printf( "\n  ====================Network %d Configuration====================\n", cfg.nwkId );
    printf( "  ssf        = %-8s", cfg.ssf < 4 ? nwkSsf[cfg.ssf] : "UNKWN" );
    printf( "  dpcLen     = %-8s", cfg.dpcLen < 4 ? nwkDpcLen[cfg.dpcLen] : "UNKWN" );

    printf( "  slsLen     = %-8u\n", cfg.slsLen );
    printf( "  suSwtch    = %-8s", cfg.suSwtch < 12 ? nwkSwtch[cfg.suSwtch] : "UNKWN" );
    switch(cfg.su2Swtch)
    {
        case M3UA_SW2_UNUSED:
            strcpy( suStr,"UNUSED" );
            break;

        case M3UA_SW2_ITU:
            strcpy( suStr,"ITU" );
            break;

        case M3UA_SW2_ETS:
            strcpy( suStr, "ETSI" );
            break;

        case M3UA_SW2_ANS:
            strcpy( suStr, "ANSI" );
            break;

        case M3UA_SW2_TTC:
            strcpy( suStr, "TTC" );
            break;

        default:
            strcpy( suStr, "UNKWN" );
    }
    printf( "  suSwtch    = %-8s\n", suStr );
    for (i = 0; i < M3UA_MAX_PSP; i++)
    {
        if (cfg.nwkApp[i] == 0 )
        {
            break;
        }
        printf( "%-4s[%-2d] = %-4u","  NetAppear", i+1, cfg.nwkApp[i] );
        if (((i+1)%3) == 0)
        {
            printf( "\n" );
        }
    }
    printf( "\n" );
}

/*****************************************************************************
 * doPsStatus - perform Peer Server status request
 *****************************************************************************/
void doPsStatus( char *args[] )
{
    M3UAPsSta   sta;
    M3UAPsCfg   cfg;
    STATUS      status;
    U32         psId;
    U8          i, j;
    U16         pspId;

    memset( &sta,0,sizeof(M3UAPsSta) );
    memset( &cfg,0,sizeof(M3UAPsCfg) );
    if (args[0] != NULL)
    {
        psId = (U32) (atol( args[0] ));
    }
    else
    {
        printf( "M3UA PS Status: Invalid PSID\n" );

        return;
    }

    status = M3uaPsStatus( (U8)cp, psId, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA PS Status Request Failed: %s %d \n", getErrStr(status), psId );

        return;
    }

    printf( "\n===================Peer Server(PS) %d Status===================\n", psId );
    printf( "PS ID      = %-8lu", sta.psId );
    printf( "AS STATE   = %-8s\n", sta.asSt < 5 ? psSta[sta.asSt] : "N/A" );
    printf( "\n=======================PS STATUS PER SEP=======================\n" );
    for (i = 0; i < M3UA_MAX_SEP ; i++)
    {
        printf( "Active PSPs    = %-8d", sta.psStaEndp[i].nmbAct );
        printf( "Registerd PSPs = %-8d\n", sta.psStaEndp[i].nmbPspReg );
        for (j = 0; j < sta.psStaEndp[i].nmbAct; j++)
        {
            pspId =  sta.psStaEndp[i].actPsp[j];
            printf( "PspId = %-6u", pspId );
            printf( "ASP State = %-6s", sta.psStaEndp[i].aspSt[pspId] < 4 ? aspSta[sta.psStaEndp[i].aspSt[pspId]] : "UNKWN" );
            printf( "RtCtx = 0x%-6X", sta.psStaEndp[i].rteCtx[pspId].rCtx );
            printf( "Valid = %-6s", sta.psStaEndp[i].rteCtx[pspId].rcValid == TRUE ? "YES" : "NO" );
            printf( "Mode = %-6s\n", sta.psStaEndp[i].rteCtx[pspId].mode == 0x01 ? "DYNAMIC" : "STATIC" );
        }
    }
    status = M3uaGetPsCfg( (U8)cp, &cfg, psId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA PS Config. Request Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n===================Current PS %d Configuration===================\n", psId );
    printf( "PS Id      = %-8d", cfg.psId );
    printf( "Net Id     = %-8d", cfg.nwkId );
    printf( "Mode       = %-8s\n", cfg.mode == 2 ? "LOADSH" : "ACT/STBY" );
    printf( "RoutCtx    = %-8d", cfg.routCtx );
    printf( "LoadShMode = %-8s", cfg.loadShareMode == 1 ? "RNDRBN" : "SLS/CIC" );
    printf( "ReqAvail   = %-8s\n", cfg.reqAvail == TRUE ? "TRUE" : "FALSE" );
    printf( "ActPspReqd = %-8d", cfg.nmbActPspReqd );
    printf( "NmbPsps    = %-8d", cfg.nmbPsp );
    printf( "PSLocal    = %-8s\n", cfg.lclFlag == TRUE ? "YES" : "NO" );
    for (i = 0; i < cfg.nmbPsp; i++)
    {
        printf( "Psp Id     = %-8d\n", cfg.psp[i] );
    }
}

/*****************************************************************************
 * doPspStatus - perform Peer Server Process status request
 *****************************************************************************/
void doPspStatus( char * args[] )
{
    M3UAPspSta      sta;
    M3UAPspCfg      cfg;
    STATUS          status;
    U16             pspId;
    U8              i,j;
    char            ipStr[64];

    memset( &sta, 0, sizeof(M3UAPspSta) );
    memset( &cfg, 0, sizeof(M3UAPspCfg) );
    if (args[0] != NULL)
    {
        pspId = (U16) (atol( args[0] ));
    }
    else
    {
        printf( "M3UA PSP Status: Invalid PSPID\n" );

        return;
    }

    status = M3uaPspStatus( (U8)cp, pspId, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA PS Status Request Failed: %s %d\n", getErrStr(status), pspId );

        return;
    }

    printf( "\n=====================PSP %d Association Status===================\n", sta.pspId );
    for (i = 0; i < M3UA_MAX_SEP; i++)
    {
        printf( "AssocId = %-8d", sta.assocSta[i].spAssocId );
        printf( "State = %-8s",sta.assocSta[i].hlSt < 5 ? assocSta[sta.assocSta[i].hlSt] : "N/A" );
        printf( "ASP State = %-8s",sta.assocSta[i].aspSt < 4 ? aspSta[sta.assocSta[i].aspSt] : "N/A" );
        printf( "Inhibit = %-8s\n", sta.assocSta[i].inhibited == TRUE ? "YES" : "NO" );
        printf( "\n=================Active PSs (%d)=================\n", sta.assocSta[i].nmbAct );
        if ( sta.assocSta[i].nmbAct > 0 )
        {
            for (j = 0; j < sta.assocSta[i].nmbAct; j++)
            {
                printf( "\tPsId   = %d\n", sta.assocSta[i].actPs[j] );
            }
        }
        else
        {
            printf( "\tNone\n" );
        }
        printf( "\n=================Registered PSs (%d)==============\n", sta.assocSta[i].nmbRegPs );
        if ( sta.assocSta[i].nmbRegPs > 0 )
        {
            for (j = 0; j < sta.assocSta[i].nmbRegPs; j++)
            {
                printf( "\tPsId   = %d\n", sta.assocSta[i].regPs[j] );
            }
        }
        else
        {
            printf( "\tNone\n" );
        }
    }
    status = M3uaGetPspCfg( (U8)cp,pspId, &cfg );
    printf( "\n=====================Current PSP %u Configuration=================\n", cfg.pspId );
    printf( "pspType       = %-8s", cfg.pspType < 4 ? pspType[cfg.pspType] : "N/A" );
    printf( "ipspMode      = %-8s", cfg.ipspMode == 1 ? "SNGL END" : "DBL END" );
    printf( "dynRegAllow   = %-8s\n", cfg.dynRegRkallwd == TRUE ? "YES" : "NO" );
    printf( "loadShareMode = %-8s", cfg.dfltLshareMode == 1 ? "RNDRBN" : "SLS" );
    printf( "nwkAppIncl    = %-8s", cfg.nwkAppIncl == TRUE ? "YES" : "NO" );
#ifdef LITV10
    printf( "rxTxAspId     = %-8u", cfg.rxTxAspId );
#else
    printf( "rxTxAspId     = %-8s\n", cfg.rxTxAspId == TRUE ? "YES" : "NO" );
#endif
    printf( "selfAspId     = %-8u", cfg.selfAspId );
    printf( "nwkId         = %-8u", cfg.nwkId );
    u32ToIp( cfg.assocCfg.priDstAddr.u.ipv4NetAddr, &ipStr[0] );
    printf( "PriDestAddr   = %s\n", ipStr );
    printf( "DestPort      = %-8u", cfg.assocCfg.dstPort );
    printf( "locOutStrms   = %-8u", cfg.assocCfg.locOutStrms );
    printf( "TOS           = %-8u\n", cfg.assocCfg.tos );
    for ( i = 0; i < cfg.assocCfg.dstAddrLst.nmb; i++)
    {
        u32ToIp( cfg.assocCfg.dstAddrLst.nAddr[i].u.ipv4NetAddr, &ipStr[0] );
        printf( "dstAddrLst[%d]= %s\n", i+1, ipStr );
    }
}

/*****************************************************************************
 * doSctSapStatus - perform SCT SAP status request
 *****************************************************************************/
void doSctSapStatus( char *args[] )
{
    M3UASctSapSta   sta;
    M3UASctSapCfg   cfg;
    STATUS          status;
    S16             suId;
    U8              i;
    char            ipStr[64];

    memset( &sta,0,sizeof(M3UASctSapSta) );
    memset( &cfg,0,sizeof(M3UASctSapCfg) );
    if ( NULL != args[0])
    {
        suId = (S16)(atoi( args[0] ));
    }
    else
    {
        printf( "M3UA SCT SAP Status: Invalid user Sap Id\n" );

        return;
    }

    status = M3uaSctSapStatus( (U8)cp, suId, &sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA SCT SAP Status Request Failed: %s\n", getErrStr( status ));

        return;
    }

    printf( "\n==========================User SAP %d Status=======================\n", sta.suId );
    printf( "State        = %-8s", sapStatus[sta.hlSt] );
    printf( "EndPoint SP  = %-8u", sta.spEndpId );
    printf( "Active Assoc = %-8u\n", sta.nmbActAssoc );
    M3uaGetSctSapCfg( (U8)cp, &cfg, suId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA SCT SAP Config. Request Failed: %s\n", getErrStr( status ) );

        return;
    }

    printf( "\n==================Current Configuration for SAP %d=================\n", cfg.suId );
    printf( "tmrPrim    = %-8u", cfg.tmrPrim.val );
    printf( "tmrSta     = %-8u", cfg.tmrSta.val );
    printf( "srcPort    = %-8u\n", cfg.srcPort );

    printf( "Selector   = %-8u", cfg.selector );
    printf( "Mem.Pool   = %-8u", cfg.mem.pool );
    printf( "Mem.Region = %-8u\n", cfg.mem.region );

    printf( "ProcId     = %-8u", cfg.procId );
    printf( "Ent        = %-8u", cfg.ent );
    printf( "Inst       = %-8u\n", cfg.inst );

    printf( "Prior      = %-8u", cfg.prior );
    printf( "Route      = %-8u", cfg.route );
    printf( "SP Id      = %-8u\n", cfg.spId );
    for (i = 0; i < cfg.srcAddrLst.nmb; i++)
    {
        u32ToIp( cfg.srcAddrLst.nAddr[i].u.ipv4NetAddr, &ipStr[0] );
        printf( "SrcAddrLst[%d] = %s\n", i+1, ipStr );
    }
}

/*****************************************************************************
 * doRouteKeyStatus - perform Routing Key status request
 *****************************************************************************/
void doRouteKeyStatus( char *args[] )
{
    M3UARkSta   sta;
    STATUS      status;

    UNREFERENCED_PARAMETER(args);

    memset( &sta, 0, sizeof(M3UARkSta) );
    status = M3uaRkStatus( (U8)cp,&sta );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA Route Key Status Request Failed: %s\n", getErrStr( status) );

        return;
    }

    printf( "\n====================== Registered Routing Keys ====================\n" );
    printf( "Number RK registered = %d\n", sta.nmbRkReg );
}

/*****************************************************************************
 * doPspRouteStatus - perform PSP Routing status request
 *****************************************************************************/
void doPspRouteStatus( char *args[] )
{
    M3UAPspRkIdSta  sta;
    STATUS          status;
    U16             pspId;
    U8              i;

    memset( &sta, 0, sizeof(M3UAPspRkIdSta) );
    if (args[0] != NULL)
    {
        pspId = (U16)(atoi( args[0] ));
    }
    else
    {
        printf( "M3UA PSP Route Status: Invalid PSP Id\n" );

        return;
    }

    status = M3uaPspRkIdStatus( (U8)cp, &sta, pspId );
    if ( M3UA_SUCCESS != status )
    {
        printf( "M3UA PSP RK Status Failed: %s \n", getErrStr(status) );

        return;
    }

    printf( "\n================== Routing Key Status for PSP %d ================\n", sta.pspId );
    printf( "SCT SuID        = %-8u", sta.sctSuId );
    printf( "Pending RK Req. = %-8u\n", sta.nmbRkReqPend );
    printf( "Local RK IDs pending: \n" );
    for (i = 0; i < sta.nmbRkReqPend; i++)
    {
        printf( "%d. %-8u",i+1, sta.lclRkId[i] );
    }
    if (sta.nmbRkReqPend < 1)
    {
        printf( "None" );
    }
    printf( "\n" );
}

/*****************************************************************************
 * statusReq - Handles all Status requests (calling relevant sub-function)
 *****************************************************************************/
void statusReq( char *args[] )
{
    short   i;
    Bool    found = FALSE;

    /* first search Ent table for match */
    found = FALSE;
    if ( NULL == args[0] )
    {
        printf( "Invalid Status Request\n" );
        printf( "Usage: STATUS SYS | M3UA | NSAP | ADDR| DPC | PS | PSP | RK | PSPRK\n" );

        return;
    }

    for (i = 0; staList[i].val != UNKNOWN; i++)
    {
        if (strcmp( args[0], staList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown STATUS Command (%s)\n", args[0] );

        return;
    }

    switch (staList[i].val)
    {
        case M3UA_STA:
            doGenStatus( (char **)&args[1] );
            break;

        case NSAP_STA:
            doNSapStatus( (char **)&args[1] );
            break;

        case ADDR_TRANS_STA:
            doAddrTransStatus( (char **)&args[1] );
            break;

        case DPC_STA:
            doDpcStatus( (char **)&args[1] );
            break;

        case NWK_STA:
            doNwkStatus( (char **)&args[1] );
            break;

        case RTE_STA:
            doRouteStatus( (char **)&args[1] );
            break;

        case PS_STA:
            doPsStatus( (char **)&args[1] );
            break;

        case PSP_STA:
            doPspStatus( (char **)&args[1] );
            break;

        case SCTSAP_STA:
            doSctSapStatus( (char **)&args[1] );
            break;

        case RK_STA:
            doRouteKeyStatus( (char **)&args[1] );
            break;

        case PSPRK_STA:
            doPspRouteStatus( (char **)&args[1] );
            break;

        default:
            printf( "Error: Unknown STATS Command!\n" );
    }
}

/*****************************************************************************
 * sctsapCtrl
 *****************************************************************************/
void sctsapCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( NULL == parms[0] )
    {
        printf( "Error: Unknown SCTSAP Command\n" );

        return;
    }

    /* first search ctrl table for match */
    for (i = 0; sctsapCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], sctsapCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown SCTSAP Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, 0, (U8)sctsapCtrlList[i].val );     /* Always sap 0 */
    if (status != M3UA_SUCCESS)
	{
        printf( "Board %d SCT sap %d Control Request %d failed: status = %x\n",
                cp, 0, sctsapCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * assocCtrl
 *****************************************************************************/
void assocCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( (parms[0] == NULL) || (parms[1] == NULL) )
    {
        printf( "Error: Unknown ASSOC Command\n" );

        return;
    }

    /* first search ctrl table for match */
    for (i = 0; assocCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], assocCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown ASSOC Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, (S16)atoi(parms[1]), (U8)assocCtrlList[i].val );     /* parms[1]=pspId */
    if (status != M3UA_SUCCESS)
	{
        printf( "Board %d Assoc (pspId=%d) Control Request %d failed: status = 0x%x\n",
                cp, atoi(parms[1]), assocCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * aspCtrl
 *****************************************************************************/
void aspCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( (parms[0] == NULL) || (parms[1] == NULL) )
    {
        printf( "Error: Unknown ASP Command\n" );

        return;
    }

    /* first search ctrl table for match */
    for (i = 0; aspCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], aspCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown asp Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, (S16)atoi(parms[1]), (U8)aspCtrlList[i].val );     /* parms[1]=pspId */
    if ( status != M3UA_SUCCESS )
	{
        printf( "Board %d asp (aspId=%d) Control Request %d failed: status = %x\n",
                cp, atoi(parms[1]), aspCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * rkCtrl
 *****************************************************************************/
void rkCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( parms[0] == NULL )
    {
        printf( "Error: Unknown RK Command\n" );

        return;
    }

    /* first search ctrl table for match */
    for (i = 0; rkCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], rkCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown rk Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, 0, (U8)rkCtrlList[i].val );     /* Always sap 0 */
    if (status != M3UA_SUCCESS)
	{
        printf( "Board %d rk (pspId=%d) Control Request %d failed: status = %x\n",
                cp, 0, rkCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * tosCtrl - Set type-of-service for a specific Peer Server Process
 *
 * SYNTAX: TOS <pspId> [ <tosval> ] - set type-of-service to be used by PSP
 *****************************************************************************/
void tosCtrl( char *parms[] )
{
    U8          pspId;
    U8          tosVal;
    STATUS      status;

    if ( parms[0] == NULL )
    {
        printf( "Error: Expecting PSP ID after TOS command\n" );

        return;
    }

    pspId = (U8)(atoi( parms[0] ));

    if ( parms[1] == NULL )
    {
        printf( "Error: Expecting <tosval> after TOS <pspId> command\n" );
    }
    else
    {   /* set the new TOS for the PSP to use */
        tosVal = (U8)(atoi( parms[1] ));

        /* Note: 2nd param, entity, combines pspId and tosVal */
        status = M3uaMgmtCtrl( (U8)cp, (U16)((pspId << 8) | tosVal), (U8)M3UA_CTRL_SET_TOS );
        if (status != M3UA_SUCCESS)
        {
            printf( "Board %d TOS Control Request %d failed: status = 0x%2x\n",
                    cp, M3UA_CTRL_SET_TOS, status );
        }
    }
}

/*****************************************************************************
 * debugCtrl
 *****************************************************************************/
void debugCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( parms[0] == NULL )
    {
        printf( "Error: Unknown RK Command\n" );

        return;
    }

    /* first search table for match */
    for (i = 0; debugCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], debugCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown DEBUG Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, 0, (U8)debugCtrlList[i].val );     /* Entity unused currently */
    if (status != M3UA_SUCCESS)
	{
        printf( "Board %d DEBUG %d Control Request %d failed: status = 0x%2x\n",
                cp, 0, debugCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * traceCtrl - enable or disable TX board-based tracing
 *****************************************************************************/
void traceCtrl( char *parms[] )
{
    short   i;
    Bool    found = FALSE;
    STATUS  status;

    if ( parms[0] == NULL )
    {
        printf( "Error: Unknown Trace Command\n" );

        return;
    }

    /* first search table for match */
    for (i = 0; traceCtrlList[i].val != UNKNOWN; i++)
    {
        if (strcmp( parms[0], traceCtrlList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }
    if ( !found )
    {
        printf( "Error: Unknown TRACE Command (%s)\n", parms[0] );

        return;
    }

    status = M3uaMgmtCtrl( (U8)cp, 0, (U8)traceCtrlList[i].val );     /* Entity unused currently */
    if (status != M3UA_SUCCESS)
	{
        printf( "Board %d TRACE %d Control Request %d failed: status = 0x%2x\n",
                cp, 0, debugCtrlList[i].val, status );
    }
}

/*****************************************************************************
 * processCmd - process operator commands
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 *****************************************************************************/
void processCmd( char *tokens[], short cnt )
{
    S16     i;
    Bool    found;

    UNREFERENCED_PARAMETER(cnt);

    /* first search command table for command match */
    found = FALSE;
    if ( tokens[0] == NULL )
    {
        printf( "Error: Unknown Command\n" );

        return;
    }

    for (i = 0; cmdList[i].val != UNKNOWN; i++)
    {
        if (strcmp( tokens[0], cmdList[i].name ) == 0)
        {
            found = TRUE;
            break;      /* command matched */
        }
    }

    if ( !found )
    {
        printf( "Error: Unknown Command (%s)\n", tokens[0] );
    }
    else
    {
        switch (cmdList[i].val)
        {
            case CMD_HELP :
                HelpCommand( (char **)&tokens[1] );
                break;

            case CMD_STATS :
                statsReq( (char **)&tokens[1] );
                break;

            case CMD_STATUS :
                statusReq( (char **)&tokens[1] );
                break;

            case CMD_SCTSAP:
                sctsapCtrl( (char **)&tokens[1] );
                break;

            case CMD_ASSOC:
                assocCtrl( (char **)&tokens[1] );
                break;

            case CMD_ASP:
                aspCtrl( (char **)&tokens[1] );
                break;

            case CMD_RK:
                rkCtrl( (char **)&tokens[1] );
                break;

            case CMD_TOS:
                tosCtrl( (char **)&tokens[1] );
                break;

            case CMD_DEBUG:
                debugCtrl( (char **)&tokens[1] );
                break;

            case CMD_BOARD :
                setCP( (char **)&tokens[1] );
                break;

            case CMD_QUIT :
                quit();
                break;

            case CMD_TRACE:
                traceCtrl( (char **)&tokens[1] );
                break;

            default:
                printf( "Unknown Command %s\n", tokens[0] );
        }
    }
}

/*****************************************************************************
 * tokenize - convert a string into an array of tokens
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 *****************************************************************************/
short tokenize( char *buf, char *tokens[] )
{
    char   *p = buf;
    char   *endp;
	short   cnt;

	/* initialize token pointers to all null */
    for (cnt = 0; cnt <= MAX_PARMS; cnt++)
    {
		tokens[cnt] = 0;
    }

	/* begin scanning for tokens */
	cnt = 0;
    while ( (*p != 0) && (cnt <= MAX_PARMS) )
	{
		/* skip leading space or tabs */
        while ( (*p == ' ') || (*p == '\t') )
        {
			p++;
        }

		/* beginning of token (or end of line) */
        if ( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
            while ( (*endp != ' ') && (*endp != '\t') && (*endp != 0) )
			{
                if ( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
            if ( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
        }
    }

	return( cnt );
}

/*****************************************************************************
 * usage
 *****************************************************************************/
void usage( char *pName )
{
    printf( "\n usage: %s [ -b <board>] \n", pName );
    printf( "\n\t<board>\tboard number to attach to [default 1]\n" );

    exit( 1 );
}

/*****************************************************************************
 * main
 *****************************************************************************/
int main( int argc, char *argv[] )
{
	char	buffer[132];
    char   *tokens[MAX_PARMS +1];
	short	cnt;
	short	i;
    char   *p;
    STATUS  status;

    printf( "\n\n%s: sample M3UA management application version %s %s\n\n",
            argv[0], M3UAMGR_VERSION, __DATE__ );

	/* parse command line for interrupt & cp # */
    for (i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
                case 'b' :    /* Set Board number */
                    i++;
                    if ( (argv[i] == NULL) || (sscanf(argv[i], "%hi", &cp) != 1) || (cp < 1) || (cp > 8) )
                    {
                        usage( argv[0] );
                    }
                    break;

                case 'd' :      /* Set debug mode on */
                    bDebug = TRUE;
                    break;

                default:
                    usage( argv[0] );
                    break;
            }
		}
		else
		{
            usage( argv[0] );
		}
    }

    /* open/init the mgmt API */
    status = M3uaMgmtInit( (U8)cp, ENTIT, 0 );
    if (status != M3UA_SUCCESS)
	{
        printf( "M3uaMgmtInit failed [%d]\n", status );

        exit( 1 );
	}

	/* command processing loop */
    printf( "m3uamgr[%-1u]>", cp );
    while ( (forever) && (fgets( buffer, 132, stdin ) != 0) )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
        {
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
        }
		cnt = tokenize( buffer, tokens );
        if (cnt > 0)
        {
			processCmd( tokens, cnt );
        }
        printf( "m3uamgr[%-1u]>", cp );
    }

    status = M3uaMgmtTerm( (U8)cp );

    return( 0 );
}

