/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1996 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/*
 * tcapcfg.c
 *
 * Scans ascii text configuration file and downloads configuration to
 * TCAP task on specified board.
 */

/* header include files -- defines (.h) */
#ifdef UNIX
#include <sys/types.h>
#ifndef LINUX
#include <stropts.h>
#endif
#include <poll.h>
/* #include <sys/stream.h> */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#endif                                  /* UNIX */

#ifdef WIN32
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* WIN32 */

#ifdef __OS2__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* __OS2__ */

#include "ss7type.h"
#include "cfgscan.h"
#include "tcapmapi.h"
#include "tcapver.h"
#include <txnuser.h>

#define TCAPCFG_ENT     TCAP_ENT    /* use same entity id as TCAP task */
#define TCAPCFG_INST    0           /* 0 == PC processor */

short tcapGenCfg( U8 board );
short tcapSapCfg( U8 board );

/* global variables */
INT32           temp;           /* temp used by scanner macros */
char            buf[80];

/* limits from general configuration section */
U16     MaxUsers;

char configFile[80] = "TCAP.cfg";
static char *usage = "Usage: tcapcfg [ -b <board number> ] [-f <filename>]\n";

/*
 * Handy macros to get a boolean value.  If formatted OK, 'dest' is set to
 * the retrieved value.  "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_BOOL( name, dest, cast )                  \
{                                                           \
    if (getcfgval(name, CFGT_BOOL, &temp, 0L) > 0) {        \
            dest = (cast) temp;                             \
    }                                                       \
}

#define GET_CHECK_BOOL_FROM( name, dest, cast, from, next )     \
{                                                               \
    if( (next = getcfgval(name, CFGT_BOOL, &temp, from)) > 0) { \
            dest = (cast) temp;                                 \
    }                                                           \
}

/* Handy macro to get & range check a long, print a warning if its out
 * of range, and copy the value (possibly cast to a smaller type)
 * to "dest" if its in range; assumes "dest" is previously set to
 * default value. "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_INT( name, min, max, dest, cast )         \
{                                                           \
    if (getcfgval(name, CFGT_INT, &temp, 0) > 0) {          \
        if ( temp < min || temp > max ) {                   \
        fprintf( stderr,                                    \
                "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest );                         \
        } else {                                            \
            dest = (cast) temp;                             \
        }                                                   \
    }                                                       \
}

#define GET_CHECK_INT_FROM( name, min, max, dest, cast, from, next )    \
{                                                                       \
    if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0) {          \
        if ( temp < min || temp > max ) {                               \
        fprintf( stderr,                                                \
                "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest );                                     \
        } else {                                                        \
            dest = (cast) temp;                                         \
        }                                                               \
    }                                                                   \
}

/* Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER( name, timer )                            \
{                                                           \
  if (getcfgval(name, CFGT_INT, &temp, 0) > 0)              \
    timer.value = (U16) temp;                               \
  timer.enable = (timer.value != 0) ? 1 : 0;                \
}

#define GET_TIMER_FROM( name, timer, from, next )               \
{                                                               \
  if( (next = getcfgval(name, CFGT_INT, &temp, from)) > 0)      \
    timer.value = (U16) temp;                                   \
  timer.enable = (timer.value != 0) ? 1 : 0;                    \
}

/*
* tcapGenCfg
*
* Configure general parameters for tcap
*/
short tcapGenCfg( U8 board )
{
    TcapGenCfg   cfg;
    short        rval;
    U16          memThresh1;
    U16          memThresh2;
    U16          memThresh3;

    /* Initialize configuration block to defaults */
    TCAPInitGenCfg( &cfg );

    /* scan for overrides from config file */
    GET_CHECK_INT ( "TCAP_ALARM_LEVEL",   0, 3,       cfg.alarmLvl,   U8 );
    GET_CHECK_INT ( "MAX_TCAP_USERS",     1, 512,     cfg.maxSaps,    S16 );
    GET_CHECK_INT ( "MAX_TCAP_DIALOGS",   1, 32767,   cfg.maxDlgs,    S16 );
    GET_CHECK_INT ( "MAX_TCAP_INVOKES",   1, 32767,   cfg.maxInvs,    S16 );
    GET_CHECK_INT ( "MIN_TID_LEN",        1, 4,       cfg.minTidLen,  U8 );
    GET_CHECK_BOOL( "TCAP_TRACE_DATA",                cfg.traceData,  U8 );

    /* allow configuration of memory congestion onset thresholds; note that
     * TCMEM_THRESH_1 must be > TCMEM_THRESH_2 and TCMEM_THRESH_2 > TCMEM_THRESH_3
     */
    memThresh1 = cfg.memThresh1;
    memThresh2 = cfg.memThresh2;
    memThresh3 = cfg.memThresh3;
    GET_CHECK_INT( "TCMEM_THRESH_1", 1, 99, memThresh1, U16 );
    GET_CHECK_INT( "TCMEM_THRESH_2", 1, 99, memThresh2, U16 );
    GET_CHECK_INT( "TCMEM_THRESH_3", 1, 99, memThresh3, U16 );

    if( memThresh2 >= memThresh1 )
    {
        printf( "WARNING - TCMEM_THRESH_1 (%d) must be > TCMEM_THRESH_2 (%d)\n",
                memThresh1, memThresh2 );
        printf( "Using default memory thresholds: %d, %d, %d\n",
                cfg.memThresh1, cfg.memThresh2, cfg.memThresh3 );
    }
    else if( memThresh3 >= memThresh2 )
    {
        printf( "WARNING - TCMEM_THRESH_2 (%d) must be > TCMEM_THRESH_3 (%d)\n",
                memThresh2, memThresh3 );
        printf( "Using default memory thresholds: %d, %d, %d\n",
                cfg.memThresh1, cfg.memThresh2, cfg.memThresh3 );
    }
    else
    {
        /* memory thresholds are OK */
        cfg.memThresh1 = memThresh1;
        cfg.memThresh2 = memThresh2;
        cfg.memThresh3 = memThresh3;
    }

    /* save MaxTcapUsers for range-checking by SAP config section */
    MaxUsers  = cfg.maxSaps;

    /* Call the Config Request */
    rval = TCAPGenCfg( board, &cfg );
    switch( rval )
    {
        case TCAPM_SUCCESS:     /* success */
            break;

        case TCAPM_BOARD:
            printf( "TCAP general configuration failed: Invalid board [%d]\n", board );
            break;

        case TCAPM_TIMEOUT:
            printf( "TCAP general configuration failed: No response from board [%d]\n", board );
            break;

        default:
            printf( "TCAP General Configuration Faild: Error %d\n", rval );
            break;
    }

   return( rval );

} /* tcapGenCfg */

/*
* tcapSapCfg
*
* Configure tcap user saps
*/
short tcapSapCfg( U8 board )
{
    TcapSapCfg  cfg;
    short       rval;
    INT32       last, next;
    U16         sapno;
    short       swType;
    S16         qThresh1;
    S16         qThresh2;
    S16         qThresh3;

   /* loop through all user sap definitions */
    for( last = 0; ; )
    {
        if ((next = getcfgval("USER_SAP", CFGT_INT, &temp, last)) <= 0)
            break;                          /* no more user SAPs */

        last = next;                        /* save starting line in config file */
        sapno = (U16) temp;
        if (sapno >= MaxUsers) {
            printf( "TCAP User SAP number %d exceeds maximum [%d]\n", sapno, MaxUsers );
            continue;
        }

        /* check for protocol variant override for this SAP */
        swType = TCAP_SW_ANS92;         /* default to ANSI 1992 */
        if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0 )
        {
            if (strcmp(buf, "ITU92") == 0)
                swType = TCAP_SW_ITU92;
            else if (strcmp(buf, "ITU88") == 0)
                swType = TCAP_SW_ITU88;
            else if (strcmp(buf, "ITU97") == 0)
                swType = TCAP_SW_ITU97;
            else if (strcmp(buf, "ANSI88") == 0)
                swType = TCAP_SW_ANS88;
            else if (strcmp(buf, "ANSI92") == 0)
                swType = TCAP_SW_ANS92;
            else if (strcmp(buf, "ANSI96") == 0)
                swType = TCAP_SW_ANS96;
            else
                printf( "TCAP User Sap %d: Unknown switch type [%s], defaulting to ANSI-92\n",
                        sapno, buf );
        }

        /* initialize user SAP defaults */
        TCAPInitSapCfg( board, sapno, swType, &cfg );

        /* fetch other overides from config file */
        GET_TIMER_FROM( "TCAP_T1", cfg.t1, last, next );
        GET_TIMER_FROM( "TCAP_T2", cfg.t2, last, next );
        GET_TIMER_FROM( "INACTIVITY_TIMER", cfg.inactvTimer, last, next );
        GET_CHECK_INT_FROM( "TCAP_SEQ_TIMER", 1, 255, cfg.seqTimer, U8, last, next );
        GET_CHECK_INT_FROM( "SCCP_SAP", 0, 32766, cfg.spIdSP, S16, last, next );
        GET_CHECK_INT_FROM( "ALT_PARAM_LEN", 0, 1, cfg.altParamLen, U8, last, next );
        GET_CHECK_INT_FROM( "SCCP_ADDR_OVERRIDE", 0, 1, cfg.addrOverride, U8, last, next );
        GET_CHECK_INT_FROM( "ALLOW_INVOKE_END", 0, 1, cfg.allowInvkEnd, U8, last, next );

        /* check for checkpoint override for this SAP */
        cfg.chkpt = CHKPT_NONE;         /* default to no checkpointing */
        if (getcfgval("DEFAULT_CHECKPOINT", CFGT_STRING, buf, last) > 0 )
        {
            if (strcmp(buf, "CHKPT_ALL") == 0)
                cfg.chkpt = CHKPT_ALL;
            else if (strcmp(buf, "CHKPT_SEND") == 0)
                cfg.chkpt = CHKPT_SEND;
            else if (strcmp(buf, "CHKPT_NONE") == 0)
                cfg.chkpt = CHKPT_NONE;
            else
                printf( "TCAP User Sap %d: Unknown default checkpoint [%s], defaulting to CHKPT_NONE\n",
                        sapno, buf );
        }

        /* allow configuration of user queue congestion onset thresholds; note that
         * TCQ_THRESH_2 must be > TCQ_THRESH_1 and TCQ_THRESH_3 > TCQ_THRESH_2
         */
        qThresh1 = cfg.qThresh1;
        qThresh2 = cfg.qThresh2;
        qThresh3 = cfg.qThresh3;
        GET_CHECK_INT_FROM( "TCQ_THRESH_1", 1, 32766, qThresh1, S16, last, next );
        GET_CHECK_INT_FROM( "TCQ_THRESH_2", 1, 32766, qThresh2, S16, last, next );
        GET_CHECK_INT_FROM( "TCQ_THRESH_3", 1, 32766, qThresh3, S16, last, next );

        if( qThresh2 <= qThresh1 )
        {
            printf( "WARNING - TCAP User Sap %d: TCQ_THRESH_2 (%d) must be > TCQ_THRESH_1 (%d)\n",
                    sapno, qThresh2, qThresh1 );
            printf( "Using default queue thresholds: %d, %d, %d\n", cfg.qThresh1, cfg.qThresh2, cfg.qThresh3 );
        }
        else if( qThresh3 <= qThresh2 )
        {
            printf( "WARNING - TCAP User Sap %d: TCQ_THRESH_3 (%d) must be > TCQ_THRESH_2 (%d)\n",
                    sapno, qThresh3, qThresh2 );
            printf( "Using default queue thresholds: %d, %d, %d\n", cfg.qThresh1, cfg.qThresh2, cfg.qThresh3 );
        }
        else
        {
            /* queue thresholds are OK */
            cfg.qThresh1 = qThresh1;
            cfg.qThresh2 = qThresh2;
            cfg.qThresh3 = qThresh3;
        }

        /* Call the Config Request */
        rval = TCAPSapCfg( board, &cfg, sapno );
        if( rval != TCAPM_SUCCESS )
        {
            switch( rval )
            {
                case TCAPM_BOARD:
                    printf( "TCAP User Sap %d configuration failed: Invalid board [%d]\n",
                            sapno, board );
                    break;

                case TCAPM_TIMEOUT:
                    printf( "TCAP User Sap %d configuration failed: No response from board [%d]\n",
                            sapno, board );
                    break;

                default:
                    printf( "TCAP User Sap %d Configuration Faild: Error %d\n", sapno, rval );
                    break;
            }
            return( rval );
        }

        /* skip up to end of section */
        next = getcfgval("END", CFGT_INT, &temp, last);
        last = next;
    }   /* end for loop */

    return( TCAPM_SUCCESS );
} /* tcapSapCfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main( int argc, char *argv[] )
{

    unsigned short  board = 1;                  /* default board is 1 */
    int             i;
    char            *p;
    int              rval;

    printf("\n%s: TCAP configuration application version %s %s\n",
            argv[0], TCAPCFG_VERSION, __DATE__);

    /* parse command line for config file name & board # */
	for(i = 1; i < argc; i++) {
		p = argv[i];
		if (*p == '-') {
			switch (*++p) {
            case 'f' :     /* configuration file name */
				i++;
                strncpy( configFile, argv[i], sizeof( configFile ) - 1 );
                configFile[sizeof( configFile ) - 1] = '\0';  /* ensure null term */
				break;

			case 'b' :    /* Set Board number */
				i++;
                if( (sscanf(argv[i], "%hu", &board) != 1) || (board < 1) || (board > 8) )
                {
                    printf( usage );
                    exit( 1 );
                }
				break;

            case 'd' :
                CDebug = 1;             /* config scanner debugging */
                break;

			default:
                printf( usage );
                exit( 1 );
                break;
			}	/* end switch */
		} else {
            printf( usage );
            exit( 1 );
		}

	}	/* end for */

    /* Initialize the TCAP management API */
    rval = TCAPInitMgmtAPI( (U8) board, TCAPCFG_ENT, TCAPCFG_INST );
    if( rval != TCAPM_SUCCESS )
    {
        printf("%s : Error %d initializing TCAP Mgmt API\n", argv[0], rval);
        exit( rval );
    }

    printf("--- Reading file %s\n", configFile);

    /* initialize config file scanner library */
    if( (rval = initcfg( configFile )) == 0 ) {
        /* config file opened successfully - do the tcap configuration */
        rval = tcapGenCfg( (U8) board );         /* General Configuration */
        if( rval == TCAPM_SUCCESS ) {
            rval = tcapSapCfg( (U8) board );     /* User SAPs */
        }
        endcfg();                           /* close config scanner */
    }

    TCAPTermMgmtAPI( (U8) board );
    exit( rval );
}
