/* UPDthrd.c
 *
 * Sockets communication handler
 *
 * This module provides the communication interface to the mate node.
 *
 * This module is composed of a receiver thread, whose job is to continuously read
 * from the socket and post any received messages to the controller FSM module, and
 * utility routines for initializing the interface, sending packets, and closing
 * down the interface.
 *
 * Connectionless datagram (UDP) sockets are employed.  All socket I/O is blocking
 * (thus the use of a separate thread for receiving).  The cleanup routine closes
 * down the scoket, which also has the effect of killing the receiver thread.
 *
 * The sending module is responsible for timing for responses and issuing any desired
 * retransmissions.
 */

#ifdef WIN32
#include <windows.h>
#include <winsock.h>
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"
#include "nmstypes.h"
#include "ss7type.h"
#include "tcap800.h"
#include "osdep.h"


/* Constants and defaults */
#define REQUIRED_WINSOCK_VERSION    0x0101  /* requires at least winsock version 1.1 */

/* Globals */
static TSockAddrIn   remSock;    /* IP address/port for mate application */
static TSockAddrIn   locSock;    /* this application's socket */
static TSocket       Socket;     /* socket to receive on */

extern U16      locPort;
extern U16      remPort;
extern U32      remAddr;
extern CTAHD    FstCtaHd;
extern U32      trans_id;
extern U32      position;
extern U8       invoke_id;
extern U8       readFlag;
extern FILE    *name;

DWORD End800 (void);

/*
 * UDPThread
 *
 * This is the entry point to the receiver thread; its job is to loop continuously
 * doing blocking reads on the socket and posting any received packets to the SCTL
 * module in the form of "data indication" events, until a socket error is received
 * (indicating that the socket has been shut down and its time to stop).
 *
 * Parameters:
 *
 *  pParam      pointer to socket handle to use
 *
 * Returns:     Always 0
 *
 */
void UDPThread( void *dummy )
{
#ifdef WIN32
    WSADATA         wsadata;        /* sockets version data */
    int             wsaret;         /* sockets return code */

    UNREFERENCED_PARAMETER(dummy);
    wsaret = WSAStartup(REQUIRED_WINSOCK_VERSION, &wsadata);
    if (wsaret != 0)
    {
        fprintf(stderr,  "UDPThread terminating, unable to initialize sockets API, error %d\n ", wsaret );
	THREAD_EXIT();
    }
    else if ((LOBYTE(wsadata.wVersion) != 1) || HIBYTE(wsadata.wVersion) != 1)
    {
        /* incompatible SOCKETS API version */
        fprintf(stderr, "UDPThread terminating, unsupported sockets API version (%d.%d)\n",
                 HIBYTE(wsadata.wVersion), LOBYTE(wsadata.wVersion));
	THREAD_EXIT();
    }
#endif

    /* next create a UDP datagram socket */
    if((SOCKET_OPEN( AF_INET, SOCK_DGRAM, 0, Socket )) == RET_FAILED )
      {
        /* fatal error */
        fprintf(stderr, "UDPThread terminating, unable to create socket, error %d", osGetLastError());
	THREAD_EXIT();
      }

    locSock.sin_port = locPort;
    locSock.sin_family = AF_INET;
    locSock.sin_addr.s_addr = htonl(INADDR_ANY);

    remSock.sin_port = remPort;
    remSock.sin_family = AF_INET;
    remSock.sin_addr.s_addr = remAddr;

    /* Bind the socket to our configured port */
    if(SOCKET_BIND(Socket, (TSockAddr*)&locSock, sizeof(locSock))
      != RET_SUCCESS)
      {
        fprintf(stderr, "UDPThread terminating, unable to bind socket, error %d", osGetLastError());
	THREAD_EXIT();
      }

    for (;;)
    {
        CTA_EVENT           ctaEvent;
        TSockAddrIn         fromAddr;       /* IP address/port for mate application */
        ChkPntMsg          *evt;
        int                 fromLen;        /* length of socket address structure */
        int                 sret;           /* sockets return value */
        int                 recvLen;        /* length of received message */

        if ((evt = (ChkPntMsg *)calloc(1, sizeof(ChkPntMsg))) == NULL)
        {
            fprintf(stderr, "UDPThread terminating, memory allocation failure\n");
            THREAD_EXIT();
        }

        fromLen = sizeof(TSockAddrIn);

        if ((SOCKET_RECVFROM(Socket, (char*) evt, sizeof(ChkPntMsg), 0,
                             (TSockAddr*) &fromAddr, &fromLen, recvLen)) != RET_SUCCESS)
        {
            sret = osGetLastError();
            if( sret == ERR_MSGSIZE )
            {
                /* message too big, probably from some other application due to port
                 * number conflict - display warning so users can resolve conflict
                 */
                fprintf(stderr,  "UDPThread: Message from %s too long [len=%d], discarded", inet_ntoa(fromAddr.sin_addr), recvLen );
            }
#ifdef UNIX
            else if (sret == ECONNREFUSED)
            {
                /* ignore this error, which occurs before other TCAPdemo starts */
            }
#else
            else  if ((sret != WSAECONNREFUSED) &&
                      (sret != WSAECONNRESET) )
            {
                fprintf(stderr, "UDPThread terminating, socket receive failed %ld\n", osGetLastError());
                THREAD_EXIT();
            }
#endif
        }
        else if (evt->msgId == CHKPNT_MSG)
        {
            trans_id  = evt->transId;
            readFlag  = evt->readFlag;
            invoke_id = evt->invokeId;
            position  = evt->position;

            fseek(name, evt->position, SEEK_SET);

            memset(&ctaEvent, 0, sizeof(CTA_EVENT));

            ctaEvent.id     = CHKPNT_EVT;
            ctaEvent.ctahd  = FstCtaHd;
            ctaEvent.size   = 0;
            ctaEvent.buffer = NULL;
            ctaQueueEvent(&ctaEvent);
            free(evt);
        }
        else
            fprintf(stderr, "UDPThread: Unknown message type 0x%x received\n", evt->msgId);
    }
}                               /* of RmgCommRecvThread */

void CheckPoint(Trans *trans)
{
    ChkPntMsg   chkPntMsg;         /* buffer for outgoing message */

    /* construct message */

    chkPntMsg.msgId     = CHKPNT_MSG;
    chkPntMsg.position  = trans->position;
    chkPntMsg.transId   = trans->transId;
    chkPntMsg.invokeId  = trans->invokeId;
    chkPntMsg.readFlag  = trans->readFlag;

    if (SOCKET_SENDTO(Socket, (char *) &chkPntMsg, sizeof(ChkPntMsg), 0,
                      (TSockAddr*) &remSock, sizeof(TSockAddrIn)) != RET_SUCCESS)
    {
      fprintf(stderr,  "UDP: Unable to send checkpoint message, error %ld\n", osGetLastError());
    }
}
