
/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**  Copyright (C) 1997 - 2012 Dialogic Inc. All Rights Reserved.        **
**                                                                      **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Dialogic Corporation                          **
**                                                                      **
*************************************************************************/

/* TCAP management sample application
 *
 * This program is designed as a sample application to illustrate the use
 * of the TCAP management API primitives.
 *
 * To Run:
 *  TCAPMGR [-b <board>]
 *
 * where:
 *  <board>     is the target Tx Series board number (default: 1).
 *
 * This application supports the following commands:
 *
 *  ALARMLVL <0..3>         sets the current alarm output level
 *  TRACE ON | OFF          turns buffer tracing on or off
 *  STATS <sapno> [RESET]   retrieves statistics on an application sap and
 *                          optionally resets them to zero after fetch
 *  CONFIG {SAP <sapno>}    displays SAP configuration parameters or
 *         {GEN}            general configuration parameters
 *  BOARD <board>           switches to a new target board
 *  ? [COMMAND]             lists available command or parameters of a
 *                          specific command
 *  Q                       quits application
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "ss7type.h"
#include "tcapmapi.h"
#include "tcapver.h"
#include "osdep.h"


/* constants */
#define TCAPMGR_ENT     0x05    /* Entity ID for this application */
#define TCAPMGR_INST    0       /* Instance ID == processor, 0==Host PC */

#define USAGE   "\n usage: TCAPMGR [-b <board>] \n"

/* function forward declarations */
void alarmCtrl( short *parms );
void traceCtrl( short *parms );
void statsReq ( short *parms );
void statusReq( short *parms );
void configReq( short *parms );
void helpReq  ( short *parms );
void setCP    ( short *parms );
void quit     ( short *parms );

static void getGenCfg( void );
static void getSapCfg( S16 sap );

/* lookup tables for translating names into values and vice versa */
struct lookupEnt {
	char 	*name;
	short	value;
};

/* parameter name tables */
struct lookupEnt TraceCtrlLookupTbl[] = {   /* TCAP Trace control action values  */
    { "ON",  TCAP_BUFTRACE_ON },
    { "OFF", TCAP_BUFTRACE_OFF },
	{ 0,	0 }/* end of table */
};

struct lookupEnt ResetLookupTbl[] = {       /* TCAP Stats reset action values  */
    { "RESET", 1 },
	{ 0,	0 }/* end of table */
};

struct lookupEnt ConfigLookupTbl[] = {      /* TCAP config display selector values  */
    { "GEN", 0 },                           /* request general config display */
    { "SAP", 1 },                           /* request SAP config display */
	{ 0,	0 }/* end of table */
};

struct lookupEnt SwtypeLookupTbl[] = {      /* TCAP switch type name-numbers  */
    { "ITU92", TCAP_SW_ITU92 },
    { "ITU88", TCAP_SW_ITU88 },
    { "ITU97", TCAP_SW_ITU97 },
    { "ANSI88", TCAP_SW_ANS88 },
    { "ANSI92", TCAP_SW_ANS92 },
    { "ANSI96", TCAP_SW_ANS96 },
	{ 0,	0 }/* end of table */
};

struct lookupEnt CkptLookupTbl[] = {        /* TCAP checkpointing options  */
    { "ALL", CHKPT_ALL },
    { "SEND", CHKPT_SEND },
    { "NONE", CHKPT_NONE },
	{ 0,	0 }/* end of table */
};

struct lookupEnt CmdLookupTbl[] =
{
        { "ALARMLVL",   1 },
        { "TRACE",      2 },
        { "STATS",      3 },
        { "STATUS",     4 },
        { "BOARD",      5 },
        { "Q",          6 },
        { "CONFIG",     7 },
        { "ALL",        0 }
};

static char alarmsCtrlHelp[]=   "    Syntax: ALARMLVL <0..3>\n\n";

static char traceCtrlHelp[] =   "    Syntax: TRACE [ON | OFF]\n\n";

static char statsHelp[]     =   "    Syntax: STATS <user SAP number> [RESET]\n\n";

static char statusHelp[]    =   "    Syntax: STATUS \n\n";

static char quitHelp[]      =   "    Syntax: Q \n\n";

static char boardHelp[]     =   "    Syntax: BOARD <new target board number>\n\n";

static char configHelp[]    =   "    Syntax: CONFIG { GEN | SAP <user SAP number>}\n\n";

static char allHelp[]       =
"    ALARMLVL   Sets Alarm Level\n"
"    TRACE      Enables/Disables Buffer Tracing\n"
"    STATS      Application Statistics\n"
"    STATUS     General     Statistics\n"
"    CONFIG     Displays General or SAP Configuration Parameters\n"
"    BOARD      Changes target board\n"
"    Q          Quit\n\n";

static char* helpTbl[] =
{
	    allHelp,
	    alarmsCtrlHelp,
        traceCtrlHelp,
	    statsHelp,
        statusHelp,
		boardHelp,
        quitHelp,
        configHelp
};


/* structure for command parse tables */
#define MAX_PARMS	10

struct CmdSpec {
	char				*command;
	struct lookupEnt	*parmTbl[MAX_PARMS];
	void				(*func)(short *);
};

/* Help Command */
struct CmdSpec  helpCmd =
{
    "?",                                            /* command name */
    { CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },    /* parameter lookups */
    helpReq                                     /* handler function */
};

/* tcapmgmt API commands */
struct CmdSpec	alarmCtrlCmd =
{
    "ALARMLVL",                                     /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },               /* parameter lookups */
    alarmCtrl                                       /* handler function */
};

struct CmdSpec  traceCtrlCmd =
{
    "TRACE",                                            /* command name */
    { TraceCtrlLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    traceCtrl                                           /* handler function */
};

struct CmdSpec	statsCmd =
{
    "STATS",                                        /* command name */
    { 0, ResetLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    statsReq                                        /* handler function */
};

struct CmdSpec  statusCmd =
{
    "STATUS",                                       /* command name */
    {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },              /* parameter lookups */
    statusReq                                       /* handler function */
};

struct CmdSpec  configCmd =
{
    "CONFIG",                                       /* command name */
    { ConfigLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* parameter lookups */
    configReq                                       /* handler function */
};

struct CmdSpec  boardCmd =
{
	"BOARD",			   							/* command name */
	{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
	setCP											/* handler function */
};

struct CmdSpec  quitCmd =
{
	"Q",			   								/* command name */
    {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },              /* parameter lookups */
    quit                                            /* handler function */
};

struct CmdSpec *CmdTable[] = {
		&alarmCtrlCmd,
        &traceCtrlCmd,
		&statsCmd,
        &statusCmd,
        &configCmd,
		&boardCmd,
		&quitCmd,
	    &helpCmd,
	    0
};

/* Misc. globals */
char	valStr[20];
short   cp = 1;                 /* default board number */
short	quittinTime = 0;


/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char *getname( struct lookupEnt *tbl, short val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( tbl->value == val )
		{
			/* match found */
			return( tbl->name );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed - convert number to ascii */
	sprintf( valStr, "%d", val );
	return( valStr );

}	/* end getname() */


/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval( struct lookupEnt *tbl, char *s, short *val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( strcmp( tbl->name, s ) == 0 )
		{
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed */
	return( 0 );
}	/* end getval() */


/*
 * ALARMLVL Ctrl command handler; command syntax:
 *      ALARMLVL <0..3>
 *
 * First parameter is alarm level entered; must be in range 0..3.
 */
void alarmCtrl( short *parms )
{
    S16    status;
    U8     alarmlvl;

    alarmlvl = (U8) parms[0];
    if( alarmlvl > 3 )
    {
        printf( "Alarm level must be in range 0..3\n" );
        return;
    }

    status = TCAPAlarmControl( (U8) cp, alarmlvl );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            default:
                printf( "TCAP Alarm Ctrl request failed [%d]\n", status );
                break;
        }
	}
}


/*
 * TRACE control command handler; command syntax:
 *      TRACE   ON | OFF
 *
 * First parameter is the on/off switch
 */
void traceCtrl( short *parms )
{
    S16    status;
    U32    action;

    action = (U32) parms[0];
    status = TCAPTraceControl( (U8) cp, action );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            default:
                printf( "TCAP Trace Ctrl request failed [%d]\n", status );
                break;
        }
		return;
	}
}


/*
 * STATS command handler
 *
 * First parameter is User SAP number; 2nd parameter is optional RESET
 * flag, if non-zero resets all stats to zero after fetch
 */
void statsReq( short *parms )
{
    S16             status;
    S16             sap;
    TcapSapStats    stats;
    U8              reset;

    sap     = (S16) parms[0];
    reset   = (U8) parms[1];

    status = TCAPSapStats( (U8) cp, sap, &stats, reset );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            case TCAPM_BADSAP :
                printf( "Board %d Sap %d undefined\n", cp, sap );
                break;

            default:
                printf( "Sap %d Stats request failed [%d]\n", sap, status );
                break;
        }
		return;
	}

    /* stats request successful, display stats */
    printf( "Board %d, Sap %d Stats:\n", cp, sap );
    printf( "                   Tx        Rx\n" );
    printf( "                   --        --\n" );
    printf( "Total Frames       %-8u  %-8u\n", stats.frmTx, stats.frmRx );
    printf( "Frames Dropped     ---       %-8u\n", stats.drop );
    printf( "Open transactions  %-8u\n", stats.openTrans );
    printf( "\n" );
    printf( "Invoke Components  %-8u  %-8u\n", stats.invTx, stats.invRx );
    printf( "Result Components  %-8u  %-8u\n", stats.resTx, stats.resRx );
    printf( "Reject Components  %-8u  %-8u\n", stats.rejTx, stats.rejRx );
    printf( "Error  Components  %-8u  %-8u\n", stats.errTx, stats.errRx );
    printf( "\n" );

    switch( stats.swtch )       /* protocol variant specific stats */
	{
        case TCAP_SW_ITU88:
        case TCAP_SW_ITU92:
        case TCAP_SW_ITU97:
            printf( "Unidir. Msgs       %-8u  %-8u\n", stats.uniTx, stats.uniRx );
            printf( "Begin Msgs         %-8u  %-8u\n", stats.beginTx, stats.beginRx );
            printf( "Continue Msgs      %-8u  %-8u\n", stats.contTx, stats.contRx );
            printf( "End Msgs           %-8u  %-8u\n", stats.endTx, stats.endRx );
            printf( "Abort Msgs         %-8u  %-8u\n", stats.abortTx, stats.abortRx );
            break;

        default:                /* ANSI */
            printf( "Unidir. Msgs       %-8u  %-8u\n", stats.uniTx, stats.uniRx );
            printf( "Query w/Perm.      %-8u  %-8u\n", stats.qryPrmTx, stats.qryPrmRx );
            printf( "Query w/o Perm.    %-8u  %-8u\n", stats.qryNoPrmTx, stats.qryNoPrmRx );
            printf( "Conv. w/Perm.      %-8u  %-8u\n", stats.conPrmTx, stats.conPrmRx );
            printf( "Conv. w/o Perm.    %-8u  %-8u\n", stats.conNoPrmTx, stats.conNoPrmRx );
            printf( "Response Msgs      %-8u  %-8u\n", stats.respTx, stats.respRx );
            printf( "Abort Msgs         %-8u  %-8u\n", stats.abortTx, stats.abortRx );
            break;
    }

    /* display congestion control stats and current state */
    printf( "Congestion Abort   %-8u  %-8u\n", stats.outCongAbort, stats.inbCongAbort );
    printf( "Congestion Discard %-8u  %-8u\n", stats.outCongDisc, stats.inbCongDisc );
    printf( "Current Cong Level %-8u  %-8u\n", stats.currOutbCongLvl, stats.currInbCongLvl );

    printf( "\n" );

}   /* statsReq */


/*
 * STATUS command handler
 *
 * First parameter is User SAP number; 2nd parameter is optional RESET
 * flag, if non-zero resets all stats to zero after fetch
 */
void statusReq( short *parms )
{
    S16           status;
    TcapGenStatus stats;

    UNREFERENCED_PARAMETER(parms);

    status = TCAPGenStatus( (U8) cp, &stats );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            case TCAPM_NULLPTR :
                printf( "Board %d Status request: bad parameter\n", cp );
                break;

            default:
                printf( "Status request failed [%d]\n", status );
                break;
        }
		return;
	}
    /* stats request successful, display stats */
    printf( "Board %d, Status:\n", cp );

    switch( stats.haSt )       /* high availability state */
    {
        case ST_HAST_STARTING:
                printf( "High availability state is STARTING\n" );
                break;
        case ST_HAST_STANDALONE:
                printf( "High availability state is STANDALONE\n" );
                break;
        case ST_HAST_PRIMARY:
                printf( "High availability state is PRIMARY\n" );
                break;
        case ST_HAST_BACKUP:
                printf( "High availability state is BACKUP\n" );
                break;
        default:
                printf( "ERROR: High availability state\n" );
                break;
    }

    switch( stats.mcSt )       /* mate connection state */
    {
        case ST_MCST_ISOLATED:
                printf( "Mate connection state is   ISOLATED\n" );
                break;
        case ST_MCST_CONNECTED:
                printf( "Mate connection state is   CONNECTED\n" );
                break;
        default:
                printf( "ERROR: Mate connection state\n" );
                break;
    }

}   /* statusReq */


/* Board command handler
 *
 * Changes the current target board (if new board number in first parameter
 * is valid) and closes out access to the old target board.
 */
void setCP( short *parms )
{
    short   newcp;
    S16     status;

    newcp = parms[0];

	if( newcp < 1 || newcp > 8 )
	{
        printf( "Board number must be in range 1..8\n" );
        return;
	}

    if( newcp == cp )
	{
        /* new target same as old target - nothing to do */
        return;
	}

	/* open/init the MTP3 mgmt API for new board*/
    status = TCAPInitMgmtAPI( (U8) newcp, TCAPMGR_ENT, TCAPMGR_INST );
    if( status != TCAPM_SUCCESS )
	{
        printf( "Board %d TCAPInitMgmtAPI failed [%d]\n", newcp, status );
	}
	else
	{
		/* successful, close old cp & set new cp as the current cp */
        status = TCAPTermMgmtAPI( (U8) cp );
		cp = newcp;
	}
}

/* Help command handler
 *
 * Displays help string based on command number in 1st parameter
 */
void helpReq( short *parms )
{
    UNREFERENCED_PARAMETER(parms);

	printf( "%s", helpTbl[*parms] );
}

/* Quit command handler
 *
 * Sets a flag to indicate to main loop to exit
 */
void quit( short *parms )
{
    UNREFERENCED_PARAMETER(parms);

	quittinTime = 1;
}


/*
 * CONFIG command handler
 *
 * First parameter is either GEN (value 0) or SAP (value 1).  If SAP,
 * then second parameter is sap number to display.
 */
void configReq( short *parms )
{
    S16             sap;
    short           selector;

    selector= (S16) parms[0];
    sap     = (S16) parms[1];

    /* determine type of request */
    switch( selector )
    {
        case 0 :
            getGenCfg();
            break;

        case 1 :
            getSapCfg( sap );
            break;

        default:
            printf( configHelp );
            break;
    }

}   /* configReq */

/*
 * GetGenCfg
 *
 * Retrieves and displays general configuration parameters.
 */
static void getGenCfg( void )
{
    TcapGenCfg      gencfg;
    S16             status;

    status = TCAPGetGenCfg( (U8) cp, &gencfg );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            default:
                printf( "Get GEN CFG request failed [%d]\n", status );
                break;
        }
		return;
	}

    /* get request successful, display gen config paramters */
    printf( "Board %d General Configuration Parameters:\n\n", cp );
    printf( "MAX_TCAP_USERS:    %-8u\n", gencfg.maxSaps );
    printf( "MAX_TCAP_DIALOGS:  %-8u\n", gencfg.maxDlgs );
    printf( "MAX_TCAP_INVOKES:  %-8u\n", gencfg.maxInvs );
    printf( "TCAP_ALARM_LEVEL:  %-8u\n", gencfg.alarmLvl );
    printf( "MIN_TID_LEN:       %-8u\n", gencfg.minTidLen );
    printf( "TCAP_TRACE_DATA:   %-8s\n", ((gencfg.traceData == 0) ? "Off" : "On") );
    printf( "TCMEM_THRESH_1:    %-8u\n", gencfg.memThresh1 );
    printf( "TCMEM_THRESH_2:    %-8u\n", gencfg.memThresh2 );
    printf( "TCMEM_THRESH_3:    %-8u\n", gencfg.memThresh3 );
    printf( "\n" );

}   /* getGenCfg */

/*
 * getSapCfg
 *
 * Retrieves and displays SAP configuration parameters.
 */
static void getSapCfg( S16 sap )
{
    TcapSapCfg      sapcfg;
    S16             status;

    status = TCAPGetSapCfg( (U8) cp, &sapcfg, sap );
    if( status != TCAPM_SUCCESS )
	{
        switch( status )
        {
            case TCAPM_TIMEOUT:
                printf( "No response from board %d\n", cp );
                break;

            case TCAPM_BADSAP :
                printf( "Board %d Sap %d undefined\n", cp, sap );
                break;

            default:
                printf( "Get Sap %d CFG request failed [%d]\n", sap, status );
                break;
        }
		return;
	}

    /* get request successful, display gen config paramters */
    printf( "Board %d SAP %d Configuration Parameters:\n\n", cp, sap );
    printf( "SWITCH_TYPE:       %-8s\n", getname( SwtypeLookupTbl, sapcfg.swtch ) );
    printf( "TCAP_T1:           %-8u\n", sapcfg.t1.value );
    printf( "TCAP_T2:           %-8u\n", sapcfg.t2.value );
    printf( "INACTIVITY_TIMER:  %-8u\n", sapcfg.inactvTimer.value );
    printf( "TCAP_SEQ_TIMER:    %-8u\n", sapcfg.seqTimer );
    printf( "SCCP_SAP:          %-8u\n", sapcfg.spIdSP );
    printf( "ALT_PARAM_LEN:     %-8s\n", ((sapcfg.altParamLen == 0) ? "Off" : "On") );
    printf( "SCCP_ADDR_OVERRIDE:%-8s\n", ((sapcfg.addrOverride == 0) ? "Off" : "On") );
    printf( "ALLOW_INVOKE_END:  %-8s\n", ((sapcfg.allowInvkEnd == 0) ? "Off" : "On") );
    printf( "DEFAULT_CHECKPOINT:%-8s\n", getname( CkptLookupTbl, sapcfg.chkpt ) );
    printf( "TCQ_THRESH_1:      %-8u\n", sapcfg.qThresh1 );
    printf( "TCQ_THRESH_2:      %-8u\n", sapcfg.qThresh2 );
    printf( "TCQ_THRESH_3:      %-8u\n", sapcfg.qThresh3 );
    printf( "\n" );

}   /* getSapCfg */

/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
	short 				i, j;
	short				found;
	short 				params[MAX_PARMS];	/* numeric parameters array */
	struct lookupEnt	*lp;

	/* first search command table for command match */
	for( i = 0; CmdTable[i] != 0; i++ )
	{
		if( strcmp( tokens[0], CmdTable[i]->command ) == 0 )
		{
			break;		/* command matched */
		}
	}	/* end for */

	/* if no match found display error message and return */
	if( CmdTable[i] == 0 )
	{
		printf("Unrecognized command: %s\n", tokens[0] );
		return;
	}

	/* convert remaining tokens (i.e., parameters) to short integers,
	 * using lookup tables  where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
	for( j = 0; j < MAX_PARMS; j++ )
		params[j] = 0;					/* first initialize to defaults */

	for( j = 1; j < cnt; j++, lp++ )
	{
		lp = CmdTable[i]->parmTbl[j - 1];
		if( lp )
		{	/* lookup table for this param ? */
			found = getval( lp, tokens[j], &params[j - 1] );
			if( found )
			{
				continue;				/* parameter succesfully converted */
			}
		}

		/* no lookup table or lookup failed - try to convert as a number */
		if( sscanf( tokens[j], "%hi", &params[j - 1] ) != 1 )
		{
			/* conversion failed */
			printf("Unrecognized parameter: %s\n", tokens[j] );
			return;
		}
	}	/* end for */

	/* call command handler function */

	if( CmdTable[i]->func != 0 )
	{
		CmdTable[i]->func( params );
	}
}


/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}


/* TcapMgr Main function:
 *
 * Scans command line arguments for options, initializes the TCAP
 * management API library, and executes the command processing loop
 * until an exit command is received.
 */
int main( int argc, char *argv[] )
{

	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	short	cnt;
	short	i;
	char	*p;
    S16     status;

    printf("%s: TCAP management application version %s %s\n",
            argv[0], TCAPMGR_VERSION, __DATE__);

	/* parse command line for interrupt & cp # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
			case 'b' :    /* Set Board number */
				i++;
				if( (sscanf(argv[i], "%hi", &cp) != 1) || (cp < 1) || (cp > 8) )
				{
                    printf( USAGE );
                    exit( 1 );
				}
				break;

			default:
                printf( USAGE );
                exit( 1 );
                break;
			}	/* end switch */
		}
		else
		{
            printf( USAGE );
            exit( 1 );
        }

	}	/* end for */

	/* open/init the MTP3 mgmt API */
    status = TCAPInitMgmtAPI( (U8) cp, TCAPMGR_ENT, TCAPMGR_INST );
    if( status != TCAPM_SUCCESS )
	{
        printf( "TCAPInitMgmtAPI failed [%d]\n", status );
		exit(1);
	}

	/* command processing loop */
    printf( "TcapMgr[%-1u]>", cp );
    while( (quittinTime == 0) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
		cnt = tokenize( buffer, tokens );
		if( cnt > 0 )
			processCmd( tokens, cnt );
        printf( "TcapMgr[%-1u]>", cp );
	}	/* end while */

    status = TCAPTermMgmtAPI( (U8) cp );
}	/* end main() */
